@@ -33,9 +33,7 @@
 
 FrameSource::FrameSource(KConfig *cfg, const QString& filename, const QString& type)
 : KstDataSource(cfg, filename, type) {
-  if (init()) {
-    _valid = true;
-  }
+  _valid = init();
 }
 
 
@@ -43,6 +41,12 @@ FrameSource::~FrameSource() {
 }
 
 
+bool FrameSource::reset() {
+  _fieldList.clear();
+  return _valid = init();
+}
+
+
 bool FrameSource::init() {
   int buf[4], err = 0;
 
@@ -54,8 +58,7 @@ bool FrameSource::init() {
            &err);
 
   if (err != E_OK) {
-    // FIXME: BAD IDEA!!
-    kdFatal() << "Problem reading FFINFO in from Frame Type File" << endl;
+    return false;
   }
 
   _bytesPerFrame = buf[0];
@@ -84,7 +87,10 @@ bool FrameSource::init() {
 
 
 KstObject::UpdateType FrameSource::update(int u) {
-  Q_UNUSED(u)
+  if (KstObject::checkUpdateCounter(u)) {
+    return lastUpdateResult();
+  }
+
   QString tmpfilename;
   struct stat stat_buf;
   int done = 0;
@@ -127,7 +133,7 @@ KstObject::UpdateType FrameSource::update(int u) {
   _frameCount = newN;
 
   updateNumFramesScalar();
-  return isnew ? KstObject::UPDATE : KstObject::NO_CHANGE;
+  return setLastUpdateResult(isnew ? KstObject::UPDATE : KstObject::NO_CHANGE);
 }
 
 
