@@ -18,6 +18,7 @@
 #ifndef ENODES_H
 #define ENODES_H
 
+#include "kststring.h"
 #include "kstvector.h"
 
 class Plugin;
@@ -27,6 +28,14 @@ class Plugin;
 
 namespace Equation {
 
+  /* Global lock for the parser */
+  QMutex& mutex();
+
+  /*    Evaluate the expression @p txt and returns the value as a double.
+   *    Returns the value, or 0.0 and sets ok = false on error.
+   */
+  double interpret(const char *txt, bool *ok = 0L);
+
   class Context {
     public:
       Context() : i(0), x(0.0), xVector(0L), noPoint(0.0), sampleCount(0) {}
@@ -40,17 +49,24 @@ namespace Equation {
   };
 
   class NodeVisitor;
-  
+
   class Node {
     public:
       Node();
       virtual ~Node();
 
       virtual bool isConst() = 0; // can't be const
-      virtual void collectVectors(KstVectorList& c);
+      virtual void collectObjects(KstVectorMap& v, KstScalarMap& s, KstStringMap& t);
+      virtual bool takeVectors(const KstVectorMap& c);
       virtual double value(Context*) = 0;
       virtual void visit(NodeVisitor*);
       virtual KstObject::UpdateType update(int counter, Context *ctx);
+      virtual QString text() const = 0;
+
+      void parenthesize() { _parentheses = true; }
+
+    protected:
+      bool _parentheses;
   };
 
   class BinaryNode : public Node {
@@ -58,13 +74,16 @@ namespace Equation {
       BinaryNode(Node *left, Node *right);
       virtual ~BinaryNode();
 
-      virtual void collectVectors(KstVectorList& c);
+      virtual void collectObjects(KstVectorMap& v, KstScalarMap& s, KstStringMap& t);
+      virtual bool takeVectors(const KstVectorMap& c);
       virtual void visit(NodeVisitor*);
       virtual KstObject::UpdateType update(int counter, Context *ctx);
 
       Node *& left();
       Node *& right(); 
 
+      virtual QString text() const = 0;
+
     protected:
       Node *_left, *_right;
   };
@@ -79,13 +98,15 @@ namespace Equation {
       void appendArgument(Node *arg);
 
       // Makes no sense for this type
-      virtual double value(Context*) { return 0.0; }
+      double value(Context*) { return 0.0; }
 
-      virtual bool isConst();
-      virtual void collectVectors(KstVectorList& c);
+      bool isConst();
+      void collectObjects(KstVectorMap& v, KstScalarMap& s, KstStringMap& t);
+      bool takeVectors(const KstVectorMap& c);
       double at(int, Context*);
-      virtual Node *node(int idx);
-      virtual KstObject::UpdateType update(int counter, Context *ctx);
+      Node *node(int idx);
+      KstObject::UpdateType update(int counter, Context *ctx);
+      QString text() const;
 
     protected:
       QPtrList<Node> _args;
@@ -97,13 +118,15 @@ namespace Equation {
   class Function : public Node {
     public:
       Function(char *name, ArgumentList *args);
-      virtual ~Function();
+      ~Function();
 
-      virtual bool isPlugin() const;
-      virtual bool isConst();
-      virtual double value(Context*);
-      virtual void collectVectors(KstVectorList& c);
-      virtual KstObject::UpdateType update(int counter, Context *ctx);
+      bool isPlugin() const;
+      bool isConst();
+      double value(Context*);
+      void collectObjects(KstVectorMap& v, KstScalarMap& s, KstStringMap& t);
+      bool takeVectors(const KstVectorMap& c);
+      KstObject::UpdateType update(int counter, Context *ctx);
+      QString text() const;
 
     protected:
       char *_name;
@@ -120,16 +143,18 @@ namespace Equation {
       double **_inVectors, **_outVectors;
       int _outputIndex; // -1 = scalar 0, -424242 = none/error, +0...n = vector
       void *_localData;
+      int _argCount;
   };
- 
+
 
   class Number : public Node {
     public:
       Number(double n);
-      virtual ~Number();
+      ~Number();
 
-      virtual bool isConst();
-      virtual double value(Context*);
+      bool isConst();
+      double value(Context*);
+      QString text() const;
 
     protected:
       double _n;
@@ -139,11 +164,12 @@ namespace Equation {
   class Identifier : public Node {
     public:
       Identifier(char *name);
-      virtual ~Identifier();
+      ~Identifier();
 
-      virtual bool isConst();
-      virtual double value(Context*);
-      virtual const char *name() const;
+      bool isConst();
+      double value(Context*);
+      const char *name() const;
+      QString text() const;
 
     protected:
       char *_name;
@@ -155,50 +181,63 @@ namespace Equation {
     friend class Function;
     public:
       Data(char *name);
-      virtual ~Data();
+      ~Data();
 
-      virtual bool isConst();
-      virtual double value(Context*);
-      virtual void collectVectors(KstVectorList& c);
-      virtual KstObject::UpdateType update(int counter, Context *ctx);
+      bool isConst();
+      double value(Context*);
+      void collectObjects(KstVectorMap& v, KstScalarMap& s, KstStringMap& t);
+      bool takeVectors(const KstVectorMap& c);
+      KstObject::UpdateType update(int counter, Context *ctx);
+      QString text() const;
 
     protected:
-      char *_name;
+      QString _tagName;
       KstVectorPtr _vector;
       KstScalarPtr _scalar;
   };
 
   class NodeVisitor {
     public:
+      NodeVisitor();
       virtual ~NodeVisitor();
 
-      virtual void visitNode(Node*);
-      virtual void visitBinaryNode(BinaryNode*);
-    
-    protected:
-      NodeVisitor();  // cannot instantiate base class directly; need a sub-class that does something interesting
-  };
-  
-    
-  class FoldVisitor : public NodeVisitor {
-    public:
-      FoldVisitor(Context*, Node*);
-      virtual ~FoldVisitor();
-      
-      virtual void visitNode(Node*);
-      virtual void visitBinaryNode(BinaryNode*);
-    
-    protected:
-      Context* _ctx;
+      virtual void visitNode(Node*) = 0;
+      virtual void visitBinaryNode(BinaryNode*) = 0;
   };
 
 
+  class FoldVisitor : public NodeVisitor {
+    public:
+      FoldVisitor(Context*, Node**);
+      ~FoldVisitor();
+
+      void visitNode(Node*);
+      void visitBinaryNode(BinaryNode*);
+
+    protected:
+      Context *_ctx;
+  };
+
+
+  class Negation : public Node {
+    public:
+      Negation(Node *node);
+      ~Negation();
+      bool isConst();
+      double value(Context*);
+      QString text() const;
+
+    protected:
+      Node *_n;
+  };
+
   class LogicalNot : public Node {
     public:
       LogicalNot(Node *node);
-      virtual ~LogicalNot();
-      virtual bool isConst();
-      virtual double value(Context*);
+      ~LogicalNot();
+      bool isConst();
+      double value(Context*);
+      QString text() const;
 
     protected:
       Node *_n;
@@ -208,9 +247,10 @@ namespace Equation {
   class x : public BinaryNode {           \
     public:                               \
       x(Node *left, Node *right);         \
-      virtual ~x();                       \
-      virtual bool isConst();             \
-      virtual double value(Context*);     \
+      ~x();                               \
+      bool isConst();                     \
+      double value(Context*);             \
+      QString text() const;               \
   };
 
 CreateNode(Addition)
