@@ -33,7 +33,7 @@ static KStaticDeleter<ExtensionMgr> sdExtension;
 
 ExtensionMgr *ExtensionMgr::self() {
   if (!_self) {
-    _self = sdExtension.setObject(_self, new ExtensionMgr);
+    sdExtension.setObject(_self, new ExtensionMgr);
   }
 
   return _self;
@@ -73,11 +73,8 @@ ExtensionMgr::ExtensionMgr() : QObject(), _window(0L) {
 
 ExtensionMgr::~ExtensionMgr() {
   save();
-  
-  //
   // we do not need to explicitly delete any remaining 
-  //	extensions as they are handled automatically...
-  //
+  // extensions as they are handled automatically...
 }
 
 
@@ -102,7 +99,7 @@ void ExtensionMgr::loadExtension(const KService::Ptr& service) {
     KstDebug::self()->log(i18n("Kst Extension %1 loaded.").arg(name));
     doRegister(name,e);
   } else {
-    KstDebug::self()->log(i18n("Error trying to load Kst extension %1.  Code=%2, \"%3\"").arg(name).arg(err).arg(err == KParts::ComponentFactory::ErrNoLibrary ? i18n("Library not found.") : KLibLoader::self()->lastErrorMessage()), KstDebug::Error);
+    KstDebug::self()->log(i18n("Error trying to load Kst extension %1.  Code=%2, \"%3\"").arg(name).arg(err).arg(err == KParts::ComponentFactory::ErrNoLibrary ? i18n("Library not found [%1].").arg(KLibLoader::self()->lastErrorMessage()) : KLibLoader::self()->lastErrorMessage()), KstDebug::Error);
   }
 }
 
