@@ -18,6 +18,8 @@
 #include <assert.h>
 
 // include files for Qt
+#include <qclipboard.h>
+#include <qdeepcopy.h>
 #include <qeventloop.h>
 #include <qpaintdevicemetrics.h>
 #include <qprogressbar.h>
@@ -30,8 +32,10 @@
 #include <kkeydialog.h>
 #include <kmessagebox.h>
 #include <kprinter.h>
+#include <kprogress.h>
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
+#include <qdatetime.h>
 
 // application specific includes
 #include "datawizard.h"
@@ -59,14 +63,17 @@
 #include "kstplugindialog_i.h"
 #include "kstprintoptionspage.h"
 #include "kstpsddialog_i.h"
+#include "kstquickstartdialog_i.h"
 #include "kstsettingsdlg.h"
+#include "kstuinames.h"
 #include "kstvectordefaults.h"
 #include "kstvectordialog_i.h"
 #include "kstviewscalarsdialog_i.h"
 #include "kstviewvectorsdialog_i.h"
+#include "kstviewmatricesdialog_i.h"
 #include "kstviewfitsdialog_i.h"
 #include "kstviewwindow.h"
-#include "kstquickstartdialog_i.h"
+#include "plotmimesource.h"
 #include "pluginmanager.h"
 #include "psversion.h"
 #include "sysinfo.h"
@@ -81,7 +88,7 @@
 
 static KstApp *inst = 0L;
 
-static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
+const QString& KstApp::defaultTag = KGlobal::staticQString("<Auto Name>");
 
 void KstApp::doubleCheckCleanup() {
   delete ::inst;
@@ -99,6 +106,8 @@ KstApp::KstApp(QWidget *parent, const char *name)
   assert(!::inst);
   ::inst = this;
 
+  KGlobal::dirs()->addResourceType("kst", KStandardDirs::kde_default("data") + "kst");
+
   _dataSourceConfig = kConfigObject;
 
   clearWFlags(WDestructiveClose);
@@ -106,8 +115,10 @@ KstApp::KstApp(QWidget *parent, const char *name)
   _stopping = false;
   config = kapp->config();
   initStatusBar();
+  setStandardToolBarMenuEnabled(true);
 
   initDocument();
+  KstDebug::self()->setHandler(doc);
   setCaption(doc->title());
 
   /* create dialogs */
@@ -116,6 +127,7 @@ KstApp::KstApp(QWidget *parent, const char *name)
   dataManager = new KstDataManagerI(doc, this);
   viewScalarsDialog = new KstViewScalarsDialogI(this);
   viewVectorsDialog = new KstViewVectorsDialogI(this);
+  viewMatricesDialog = new KstViewMatricesDialogI(this);
   viewFitsDialog = new KstViewFitsDialogI(this);
   changeFileDialog = new KstChangeFileDialogI(this);
   changeNptsDialog = new KstChangeNptsDialogI(this);
@@ -136,12 +148,10 @@ KstApp::KstApp(QWidget *parent, const char *name)
   connect(KstFitDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstFilterDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstEventMonitorI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
-
   connect(KstImageDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstMatrixDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
 
   initActions();
-
   readOptions();
 
   _updateThread = new UpdateThread(doc);
@@ -153,20 +163,20 @@ KstApp::KstApp(QWidget *parent, const char *name)
           this,             SLOT(registerDocChange()));
   connect(changeNptsDialog, SIGNAL(docChanged()),
           this,             SLOT(registerDocChange()));
-  connect(graphFileDialog, SIGNAL(graphFileReq(const QString&,const QString&,int,int,bool)), this, SLOT(immediatePrintToPng(const QString&,const QString&,int,int,bool)));
+  connect(graphFileDialog, SIGNAL(graphFileReq(const QString&,const QString&,int,int,bool,int)), this, SLOT(immediatePrintToPng(const QString&,const QString&,int,int,bool,int)));
 
   /*** plot dialog ***/
   connect(_plotDialog, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
 
-  //data manager signals
+  // data manager signals
   connect(dataManager, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
-
-  /*** ***/
   connect(doc, SIGNAL(updateDialogs()), this, SLOT(updateDialogs()));
   connect(doc, SIGNAL(dataChanged()), this, SLOT(updateDataDialogs()));
 
   connect(dataManager, SIGNAL(editDataVector(const QString&)),
           KstVectorDialogI::globalInstance(), SLOT(show_Edit(const QString&)));
+  connect(dataManager, SIGNAL(editStaticVector(const QString&)),
+          KstVectorDialogI::globalInstance(), SLOT(show_Edit(const QString&)));
 
   setAutoSaveSettings("KST-KMainWindow", true);
   _dcopIface = new KstIfaceImpl(doc, this);
@@ -175,9 +185,7 @@ KstApp::KstApp(QWidget *parent, const char *name)
 
   QTimer::singleShot(0, this, SLOT(updateActions()));
 
-  //
   // load any extensions
-  //
   ExtensionMgr *mgr = ExtensionMgr::self();
   mgr->setWindow(this);
   KService::List sl = KServiceType::offers("Kst Extension");
@@ -191,7 +199,7 @@ KstApp::KstApp(QWidget *parent, const char *name)
   }
 
   checkFontPresent("Symbol");
-  //checkFontPresent("Helvetica");
+  checkFontPresent("Helvetica");
 
   QFont defaultFont; // QT's current default font
   QFontInfo info(defaultFont);
@@ -215,21 +223,16 @@ KstApp::KstApp(QWidget *parent, const char *name)
     if (!addedWindowMenu) {
       menuBar()->insertItem(i18n("&Window"), windowMenu());
     }
-    QObject::connect( windowMenu(), SIGNAL(aboutToShow()), this, SLOT(addNewWindowMenu()) );
+    connect(windowMenu(), SIGNAL(aboutToShow()), this, SLOT(addNewWindowMenu()));
   }
 }
 
 
 KstApp::~KstApp() {
   _updateThread->setFinished(true);
-#if 0
-  if (!_updateThread->wait(250)) { // 250ms
+  if (!_updateThread->wait(3000)) { // 3s
     _updateThread->terminate();
-#endif
-    _updateThread->wait(3000); // QThread object destroyed while thread running on RH 9, but freezes on exit without it.  ->terminate() crashes.  Can we win?
-#if 0
   }
-#endif
 
   KstDataSource::cleanupForExit(); // must be before deletions
   delete _updateThread;
@@ -247,6 +250,7 @@ KstApp::~KstApp() {
 
 
 void KstApp::initialize() {
+  KstSettings::checkUpdates();
   kConfigObject = new KConfig("kstdatarc", false, false);
   KstDataSource::setupOnStartup(kConfigObject);
   // NOTE: This is leaked in commandline mode if we never create a KstApp.
@@ -323,21 +327,11 @@ void KstApp::customEvent(QCustomEvent *pEvent) {
 
 void KstApp::updateActions() {
   // Hack
-  ToolBarAction->setChecked(toolBar("mainToolBar")->isShown());
   StatusBarAction->setChecked(statusBar()->isShown());
 }
 
 
 void KstApp::initActions() {
-#if 0
-  fileNewWindow = new KAction(i18n("New &Window..."), 0, 0,
-                              this, SLOT(slotFileNewWindow()),
-                              actionCollection(),"file_new_window");
-
-  fileOpenNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
-  fileOpenNew->setWhatsThis(i18n("Open Kst plot file."));
-#endif
-
   fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
   fileSave->setWhatsThis(i18n("Save to current Kst plot file."));
 
@@ -364,7 +358,10 @@ void KstApp::initActions() {
                                      "to configure Kst settings."));
 
   fileCopy = KStdAction::copy(this, SLOT(slotCopy()), actionCollection());
-  fileCopy->setWhatsThis(i18n("Copy cursor position to the clipboard."));
+  fileCopy->setWhatsThis(i18n("Copy cursor position or plots to the clipboard."));
+
+  filePaste = KStdAction::paste(this, SLOT(slotPaste()), actionCollection());
+  filePaste->setWhatsThis(i18n("Paste plots from the clipboard."));
 
   /************/
   filePrint = KStdAction::print(this, SLOT(slotFilePrint()),
@@ -372,12 +369,6 @@ void KstApp::initActions() {
   filePrint->setToolTip(i18n("Print"));
   filePrint->setWhatsThis(i18n("Print current display"));
 
-  /************/
-  ToolBarAction = KStdAction::showToolbar(this, SLOT(slotViewToolBar()),
-                                          actionCollection());
-  ToolBarAction->setWhatsThis(i18n("Toggle Toolbar"));
-  connect(ToolBarAction, SIGNAL(activated()), this, SLOT(setSettingsDirty()));
-
   /************/
   StatusBarAction = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()),
                                               actionCollection());
@@ -458,7 +449,7 @@ void KstApp::initActions() {
                                        "to manage data."));
 
   /************/
-  VectorDialogAction = new KAction(i18n("New &Vector..."), 0, 0,
+  VectorDialogAction = new KAction(i18n("New &Vector..."), "kst_vectornew", 0,
                                  KstVectorDialogI::globalInstance(),
                                  SLOT(show_New()), actionCollection(),
                                  "vectordialog_action");
@@ -466,7 +457,7 @@ void KstApp::initActions() {
                                         "to create a new vector."));
 
   /************/
-  CurveDialogAction = new KAction(i18n("New &Curve..."), 0, 0,
+  CurveDialogAction = new KAction(i18n("New &Curve..."), "kst_curvenew", 0,
                                   KstCurveDialogI::globalInstance(),
                                   SLOT(show_New()), actionCollection(),
                                   "curvedialog_action");
@@ -474,7 +465,8 @@ void KstApp::initActions() {
                                        "to create a new curve."));
 
   /************/
-  EqDialogAction = new KAction(i18n("New &Equation..."), 0, 0,
+  EqDialogAction = new KAction(i18n("New &Equation..."),
+			       "kst_equationnew", 0,
                                KstEqDialogI::globalInstance(),
                                SLOT(show_New()),
                                actionCollection(), "eqdialog_action");
@@ -482,7 +474,8 @@ void KstApp::initActions() {
                                     "to create a new equation."));
 
   /************/
-  HsDialogAction = new KAction(i18n("New &Histogram..."), 0, 0,
+  HsDialogAction = new KAction(i18n("New &Histogram..."),
+			       "kst_histogramnew", 0,
                                KstHsDialogI::globalInstance(),
                                SLOT(show_New()), actionCollection(),
                                "hsdialog_action");
@@ -490,7 +483,8 @@ void KstApp::initActions() {
                                     "to create a new histogram."));
 
   /************/
-  PsdDialogAction = new KAction(i18n("New Power &Spectrum..."), 0, 0,
+  PsdDialogAction = new KAction(i18n("New Power &Spectrum..."),
+				"kst_psdnew", 0,
                                 KstPsdDialogI::globalInstance(),
                                 SLOT(show_New()), actionCollection(),
                                 "psddialog_action");
@@ -498,7 +492,8 @@ void KstApp::initActions() {
                                      "to create a new power spectrum."));
 
   /************/
-  PluginDialogAction = new KAction(i18n("New &Plugin..."), 0, 0,
+  PluginDialogAction = new KAction(i18n("New &Plugin..."),
+				   "kst_pluginnew", 0,
                                    KstPluginDialogI::globalInstance(),
                                    SLOT(show_New()), actionCollection(),
                                    "plugindialog_action");
@@ -506,7 +501,7 @@ void KstApp::initActions() {
                                         "to create a new plugin instance."));
 
   /************/
-  MatrixDialogAction = new KAction(i18n("New M&atrix..."), 0, 0,
+  MatrixDialogAction = new KAction(i18n("New M&atrix..."), "kst_matrixnew", 0,
                                    KstMatrixDialogI::globalInstance(),
                                    SLOT(show_New()), actionCollection(),
                                    "matrixdialog_action");
@@ -514,7 +509,8 @@ void KstApp::initActions() {
                                         "to create a new matrix."));
 
   /************/
-  ImageDialogAction = new KAction(i18n("New &Image..."), 0, 0,
+  ImageDialogAction = new KAction(i18n("New &Image..."),
+				  "kst_imagenew", 0,
                                    KstImageDialogI::globalInstance(),
                                    SLOT(show_New()), actionCollection(),
                                    "imagedialog_action");
@@ -548,6 +544,15 @@ void KstApp::initActions() {
   ViewVectorsDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
                                             "to view vector values."));
 
+  /************/
+  ViewMatricesDialogAction = new KAction(i18n("View &Matrix Values"),
+                                       0, 0, this,
+                                       SLOT(showViewMatricesDialog()),
+                                       actionCollection(),
+                                       "viewmatricesdialog_action");
+  ViewMatricesDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
+                                            "to view matrix values."));
+
   /************/
   ViewFitsDialogAction = new KAction(i18n("View &Fit Results"),
                                        0, 0, this,
@@ -568,7 +573,7 @@ void KstApp::initActions() {
 
   /************/
   EventMonitorAction = new KAction(i18n("New Event &Monitor..."),
-                                     0, 0,
+                                     "kst_eventnew", 0,
                                      KstEventMonitorI::globalInstance(),
                                      SLOT(show_New()),
                                      actionCollection(),
@@ -598,7 +603,7 @@ void KstApp::initActions() {
   /************/
   SamplesDownAction = new KAction(i18n("&Back 1 Screen"),
                                   "kst_back",
-                                  KAccel::stringToKey("Ctrl+Left"),
+                                  KShortcut(CTRL + Key_Left),
                                   this, SLOT(samplesDown()),
                                   actionCollection(),
                                   "samplesdown_action");
@@ -609,7 +614,7 @@ void KstApp::initActions() {
   /************/
   SamplesUpAction = new KAction(i18n("&Advance 1 Screen"),
                                 "kst_advance",
-                                KAccel::stringToKey("Ctrl+Right"),
+                                KShortcut(CTRL + Key_Right),
                                 this, SLOT(samplesUp()),
                                 actionCollection(),
                                 "samplesup_action");
@@ -621,7 +626,7 @@ void KstApp::initActions() {
   /************/
   SamplesFromEndAction = new KAction(i18n("Read From &End"),
                                      "1rightarrow",
-                                     KAccel::stringToKey("Shift+Ctrl+Right"),
+                                     KShortcut(SHIFT + CTRL + Key_Right),
                                      this, SLOT(fromEnd()),
                                      actionCollection(),
                                      "samplesend_action");
@@ -708,15 +713,22 @@ void KstApp::slotConfigureKeys() {
 }
 
 
+bool KstApp::paused() const {
+  return _updateThread->paused();
+}
+
+
 void KstApp::setPaused(bool in_paused) {
   PauseAction->setChecked(in_paused);
   _updateThread->setPaused(in_paused);
 }
 
+
 void KstApp::togglePaused() {
   setPaused(!PauseAction->isChecked());
 }
 
+
 KstApp::KstZoomType KstApp::getZoomRadio() {
   if (XZoomAction->isChecked()) {
     return XZOOM;
@@ -736,14 +748,16 @@ void KstApp::initStatusBar() {
   _dataBar = new QLabel(QString::null, statusBar());
   statusBar()->addWidget(_dataBar, 5, true);
 
-  _readyBar = new KstQuickLabel(i18n("Almost Ready"), statusBar());
+  _readyBar = new QLabel(i18n("Almost Ready"), statusBar());
   _readyBar->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
   statusBar()->addWidget(_readyBar, 5, true);
 
-  _progressBar = new KstQuickProgressBar(statusBar());
+  _progressBar = new KProgress(statusBar());
   _progressBar->setPercentageVisible(false);
   _progressBar->setCenterIndicator(true);
   statusBar()->addWidget(_progressBar, 2, true);
+  _progressBar->setMaximumHeight( fontMetrics().height() );
+  _progressBar->hide();
 
 #ifdef HAVE_LINUX
   _memoryBar = new QLabel(i18n("0 MB available"), statusBar());
@@ -763,7 +777,6 @@ void KstApp::initDocument() {
 
 void KstApp::forceUpdate() {
   _updateThread->forceUpdate();
-  QTimer::singleShot(0, doc, SIGNAL(updateDialogs()));
 }
 
 
@@ -771,10 +784,56 @@ void KstApp::addRecentFile(const KURL& url) {
   recent->addURL(url);
 }
 
+
+void KstApp::selectRecentFile(const KURL &url) {
+  QStringList urls;
+  int count;
+  int i;
+
+  if (url.isEmpty()) {
+    recent->setCurrentItem(-1);
+  } else {
+    urls = recent->items();
+    count = urls.count();
+    for (i=0; i<count; i++) {
+      if (KURL(urls[i]) == url) {
+        recent->setCurrentItem(i);
+        break;
+      }
+    }
+  }
+}
+
+
+void KstApp::doDelayedOpens() {
+  QValueList<KstOpen> queueCopy = QDeepCopy<QValueList<KstOpen> >(_openQueue);
+  _openQueue.clear();
+
+  for (QValueList<KstOpen>::ConstIterator i = queueCopy.begin(); i != queueCopy.end(); ++i) {
+    openDocumentFile((*i).filename, (*i).file, (*i).n, (*i).f, (*i).s, (*i).ave);
+  }
+}
+
+
 bool KstApp::openDocumentFile(const QString& in_filename,
-			      const QString& o_file, int o_n, int o_f, int o_s, bool o_ave) {
+    const QString& o_file, int o_n, int o_f, int o_s, bool o_ave, bool delayed) {
+  static bool opening = false;
+
+  if (delayed || opening) {
+    KstOpen job;
+    job.filename = in_filename;
+    job.file = o_file;
+    job.n = o_n;
+    job.f = o_f;
+    job.s = o_s;
+    job.ave = o_ave;
+    _openQueue.append(job);
+    QTimer::singleShot(0, this, SLOT(doDelayedOpens()));
+    return true;
+  }
+
+  opening = true;
   KURL url;
-  QFileInfo finfo(in_filename);
   bool rc = false;
 
   url.setPath(in_filename);
@@ -783,6 +842,7 @@ bool KstApp::openDocumentFile(const QString& in_filename,
   if (doc->openDocument(url, o_file, o_n, o_f, o_s, o_ave)) {
     setCaption(doc->title());
     if (url.isLocalFile()) {
+      QFileInfo finfo(in_filename);
       addRecentFile(finfo.absFilePath());
     } else {
       addRecentFile(url);
@@ -790,6 +850,7 @@ bool KstApp::openDocumentFile(const QString& in_filename,
     rc = true;
   }
   slotUpdateStatusMsg(i18n("Ready"));
+  opening = false;
   return rc;
 }
 
@@ -892,6 +953,7 @@ void KstApp::slotFileNew() {
   if (doc->saveModified()) {
     doc->newDocument();
     setCaption(doc->title());
+    selectRecentFile(KURL(""));
   }
 
   slotUpdateStatusMsg(i18n("Ready"));
@@ -984,13 +1046,6 @@ void KstApp::slotFileClose() {
   }
 }
 
-void KstApp::slotFilePrintActiveWindow() {
-  KstViewWindow *view = dynamic_cast<KstViewWindow*>(activeWindow());
-  if (view) {
-    view->slotFilePrint();
-  }
-}
-
 void KstApp::immediatePrintWindowToFile(KMdiChildView *win, const QString& filename) {
   KstViewWindow *view = dynamic_cast<KstViewWindow*>(win);
   if (view && !view->view()->children().isEmpty()) {
@@ -1005,20 +1060,38 @@ void KstApp::immediatePrintActiveWindowToFile(const QString& filename) {
   }
 }
 
-void KstApp::immediatePrintWindowToPng(KMdiChildView *win, const QString& filename, const QString& format, int width, int height) {
-  QSize size(width, height);
-
+void KstApp::immediatePrintWindowToPng(KMdiChildView *win, const QString& filename, const QString& format, int width, int height, int display) {
   KstViewWindow *view = dynamic_cast<KstViewWindow*>(win);
   if (view && !view->view()->children().isEmpty()) {
-    view->immediatePrintToPng(filename,size,format);
+    QSize size;
+
+    if (display == 0) {
+      size.setWidth(width);
+      size.setHeight(height);
+    } else if (display == 1) {
+      size.setWidth(width);
+      size.setHeight(width);
+    } else if (display == 2) {
+      QSize sizeWindow(view->geometry().size());
+
+      size.setWidth(width);
+      size.setHeight((int)((double)width * (double)sizeWindow.height() / (double)sizeWindow.width()));
+    } else {
+      QSize sizeWindow(view->geometry().size());
+
+      size.setHeight(height);
+      size.setWidth((int)((double)height * (double)sizeWindow.width() / (double)sizeWindow.height()));
+    }
+
+    view->immediatePrintToPng(filename, size, format);
   }
 }
 
-void KstApp::immediatePrintActiveWindowToPng(const QString& filename, const QString& format, int width, int height) {
+void KstApp::immediatePrintActiveWindowToPng(const QString& filename, const QString& format, int width, int height, int display) {
   KMdiChildView *win = activeWindow();
 
   if (win) {
-    immediatePrintWindowToPng(win, filename, format, width, height);
+    immediatePrintWindowToPng(win, filename, format, width, height, display);
   }
 }
 
@@ -1040,34 +1113,39 @@ void KstApp::slotFilePrint() {
 
   if (pages > 0) {
     KPrinter printer;
-    printer.setPageSize(KPrinter::Letter);
-    printer.setOrientation(KPrinter::Landscape);
+    KstSettings *ks = KstSettings::globalSettings();
+
+    printer.setOption("kde-pagesize", ks->printing.pageSize);
+    printer.setOption("kde-orientation", ks->printing.orientation);
+    printer.setOption("kst-plot-datetime-footer", ks->printing.plotDateTimeFooter);
+    printer.setOption("kst-plot-maintain-aspect-ratio", ks->printing.maintainAspect);
+    printer.setOption("kst-plot-curve-width-adjust", ks->printing.curveWidthAdjust);
+    printer.setOption("kst-plot-monochrome", ks->printing.monochrome);
     printer.setFromTo(0, 0);
     printer.setMinMax(1, pages);
 
     pages = 0;
     printer.addDialogPage(new KstPrintOptionsPage);
     if (printer.setup(this, i18n("Print"))) {
-      QDateTime dateTime = QDateTime::currentDateTime();
       QPainter paint(&printer);
       QPaintDeviceMetrics metrics(&printer);
-      QRect rect;
       QSize size(metrics.width(), metrics.height());
       int iFromPage = printer.fromPage();
       int iToPage = printer.toPage();
       bool datetimeFooter = printer.option("kst-plot-datetime-footer") == "1";
+      bool maintainAspectRatio = printer.option("kst-plot-maintain-aspect-ratio") == "1";
       int lineAdjust = printer.option("kst-plot-curve-width-adjust").toInt();
       bool monochrome = printer.option("kst-plot-monochrome") == "1";
 
       slotUpdateStatusMsg(i18n("Printing..."));
 
-      rect.setLeft(0);
-      rect.setRight(size.height());
-      rect.setBottom(size.height());
-      if (datetimeFooter) {
-        size.setHeight(9 * size.height() / 10);
-      }
-      rect.setTop(size.height());
+      ks->printing.pageSize = printer.option("kde-pagesize");
+      ks->printing.orientation = printer.option("kde-orientation");
+      ks->printing.plotDateTimeFooter = printer.option("kst-plot-datetime-footer");
+      ks->printing.maintainAspect = printer.option("kst-plot-maintain-aspect-ratio");
+      ks->printing.curveWidthAdjust = printer.option("kst-plot-curve-width-adjust");
+      ks->printing.monochrome = printer.option("kst-plot-monochrome");
+      ks->save();
 
       it = createIterator();
       if (it) {
@@ -1093,11 +1171,34 @@ void KstApp::slotFilePrint() {
                 }
               }
 
-              win->view()->resizeForPrint(size);
               if (datetimeFooter) {
+                QDateTime dateTime = QDateTime::currentDateTime();
                 QString title = i18n("Page: %1  Name: %2  Date: %3").arg(pages).arg(win->caption()).arg(dateTime.toString(Qt::ISODate));
+                QRect rect(0, 0, size.width(), size.height());
+                QRect rectBounding;
+
+                rectBounding = paint.boundingRect(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
+                rect.setTop(size.height() - (2 * rectBounding.height()));
                 paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
+                size.setHeight(rect.top());
               }
+
+              if (maintainAspectRatio) {
+                QRect  rectGeom;
+                double dRatioWindow;
+                double dRatioPrinter;
+
+                rectGeom      = win->view()->geometry();
+                dRatioWindow  = (double)rectGeom.width() / (double)rectGeom.height();
+                dRatioPrinter = (double)size.width() / (double)size.height();
+                if (dRatioWindow > dRatioPrinter) {
+                  size.setHeight((int)((double)size.width() / dRatioWindow));
+                } else if (dRatioWindow < dRatioPrinter) {
+                  size.setWidth((int)(dRatioWindow * (double)size.height()));
+                }
+              }
+
+              win->view()->resizeForPrint(size);
               win->view()->paint(P_PRINT, paint);
 
               if (tlv) {
@@ -1179,7 +1280,7 @@ void KstApp::immediatePrintToFile(const QString& filename) {
   }
 }
 
-void KstApp::immediatePrintToPng(const QString& filename, const QString& format, int width, int height, bool all) {
+void KstApp::immediatePrintToPng(const QString& filename, const QString& format, int width, int height, bool all, int display) {
   if (all) {
     QString filenameSub;
     int pages = 0;
@@ -1197,13 +1298,13 @@ void KstApp::immediatePrintToPng(const QString& filename, const QString& format,
       while (it->currentItem()) {
         pages++;
         QString filenameNew = i18n("%1_%2").arg(filenameSub).arg(pages);
-        immediatePrintWindowToPng(it->currentItem(), filenameNew, format, width, height);
+        immediatePrintWindowToPng(it->currentItem(), filenameNew, format, width, height, display);
         it->next();
       }
       deleteIterator(it);
     }
   } else {
-    immediatePrintActiveWindowToPng(filename, format, width, height);
+    immediatePrintActiveWindowToPng(filename, format, width, height, display);
   }
 }
 
@@ -1211,18 +1312,6 @@ void KstApp::slotFileQuit() {
   slotFileClose();
 }
 
-void KstApp::slotViewToolBar() {
-  slotUpdateStatusMsg(i18n("Toggling toolbar..."));
-  // turn Toolbar on or off
-  if (ToolBarAction->isChecked()) {
-    toolBar("mainToolBar")->show();
-  } else {
-    toolBar("mainToolBar")->hide();
-  }
-
-  slotUpdateStatusMsg(i18n("Ready."));
-}
-
 void KstApp::slotViewStatusBar() {
   if (StatusBarAction->isChecked()) {
     statusBar()->show();
@@ -1243,6 +1332,13 @@ void KstApp::slotUpdateDataMsg(const QString& msg) {
 
 
 void KstApp::slotUpdateProgress(int total, int step, const QString &msg) {
+  if (step == 0 && msg.isNull()) {
+    _readyBar->setText(i18n("Ready"));
+    _progressBar->hide();
+    return;
+  }
+
+  _progressBar->show();
   if (step > 0) {
     if (!_progressBar->percentageVisible()) {
       _progressBar->setPercentageVisible(true);
@@ -1263,6 +1359,8 @@ void KstApp::slotUpdateProgress(int total, int step, const QString &msg) {
   } else {
     _readyBar->setText(msg);
   }
+
+  kapp->eventLoop()->processEvents(QEventLoop::ExcludeSocketNotifiers, 10);
 }
 
 
@@ -1314,33 +1412,48 @@ void KstApp::tieAll() {
   paintAll(P_PAINT);
 }
 
-
+//#define PAINTTIMER
 void KstApp::paintAll(KstPaintType pt) {
-  KMdiIterator<KMdiChildView*> *it = createIterator();
-  if (it) {
-    while (it->currentItem()) {
-      KstViewWindow *pView = dynamic_cast<KstViewWindow*>(it->currentItem());
-      if (pView) {
-        pView->view()->paint(pt);
+#ifdef PAINTTIMER
+  QTime t;
+  t.start();
+#endif
+  if (mdiMode() == KMdi::TabPageMode || mdiMode() == KMdi::IDEAlMode) { // Optimization
+    KstViewWindow *view = dynamic_cast<KstViewWindow*>(activeWindow());
+    if (view) {
+      view->view()->paint(pt);
+    }
+  } else {
+    KMdiIterator<KMdiChildView*> *it = createIterator();
+    if (it) {
+      while (it->currentItem()) {
+        KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
+        if (view) {
+          view->view()->paint(pt);
+        }
+        it->next();
       }
-      it->next();
+      deleteIterator(it);
     }
-    deleteIterator(it);
   }
+#ifdef PAINTTIMER
+  int x = t.elapsed();
+  kdDebug() << "paintAll with painttype " << (int)pt << " - " << x << "ms" << endl;
+#endif
 }
 
 void KstApp::showPlotDialog() {
-  KMdiChildView *pWindow = activeWindow();
+  KMdiChildView *win = activeWindow();
 
-  if (_plotDialog->isHidden() && pWindow) {
-    _plotDialog->show_I(pWindow->caption(), QString::null);
+  if (win && _plotDialog->isHidden()) {
+    _plotDialog->show_I(win->caption(), QString::null);
   } else {
     _plotDialog->show_I();
   }
 }
 
-void KstApp::showPlotDialog(const QString& strWindow, const QString& strPlot) {
-  _plotDialog->show_I(strWindow, strPlot);
+void KstApp::showPlotDialog(const QString& window, const QString& plot) {
+  _plotDialog->show_I(window, plot);
 }
 
 void KstApp::showDataManager() {
@@ -1355,6 +1468,10 @@ void KstApp::showViewVectorsDialog() {
   viewVectorsDialog->showViewVectorsDialog();
 }
 
+void KstApp::showViewMatricesDialog() {
+  viewMatricesDialog->showViewMatricesDialog();
+}
+
 void KstApp::showViewFitsDialog() {
   viewFitsDialog->showViewFitsDialog();
 }
@@ -1368,7 +1485,7 @@ void KstApp::showChangeNptsDialog() {
 }
 
 void KstApp::showGraphFileDialog() {
-  graphFileDialog->showGraphFileDialog();
+  graphFileDialog->show_I();
 }
 
 void KstApp::showDebugDialog() {
@@ -1393,6 +1510,9 @@ void KstApp::updateDataDialogs(bool dm) {
   if (!viewVectorsDialog->isHidden()) {
     viewVectorsDialog->updateViewVectorsDialog();
   }
+  if (!viewMatricesDialog->isHidden()) {
+    viewMatricesDialog->updateViewMatricesDialog();
+  }
   if (!viewFitsDialog->isHidden()) {
     viewFitsDialog->updateViewFitsDialog();
   }
@@ -1405,25 +1525,67 @@ void KstApp::updateDataDialogs(bool dm) {
 }
 
 
-void KstApp::updateDialogs() {
+void KstApp::updateVisibleDialogs() {
+  updateDialogs(true);
+}
+
+
+void KstApp::updateDialogs(bool onlyVisible) {
   if (!_stopping) {
-    KstVectorDialogI::globalInstance()->update();
-    KstPluginDialogI::globalInstance()->updateForm();
-    KstFitDialogI::globalInstance()->update();
-    KstFilterDialogI::globalInstance()->update();
-    KstEqDialogI::globalInstance()->update();
-    KstHsDialogI::globalInstance()->update();
-    KstPsdDialogI::globalInstance()->update();
-    KstCurveDialogI::globalInstance()->update();
-    KstEventMonitorI::globalInstance()->update();
-    KstImageDialogI::globalInstance()->update();
-    KstMatrixDialogI::globalInstance()->update();
-    _plotDialog->update();
-    changeFileDialog->updateChangeFileDialog();
-    changeNptsDialog->updateChangeNptsDialog();
-    updateDataDialogs(false);
-    vectorSaveDialog->init();
-    dataManager->update();
+    QTime t;
+    t.start();
+    if (!onlyVisible || KstVectorDialogI::globalInstance()->isShown()) {
+      KstVectorDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstPluginDialogI::globalInstance()->isShown()) {
+      KstPluginDialogI::globalInstance()->updateForm();
+    }
+    if (!onlyVisible || KstFitDialogI::globalInstance()->isShown()) {
+      KstFitDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstFilterDialogI::globalInstance()->isShown()) {
+      KstFilterDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstEqDialogI::globalInstance()->isShown()) {
+      KstEqDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstHsDialogI::globalInstance()->isShown()) {
+      KstHsDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstPsdDialogI::globalInstance()->isShown()) {
+      KstPsdDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstCurveDialogI::globalInstance()->isShown()) {
+      KstCurveDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstEventMonitorI::globalInstance()->isShown()) {
+      KstEventMonitorI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstImageDialogI::globalInstance()->isShown()) {
+      KstImageDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || KstMatrixDialogI::globalInstance()->isShown()) {
+      KstMatrixDialogI::globalInstance()->update();
+    }
+    if (!onlyVisible || _plotDialog->isShown()) {
+      _plotDialog->update();
+    }
+    if (!onlyVisible || changeFileDialog->isShown()) {
+      changeFileDialog->updateChangeFileDialog();
+    }
+    if (!onlyVisible || changeNptsDialog->isShown()) {
+      changeNptsDialog->updateChangeNptsDialog();
+    }
+    if (!onlyVisible) { // FIXME: might want to make this sensible one day
+      updateDataDialogs(false);
+    }
+    if (!onlyVisible || vectorSaveDialog->isShown()) {
+      vectorSaveDialog->init();
+    }
+    if (!onlyVisible || dataManager->isShown()) {
+      dataManager->update();
+    }
+    kdDebug() << "Dialogs updated in " << t.elapsed() << "ms" << endl;
   }
 }
 
@@ -1473,6 +1635,7 @@ void KstApp::showDataWizard() {
 void KstApp::registerDocChange() {
   kdDebug() << "register doc changed" << endl;
   forceUpdate();
+  updateVisibleDialogs();
   doc->setModified();
 }
 
@@ -1482,7 +1645,9 @@ void KstApp::reload() {
   for (KstVectorList::ConstIterator i = KST::vectorList.begin(); i != KST::vectorList.end(); ++i) {
     KstRVectorPtr r = kst_cast<KstRVector>(*i);
     if (r) {
+      r->writeLock();
       r->reload();
+      r->writeUnlock();
     }
   }
 }
@@ -1502,20 +1667,97 @@ void KstApp::slotSettingsChanged() {
 
 
 void KstApp::slotCopy() {
-  KstTopLevelViewPtr tlv = activeView();
-  if (tlv) {
-    KstViewWidget *w = tlv->widget();
-    KstViewObjectPtr o = w->findChildFor(w->mapFromGlobal(QCursor::pos()));
-    Kst2DPlotPtr p = kst_cast<Kst2DPlot>(o);
-    if (p) {
-      p->copy();
+  if (!LayoutAction->isChecked()) {
+    KstTopLevelViewPtr tlv = activeView();
+    if (tlv) {
+      KstViewWidget *w = tlv->widget();
+      KstViewObjectPtr o = w->findChildFor(w->mapFromGlobal(QCursor::pos()));
+      Kst2DPlotPtr p = kst_cast<Kst2DPlot>(o);
+      if (p) {
+        p->copy();
+      }
+    }
+  } else {
+    KstViewWindow *vw = dynamic_cast<KstViewWindow*>(activeWindow());
+    if (vw) {
+      QApplication::clipboard()->setData(vw->view()->widget()->dragObject(), QClipboard::Clipboard);
     }
   }
 }
 
 
-void KstApp::slotFileNewWindow() {
-  newWindow();
+bool KstApp::paste(QMimeSource* source, KstTopLevelViewPtr tlv) {
+  QStringList plotList;
+  QString window;
+  bool rc = false;
+
+  if (source && source->provides(PlotMimeSource::mimeType())) {
+    QDataStream ds(source->encodedData(PlotMimeSource::mimeType()), IO_ReadOnly);
+    KstViewWindow *w;
+
+    ds >> window;
+    ds >> plotList;
+
+    w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(window));
+    if (w && plotList.size() > 0) {
+      for (size_t i=0; i<plotList.size(); i++) {
+        KstViewObjectPtr copy = w->view()->findChild(plotList[i]);
+
+        if (copy) {
+          QString plotName;
+          bool duplicate = true;
+          int number = 0;
+
+          while (duplicate) {
+            if (number == 0) {
+              plotName = copy->tagName();
+            } else if (number == 1) {
+              plotName = i18n("%1-copy").arg(copy->tagName());
+            } else {
+              plotName = i18n("%1-copy%2").arg(copy->tagName()).arg(number);
+            }
+            number++;
+            if (tlv->findChild(plotName)) {
+              duplicate = true;
+            } else {
+              duplicate = false;
+            }
+          }
+
+          copy->copyObjectQuietly(*tlv, plotName);
+        }
+      }
+
+      rc = true;
+    }
+  }
+
+  return rc;
+}
+
+
+void KstApp::slotPaste() {
+  if (LayoutAction->isChecked()) {
+    KstTopLevelViewPtr tlv = activeView();
+
+    if (tlv) {
+      QMimeSource* source;
+
+      source = QApplication::clipboard()->data(QClipboard::Clipboard);
+      if (!paste(source, tlv)) {
+        KstDebug::self()->log(i18n("Paste operation failed: clipboard data was not found or of the wrong format."));
+      }
+    } else {
+      KstDebug::self()->log(i18n("Paste operation failed: there is currently no active view."));
+    }
+  } else {
+    KstDebug::self()->log(i18n("Paste operation failed: must be in layout mode."));
+  }
+}
+
+
+void KstApp::slotFileNewWindow(QWidget *parent) {
+  newWindow(true, parent);
   doc->setModified();
 }
 
@@ -1535,6 +1777,15 @@ void KstApp::slotFileRenameWindow() {
   }
 }
 
+QString KstApp::newWindow(bool prompt, QWidget *parent) {
+  QString nameUsed;
+  QString name = windowName(prompt, defaultTag, false, parent);
+  if (!name.isEmpty()) {
+    nameUsed = newWindow(name);
+  }
+
+  return nameUsed;
+}
 
 QString KstApp::newWindow(const QString& name_in) {
   KstViewWindow *w = new KstViewWindow;
@@ -1548,14 +1799,14 @@ QString KstApp::newWindow(const QString& name_in) {
 
   w->setCaption(nameToUse);
   w->setTabCaption(nameToUse);
-  addWindow(w);
+  addWindow(w, KMdi::StandardAdd | KMdi::UseKMdiSizeHint);
   w->activate();
   updateDialogsForWindow();
   return nameToUse;
 }
 
 
-QString KstApp::windowName(bool prompt, const QString& nameOriginal, bool rename) {
+QString KstApp::windowName(bool prompt, const QString& nameOriginal, bool rename, QWidget *parent) {
   bool ok = false;
   QString name = nameOriginal;
 
@@ -1563,10 +1814,19 @@ QString KstApp::windowName(bool prompt, const QString& nameOriginal, bool rename
     if (prompt) {
       QRegExp exp("\\S+.*");
       QRegExpValidator val(exp, 0L);
+
       if (rename) {
-        name = KLineEditDlg::getText(i18n("Enter a new name for the window:"), name, &ok, 0L, &val).stripWhiteSpace();
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+        name = KInputDialog::getText(i18n("Kst"), i18n("Enter a new name for the window:"), name, &ok, parent, 0L, &val).stripWhiteSpace();
+#else
+        name = KLineEditDlg::getText(i18n("Enter a new name for the window:"), name, &ok, parent, &val).stripWhiteSpace();
+#endif
       } else {
-        name = KLineEditDlg::getText(i18n("Enter a name for the new window:"), name, &ok, 0L, &val).stripWhiteSpace();
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+        name = KInputDialog::getText(i18n("Kst"), i18n("Enter a name for the new window:"), name, &ok, parent, 0L, &val).stripWhiteSpace();
+#else
+        name = KLineEditDlg::getText(i18n("Enter a name for the new window:"), name, &ok, parent, &val).stripWhiteSpace();
+#endif
       }
       if (ok && name==defaultTag) {
         name = KST::suggestWinName();
@@ -1591,17 +1851,6 @@ QString KstApp::windowName(bool prompt, const QString& nameOriginal, bool rename
 }
 
 
-QString KstApp::newWindow(bool prompt) {
-  QString nameUsed;
-  QString name = windowName(prompt, defaultTag);
-  if (!name.isEmpty()) {
-    nameUsed = newWindow(name);
-  }
-
-  return nameUsed;
-}
-
-
 KstTopLevelViewPtr KstApp::activeView() {
   KstViewWindow *vw = dynamic_cast<KstViewWindow*>(activeWindow());
 
