@@ -31,10 +31,11 @@
 #include <qvaluestack.h>
 
 class KstLegend;
+typedef KstSharedPtr<KstLegend> KstLegendPtr;
 class Kst2DPlot;
 typedef KstObjectList<KstSharedPtr<Kst2DPlot> > Kst2DPlotList;
 
-enum KstScaleModeType { AUTO, AC, FIXED, AUTOUP, NOSPIKE };
+enum KstScaleModeType { AUTO, AC, FIXED, AUTOUP, NOSPIKE, AUTOBORDER };
 
 struct KstPlotScale {
   double xmin;
@@ -57,10 +58,11 @@ typedef QValueList<KstMarker> KstMarkerList;
 enum KstMouseModeType { INACTIVE, XY_ZOOMBOX, Y_ZOOMBOX, X_ZOOMBOX, LABEL_TOOL, LAYOUT_TOOL };
 
 struct KstMouse {
-  KstMouse() { mode = INACTIVE; label = -1; minMove = 2; }
+  KstMouse();
   KstMouseModeType mode;
   int label, minMove;
-  QPoint lastLocation, pressLocation;
+  QPoint lastLocation, pressLocation;  // for zooming primarily
+  QPoint tracker; // for tracking the mouse location
   QRect plotGeometry;
   bool zooming() const;
   void zoomStart(KstMouseModeType t, const QPoint& location);
@@ -74,33 +76,36 @@ class Kst2DPlot : public KstPlotBase {
   Q_OBJECT
 public:
   Kst2DPlot(const QString& in_tag = "SomePlot",
-          KstScaleModeType yscale = AUTO,
+          KstScaleModeType yscale = AUTOBORDER,
           KstScaleModeType xscale = AUTO,
           double xmin = 0, double ymin = 0,
           double xmax = 1, double ymax = 1);
-  Kst2DPlot(QDomElement& e);
+  Kst2DPlot(const QDomElement& e);
+  Kst2DPlot(const Kst2DPlot& plot, const QString& name);
   virtual ~Kst2DPlot();
 
   static Kst2DPlotList globalPlotList();
   static bool checkRange(double& min_in, double& max_in);
   static bool checkLRange(double& min_in, double& max_in, bool isLog);
   static void genAxisTickLabel(QString& label, double z, bool isLog);
-
-  virtual UpdateType update();
+  static double timezoneHrs( );
+  
+  virtual UpdateType update(int update_counter);
   virtual void save(QTextStream& ts, const QString& indent = QString::null);
   virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
 
   virtual bool popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
   virtual bool layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
 
-  void drawDotAt(QPainter& p, double x, double y, KstBaseCurvePtr curve);
-  void internalAlignment(QRect& plotRegion);
+  void drawDotAt(QPainter& p, double x, double y);
+  void drawPlusAt(QPainter& p, double x, double y);
+  void internalAlignment(KstPaintType type, QPainter& p, QRect& plotRegion);
   void addCurve(KstBaseCurvePtr curve);
-  void addLabel(KstLabel* label);
+  void addLabel(KstLabelPtr label);
   void removeCurve(KstBaseCurvePtr curve);
   void setXScaleMode(KstScaleModeType scalemode);
   void setYScaleMode(KstScaleModeType scalemode);
-  
+
   void pushAdjustLineWidth(int adjustment);
   void popLineWidth();
   void pushCurveColor(const QColor& c);
@@ -129,8 +134,10 @@ public:
   void setLog(bool x_log, bool y_log);
   bool isXLog() const;
   bool isYLog() const;
-  void setXAxisInterpretation( bool isXAxisInterpreted, KstXAxisInterpretation xAxisInterpretation, KstXAxisDisplay xAxisDisplay);
-  void getXAxisInterpretation( bool& isXAxisInterpreted, KstXAxisInterpretation& xAxisInterpretation, KstXAxisDisplay& xAxisDisplay) const;
+  void setXAxisInterpretation( bool isXAxisInterpreted, KstAxisInterpretation xAxisInterpretation, KstAxisDisplay xAxisDisplay, bool xAxisTimezoneLocal, double xAxisTimezoneHrs);
+  void getXAxisInterpretation( bool& isXAxisInterpreted, KstAxisInterpretation& xAxisInterpretation, KstAxisDisplay& xAxisDisplay, bool& xAxisTimezoneLocal, double& xAxisTimezoneHrs) const;
+  void setYAxisInterpretation( bool isYAxisInterpreted, KstAxisInterpretation yAxisInterpretation, KstAxisDisplay yAxisDisplay, bool yAxisTimezoneLocal, double yAxisTimezoneHrs);
+  void getYAxisInterpretation( bool& isYAxisInterpreted, KstAxisInterpretation& yAxisInterpretation, KstAxisDisplay& yAxisDisplay, bool& yAxisTimezoneLocal, double& yAxisTimezoneHrs ) const;
   void getScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
   void getLScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
 
@@ -151,15 +158,17 @@ public:
   void toggleTied();
   void setTied(bool is_tied);
 
+  virtual QString menuTitle() const;
+
   /** Labels */
-  KstLabel *XLabel;
-  KstLabel *YLabel;
-  KstLabel *TopLabel;
-  KstLabel *XTickLabel;
-  KstLabel *YTickLabel;
-  KstLabel *XFullTickLabel;
-  KstLabel *YFullTickLabel;
-  KstLegend *Legend;
+  KstLabelPtr XLabel;
+  KstLabelPtr YLabel;
+  KstLabelPtr TopLabel;
+  KstLabelPtr XTickLabel;
+  KstLabelPtr YTickLabel;
+  KstLabelPtr XFullTickLabel;
+  KstLabelPtr YFullTickLabel;
+  KstLegendPtr Legend;
 
   double _width;
   double _height;
@@ -170,6 +179,7 @@ public:
   KstLabelList _labelList;
 
   KstBaseCurveList Curves;
+  KstImageList _images;
 
   void GenerateDefaultLabels();
 
@@ -184,6 +194,7 @@ public:
   virtual bool mouseHandler() const;
   virtual void mouseMoveEvent(QWidget *view, QMouseEvent *e);
   virtual void mousePressEvent(QWidget *view, QMouseEvent *e);
+  virtual void mouseDoubleClickEvent(QWidget *view, QMouseEvent *e);
   virtual void mouseReleaseEvent(QWidget *view, QMouseEvent *e);
   virtual void keyPressEvent(QWidget *view, QKeyEvent *e);
   virtual void keyReleaseEvent(QWidget *view, QKeyEvent *e);
@@ -219,9 +230,7 @@ public:
   void yZoomNormal(KstViewWidget *);
   void zoomSpikeInsensitiveMax(KstViewWidget *);
 
-  //
   // plot Markers
-  //
   bool setPlotMarker(const double xValue, bool isRisng = false, bool isFalling = false);
   bool removePlotMarker(const double xValue);
   void setPlotMarkerList(const KstMarkerList& newMarkers);
@@ -229,13 +238,11 @@ public:
   const KstMarkerList& plotMarkers();
   bool nextMarker(const double currentPosition, double& marker);
   bool prevMarker(const double currentPosition, double& marker);
-  void moveToNextMarker(KstViewWidget *);
-  void moveToPrevMarker(KstViewWidget *);
+  void moveToNextMarker(KstViewWidget*);
+  void moveToPrevMarker(KstViewWidget*);
   void moveSelfToCenter(double center);
 
-  //
   // curve related plot marker functions
-  //
   void setCurveToMarkers(KstBaseCurvePtr curve, bool risingDetect, bool fallingDetect);
   bool hasCurveToMarkers() const;
   void removeCurveToMarkers();
@@ -243,22 +250,18 @@ public:
   bool curveToMarkersRisingDetect() const;
   bool curveToMarkersFallingDetect() const;
 
-  void setXOffsetMode( bool xOffsetMode ) { _xOffsetMode = xOffsetMode; }
-  void setYOffsetMode( bool yOffsetMode ) { _yOffsetMode = yOffsetMode; }
-  bool xOffsetMode( ) const { return _xOffsetMode; }
-  bool yOffsetMode( ) const { return _yOffsetMode; }
-  
-  //
+  void setXOffsetMode(bool xOffsetMode) { _xOffsetMode = xOffsetMode; }
+  void setYOffsetMode(bool yOffsetMode) { _yOffsetMode = yOffsetMode; }
+  bool xOffsetMode() const { return _xOffsetMode; }
+  bool yOffsetMode() const { return _yOffsetMode; }
+
   // images
-  //
   void addImage(KstImagePtr inImage, bool set_dirty = true);
   void removeImage(KstImagePtr inImage, bool set_dirty = true);
   void removeAllImages(bool set_dirty = true);
   bool hasImage(KstImagePtr inImage) const;
 
-  //
   // plot grid lines
-  //
   void setGridLines(bool xMajor, bool yMajor, bool xMinor, bool yMinor,
                     const QColor& majorColor, const QColor& minorColor,
                     bool majorGridColorDefault, bool minorGridColorDefault);
@@ -289,17 +292,20 @@ public slots:
   void copy();
 
   virtual void edit();
+  virtual void copyObject();
+  virtual void copyObjectQuietly(KstViewObject& parent, const QString& name = QString::null) const;
   void draw(); // draw into back buffer
   void draw(QPainter &p, KstPaintType type, double resolutionEnhancement = 1); // This actually does the drawing
   virtual void paint(KstPaintType type, QPainter& p);
   void editCurve(int id);
+  void editObject(int id);
+  void editVector(int id);
   void editImage(int id);
   void matchAxis(int id);
   void fitCurve(int id);
   void filterCurve(int id);
   void removeCurve(int id);
   void removeImage(int id);
-  void setDirty();
 
 protected slots:
   void menuMoveUp();
@@ -327,16 +333,23 @@ protected:
   virtual KstViewObjectFactoryMethod factory() const;
 
 private:
+  void updateDirtyFromLabels();
+
   template<class T> void updateTiedPlots(void (Kst2DPlot::*method)(T), T arg);
   template<class T> void updateTiedPlots(bool (Kst2DPlot::*method)(T), T arg);
 
-  void updateMousePos(QPoint pos);
+  void setCursorPos(QWidget *view);
+  void unsetCursorPos(QWidget *view);
+  void drawCursorPos(QWidget *view);
+  void drawCursorPos(QPainter& p);
+  void updateMousePos(const QPoint& pos);
+  bool getNearestDataPoint(const QPoint& pos, double& newxpos, double& newypos, QString& name, double& xmin, double &xmax, double& ymin, double& ymax);
   void highlightNearestDataPoint(bool repaint, QWidget *view, const QPoint& pos);
   void updateTieBox(QPainter&);
   int labelNumber(QMouseEvent *e);
   bool legendUnder(QMouseEvent *e);
   KstMouseModeType globalZoomType() const;
-  void setCursorForMode(QWidget *view, KstMouseModeType mode);
+  void setCursorForMode(QWidget *view, KstMouseModeType mode, const QPoint& pos);
   void setCursorForMode(QWidget *view); // gets the mode from the mouse
   void zoomRectUpdate(QWidget *view, KstMouseModeType t, int x, int y);
   inline void commonConstructor(const QString& in_tag,
@@ -353,22 +366,27 @@ private:
                          TickParameters &tpx,  TickParameters &tpy,
                          QPainter& p, bool& bOffsetX, bool& bOffsetY);
   void setTicks(double& tick, double& org,
-                double max, double min, bool is_log, bool isX);
-  double convertTimeValueToJD(double valueIn);
-  double convertTimeDiffValueToDays(double diffIn);
-  void convertJDToDateString(QString& label, uint& length, KstXAxisDisplay display, double dJD);
-  void convertTimeValueToString(QString& label, uint& length, double z, bool isLog);
-  void convertDiffTimevalueToString(QString& label, double zbase, double zvalue, bool isLog, double scale);
-  void genAxisTickLabelFullPrecision(QString& label, uint& length, double z, bool isLog, bool isX);
-  void genAxisTickLabelDifference(QString& label, double zbase, double zvalue, bool isLog, bool isX, double scale);
-  void genAxisTickLabels(QPainter& p, TickParameters &tp,
-			 double Min, double Max,
-			 bool bLog, KstLabel* Label, bool isX);
+                double max, double min, bool is_log, bool isX, int base);
+  double convertTimeValueToJD(KstAxisInterpretation axisInterpretation, double valueIn);
+  double convertTimeDiffValueToDays(KstAxisInterpretation axisInterpretation, double diffIn);
+  void convertJDToDateString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool axisTimezoneLocal, double axisTimezoneHrs, QString& label, uint& length, double dJD);
+  void convertTimeValueToString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool axisTimezoneLocal, double axisTimezoneHrs, QString& label, uint& length, double z, bool isLog);
+  void convertDiffTimevalueToString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, double& zdiff, double zbase, double zvalue, bool isLog, double scale);
+  void genAxisTickLabelFullPrecision(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool axisTimezoneLocal, double axisTimezoneHrs, QString& label, uint& length, double z, bool isLog, bool isInterpreted);
+  void genAxisTickLabelDifference(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, double zbase, double zvalue, bool isLog, bool isInterpreted, double scale);
+  void getPrefixUnitsScale(bool isInterpreted, KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool bLog, double Min, double Max, double& range, double& scale, int& base, QString& strPrefix, QString& strUnits);
+  void genOffsetLabel(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, double zbase, double zvalue, double Min, double Max, bool isLog, bool isInterpreted);
+  void genAxisTickLabels(QPainter& p, TickParameters &tp, double Min, double Max, bool bLog, KstLabelPtr Label,
+                         KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, 
+                         bool axisTimezoneLocal, double axisTimezoneHours, 
+                         bool isX, bool isInterpreted, bool isOffsetMode);
 
   // range and domain of plot: not plot dimentions
   double XMin, XMax, YMin, YMax;
   double _copy_x, _copy_y;
-
+  double _cursor_x, _cursor_y;
+  bool _cursorOffset;
+  
   // grid lines
   QColor _majorGridColor;
   QColor _minorGridColor;
@@ -378,13 +396,11 @@ private:
   bool _yMinorGrid : 1;
   bool _majorGridColorDefault : 1;
   bool _minorGridColorDefault : 1;
-  
-  bool _xLog : 1, _yLog : 1;
+
+  bool _xLog : 1;
+  bool _yLog : 1;
   bool _isTied : 1;
-  bool _dirty : 1;
   bool _zoomPaused : 1;
-  bool _highlighting : 1; // lock to prevent reentrancy on data mode
-                          // (shouldn't this be avoidable?)
   bool _curveToMarkersRisingDetect : 1;
   bool _curveToMarkersFallingDetect : 1;
   bool _isLogLast : 1;
@@ -394,10 +410,12 @@ private:
 
   int _draggableLabel : 15; // I think this should be enough
 
-  bool _isXAxisInterpreted;
-  KstXAxisInterpretation _xAxisInterpretation;
-  KstXAxisDisplay _xAxisDisplay;
-  
+  bool _isXAxisInterpreted, _isYAxisInterpreted;
+  KstAxisInterpretation _xAxisInterpretation, _yAxisInterpretation;
+  KstAxisDisplay _xAxisDisplay, _yAxisDisplay;
+  bool _xAxisTimezoneLocal, _yAxisTimezoneLocal;
+  double _xAxisTimezoneHrs, _yAxisTimezoneHrs;
+
   KstScaleModeType _xScaleMode, _yScaleMode;
 
   QPtrList<KstPlotScale> _plotScaleList;
@@ -413,19 +431,17 @@ private:
   KstMouse _mouse;
   QPoint _draggablePoint;
 
-  QMap<int, QString> _curveEditMap, _curveFitMap, _curveRemoveMap, _imageEditMap, _imageRemoveMap;
+  QMap<int, QString> _curveEditMap, _curveFitMap, _curveRemoveMap, _imageEditMap, _imageRemoveMap, _objectEditMap;
   QMap<int, QGuardedPtr<Kst2DPlot> > _plotMap;
 
   KstBackBuffer _buffer;
   QGuardedPtr<KstViewWidget> _menuView;
+  QGuardedPtr<KstViewWidget> _layoutMenuView;
   QSize _oldSize;
   QRect _oldAlignment;
   double _m_X, _b_X, _m_Y, _b_Y;
   double _tickYLast, _stLast;
   int _autoTickYLast;
-  
-  //the list of more than one image
-  KstImageList _images;
 
   // plot markers. This needs to remain sorted
   KstMarkerList _plotMarkers;
@@ -447,15 +463,15 @@ private:
   // helper functions for draw(...)
   void plotLabels(QPainter& p, int x_px, int y_px, double xleft_bdr_px, double ytop_bdr_px);
   void plotAxes(QPainter& p, QRect& plotRegion,
-		TickParameters tpx, 
-		double xleft_bdr_px, double xright_bdr_px,
-		double x_orig_px, double xtick_px,
-		double xtick_len_px, int x_px,
-		TickParameters tpy, 
-		double ytop_bdr_px, double ybot_bdr_px,
-		double y_orig_px, double ytick_px,
-		double ytick_len_px, int y_px,
-		bool offsetX, bool offsetY);
+      TickParameters tpx, 
+      double xleft_bdr_px, double xright_bdr_px,
+      double x_orig_px, double xtick_px,
+      double xtick_len_px, int x_px,
+      TickParameters tpy, 
+      double ytop_bdr_px, double ybot_bdr_px,
+      double y_orig_px, double ytick_px,
+      double ytick_len_px, int y_px,
+      bool offsetX, bool offsetY);
   void plotGridLines(QPainter& p,
       double XTick, double xleft_bdr_px, double xright_bdr_px,
       double x_orig_px, double xtick_px, double xtick_len_px, int x_px,
