@@ -17,7 +17,8 @@
 
 #include "kstbackbuffer.h"
 
-KstBackBuffer::KstBackBuffer() {
+KstBackBuffer::KstBackBuffer(int depth)
+: _buffer(0, 0, depth, QPixmap::MemoryOptim) {
 }
 
 
