@@ -20,33 +20,35 @@
 
 #include "changefiledialog.h"
 
-#include <qlistbox.h>
-#include <qlabel.h>
-
 class KstChangeFileDialogI : public KstChangeFileDialog {
-    Q_OBJECT
-public:
+  Q_OBJECT
+  public:
     KstChangeFileDialogI(QWidget* parent = 0,
                          const char* name = 0,
                          bool modal = FALSE, WFlags fl = 0 );
     virtual ~KstChangeFileDialogI();
 
-public slots:
-  /** update the entries in changeFileDialog to represent current vectors */
-  void updateChangeFileDialog();
+  public slots:
+    /** update the entries in changeFileDialog to represent current vectors */
+    void updateChangeFileDialog();
 
-  /** calls updateChangeFileDialog(), then shows and raises changeFileDialog */
-  void showChangeFileDialog();
+    /** calls updateChangeFileDialog(), then shows and raises the dialog */
+    void showChangeFileDialog();
 
-  /** select all in selection box */
-  void selectAll();
+    /** select all in selection box */
+    void selectAll();
 
-private slots:
-  void applyFileChange();
+    void allFromFile();
 
-signals:
-  /** signal that vectors have changed */
-  void docChanged();
+  private slots:
+    void applyFileChange();
+
+    void updateSelection(const QString&);
+
+  signals:
+    /** signal that vectors have changed */
+    void docChanged();
 };
 
 #endif
+// vim: ts=2 sw=2 et
