@@ -17,6 +17,7 @@
  ***************************************************************************/
 
 // include files for Qt
+#include <qcheckbox.h>
 
 // include files for KDE
 #include <kcombobox.h>
@@ -28,22 +29,22 @@
 #include "curveplacementwidget.h"
 #include "kstcurvedialog_i.h"
 #include "kstviewwindow.h"
+#include "kstuinames.h"
 #include "vectorselector.h"
 
 #define DIALOGTYPE KstCurveDialogI
 #define DTYPE "Curve"
 #include "dataobjectdialog.h"
 
-static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
+const QString& KstCurveDialogI::defaultTag = KGlobal::staticQString("<Auto Name>");
 
-KstCurveDialogI *KstCurveDialogI::_inst = 0L;
-static KStaticDeleter<KstCurveDialogI> _cuInst;
+QGuardedPtr<KstCurveDialogI> KstCurveDialogI::_inst;
 
 KstCurveDialogI *KstCurveDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _cuInst.setObject(new KstCurveDialogI);
+    _inst = new KstCurveDialogI(KstApp::inst());
   }
-return _inst;
+  return _inst;
 }
 
 
@@ -60,9 +61,20 @@ KstCurveDialogI::KstCurveDialogI(QWidget* parent,
           this, SIGNAL(modified()));
   connect(_yError, SIGNAL(newVectorCreated(const QString&)),
           this, SIGNAL(modified()));
+  connect(_xMinusError, SIGNAL(newVectorCreated(const QString&)),
+          this, SIGNAL(modified()));
+  connect(_yMinusError, SIGNAL(newVectorCreated(const QString&)),
+          this, SIGNAL(modified()));
+  connect(_checkBoxXMinusSameAsPlus, SIGNAL(toggled(bool)), this, SLOT(toggledXErrorSame(bool)));
+  connect(_checkBoxYMinusSameAsPlus, SIGNAL(toggled(bool)), this, SLOT(toggledYErrorSame(bool)));
 
   _xError->provideNoneVector(true);
   _yError->provideNoneVector(true);
+  _xMinusError->provideNoneVector(true);
+  _yMinusError->provideNoneVector(true);
+
+  _checkBoxXMinusSameAsPlus->setChecked(true);
+  _checkBoxYMinusSameAsPlus->setChecked(true);
 }
 
 KstCurveDialogI::~KstCurveDialogI() {
@@ -80,46 +92,65 @@ void KstCurveDialogI::updateWindow() {
   _curvePlacement->update();
 }
 
+void KstCurveDialogI::toggledXErrorSame(bool on) {
+  textLabelXMinus->setEnabled(!on);
+  _xMinusError->setEnabled(!on);
+}
+
+void KstCurveDialogI::toggledYErrorSame(bool on) {
+  textLabelYMinus->setEnabled(!on);
+  _yMinusError->setEnabled(!on);
+}
+
 void KstCurveDialogI::_fillFieldsForEdit() {
-  if ( DP == 0L ) {
-    return; // shouldn't be needed
+  if (DP != 0L) {
+    DP->readLock();
+
+    _tagName->setText(DP->tagName());
+
+    _xVector->setSelection(DP->xVTag());
+    _yVector->setSelection(DP->yVTag());
+    _xError->setSelection(DP->xETag());
+    _yError->setSelection(DP->yETag());
+    _xMinusError->setSelection(DP->xEMinusTag());
+    _yMinusError->setSelection(DP->yEMinusTag());
+
+    if (DP->xEMinusTag() == DP->xETag()) {
+      _checkBoxXMinusSameAsPlus->setChecked(true);
+    } else {
+      _checkBoxXMinusSameAsPlus->setChecked(false);
+    }
+    if (DP->yEMinusTag() == DP->yETag()) {
+      _checkBoxYMinusSameAsPlus->setChecked(true);
+    }  else {
+      _checkBoxYMinusSameAsPlus->setChecked(false);
+    }
+
+    _curveAppearance->setValue(DP->hasLines(), DP->hasPoints(),
+        DP->hasBars(), DP->color(),
+        DP->Point.type(), DP->lineWidth(),
+        DP->lineStyle(), DP->barStyle(), DP->pointDensity());
+
+    _checkBoxIgnoreAutoscale->setChecked(DP->ignoreAutoScale());
+
+    DP->readUnlock();
+
+    _curvePlacement->hide();
+
+    adjustSize();
+    resize(minimumSizeHint());
+    setFixedHeight(height());
   }
-
-  DP->readLock();
-
-  _tagName->setText(DP->tagName());
-
-  _xVector->setSelection(DP->xVTag());
-  _yVector->setSelection(DP->yVTag());
-  _xError->setSelection(DP->xETag());
-  _yError->setSelection(DP->yETag());
-
-  //_curveAppearance->set(DP);
-  _curveAppearance->setValue(DP->hasLines(), DP->hasPoints(), 
-			     DP->hasBars(), DP->color(),
-			     DP->Point.type(), DP->lineWidth(), 
-			     DP->lineStyle(), DP->barStyle());
-  
-  DP->readUnlock();
-
-  _curvePlacement->hide();
-
-  adjustSize();
-  resize(minimumSizeHint());
-  setFixedHeight(height());
 }
 
 void KstCurveDialogI::_fillFieldsForNew() {
   KstVCurveList curves =
     kstObjectSubList<KstDataObject, KstVCurve>(KST::dataObjectList);
 
-  /* set tag name */
   _tagName->setText(defaultTag);
-
-  /* set the curve placement window  */
   _curvePlacement->update();
 
-  //for some reason the lower widget needs to be shown first to prevent overlapping?
+  // for some reason the lower widget needs to be shown first to prevent overlapping?
   _curveAppearance->hide();
   _curvePlacement->show();
   _curveAppearance->show();
@@ -136,6 +167,8 @@ void KstCurveDialogI::update() {
   _yVector->update();
   _xError->update();
   _yError->update();
+  _xMinusError->update();
+  _yMinusError->update();
 }
 
 bool KstCurveDialogI::new_I() {
@@ -147,36 +180,54 @@ bool KstCurveDialogI::new_I() {
   }
 
   KstVectorList::Iterator VX, VY;
-  KstVectorList::Iterator EX, EY;
+  KstVectorList::Iterator EX, EY, EXMinus, EYMinus;
 
-  /* find VX and VY */
+  // find VX and VY
   KST::vectorList.lock().readLock();
   VX = KST::vectorList.findTag(_xVector->selectedVector());
   VY = KST::vectorList.findTag(_yVector->selectedVector());
   EX = KST::vectorList.findTag(_xError->selectedVector());
   EY = KST::vectorList.findTag(_yError->selectedVector());
+  if(_checkBoxXMinusSameAsPlus->isChecked()) {
+    EXMinus = EX;
+  } else {
+    EXMinus = KST::vectorList.findTag(_xMinusError->selectedVector());
+  }
+  if(_checkBoxYMinusSameAsPlus->isChecked()) {
+    EYMinus = EY;
+  } else {
+    EYMinus = KST::vectorList.findTag(_yMinusError->selectedVector());
+  }
   if (VX == KST::vectorList.end() || VY == KST::vectorList.end()) {
     kdFatal() << "Bug in kst: the XVector field in plotDialog refers to "
-              << "a non existant vector...." << endl;
+              << "a non existent vector...." << endl;
   }
 
   // readlock the vectors, bercause when we update the curve, they get read
   KstReadLocker rl1(*VX), rl2(*VY);
   bool haveEx = (EX != KST::vectorList.end());
   bool haveEy = (EY != KST::vectorList.end());
+  bool haveExMinus = (EXMinus != KST::vectorList.end());
+  bool haveEyMinus = (EYMinus != KST::vectorList.end());
   if (haveEx) {
     (*EX)->readLock();
   }
   if (haveEy) {
     (*EY)->readLock();
   }
+  if (haveExMinus) {
+    (*EXMinus)->readLock();
+  }
+  if (haveEyMinus) {
+    (*EYMinus)->readLock();
+  }
 
   QString tag_name = _tagName->text();
   if (tag_name == defaultTag) {
     tag_name = KST::suggestCurveName((*VY)->tagName());
   }
 
-  /* verify that the curve name is unique */
+  // verify that the curve name is unique
   if (KST::dataTagNameNotUnique(tag_name)) {
     _tagName->setFocus();
     KST::vectorList.lock().readUnlock();
@@ -186,13 +237,21 @@ bool KstCurveDialogI::new_I() {
     if (haveEy) {
       (*EY)->readUnlock();
     }
+    if (haveExMinus) {
+      (*EXMinus)->readUnlock();
+    }
+    if (haveEyMinus) {
+      (*EYMinus)->readUnlock();
+    }
     return false;
   }
 
-  /* create the curve */
+  // create the curve
   KstVCurvePtr curve = new KstVCurve(tag_name, *VX, *VY,
                         haveEx ? *EX : KstVectorPtr(0L),
                         haveEy ? *EY : KstVectorPtr(0L),
+                        haveExMinus ? *EXMinus : KstVectorPtr(0L),
+                        haveEyMinus ? *EYMinus : KstVectorPtr(0L),
                         _curveAppearance->color());
   KST::vectorList.lock().readUnlock();
 
@@ -203,8 +262,10 @@ bool KstCurveDialogI::new_I() {
   curve->setLineStyle(_curveAppearance->lineStyle());
   curve->Point.setType(_curveAppearance->pointType());
   curve->setBarStyle(_curveAppearance->barStyle());
-  if (_curvePlacement->existingPlot() || _curvePlacement->newPlot()) {
+  curve->setPointDensity(_curveAppearance->pointDensity());
+  curve->setIgnoreAutoScale(_checkBoxIgnoreAutoscale->isChecked());
 
+  if (_curvePlacement->existingPlot() || _curvePlacement->newPlot()) {
     KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_curvePlacement->_plotWindow->currentText()));
     if (!w) {
       QString n = KstApp::inst()->newWindow(KST::suggestWinName());
@@ -213,16 +274,15 @@ bool KstCurveDialogI::new_I() {
     if (w) {
       Kst2DPlotPtr plot;
       if (_curvePlacement->existingPlot()) {
-        /* assign curve to plot */
+        // assign curve to plot
         plot = kst_cast<Kst2DPlot>(w->view()->findChild(_curvePlacement->plotName()));
         if (plot) {
           plot->addCurve(curve.data());
-          w->view()->paint(P_DATA);
         }
       }
 
       if (_curvePlacement->newPlot()) {
-        /* assign curve to plot */
+        // assign curve to plot
         QString name = w->createPlot<Kst2DPlot>(KST::suggestPlotName());
         plot = kst_cast<Kst2DPlot>(w->view()->findChild(name));
         if (_curvePlacement->reGrid()) {
@@ -233,7 +293,6 @@ bool KstCurveDialogI::new_I() {
           _curvePlacement->setCurrentPlot(plot->tagName());
           plot->addCurve(curve.data());
           plot->GenerateDefaultLabels();
-          w->view()->paint(P_DATA);
         }
       }
     }
@@ -245,6 +304,12 @@ bool KstCurveDialogI::new_I() {
   if (haveEy) {
     (*EY)->readUnlock();
   }
+  if (haveExMinus) {
+    (*EXMinus)->readUnlock();
+  }
+  if (haveEyMinus) {
+    (*EYMinus)->readUnlock();
+  }
 
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(curve.data());
@@ -260,7 +325,7 @@ bool KstCurveDialogI::edit_I() {
 
   DP->writeLock();
 
-  /* verify that the curve name is unique */
+  // verify that the curve name is unique
   if (_tagName->text() != DP->tagName() && KST::dataTagNameNotUnique(_tagName->text())) {
     _tagName->setFocus();
   } else {
@@ -280,9 +345,20 @@ bool KstCurveDialogI::edit_I() {
 
       it = KST::vectorList.findTag(_xError->selectedVector());
       DP->setXError(*it);
-
+      if (_checkBoxXMinusSameAsPlus->isChecked()) {
+        DP->setXMinusError(*it);
+      } else {
+        it = KST::vectorList.findTag(_xMinusError->selectedVector());
+        DP->setXMinusError(*it);    
+      }
       it = KST::vectorList.findTag(_yError->selectedVector());
       DP->setYError(*it);
+      if (_checkBoxYMinusSameAsPlus->isChecked()) {
+        DP->setYMinusError(*it);
+      } else {
+        it = KST::vectorList.findTag(_yMinusError->selectedVector());
+        DP->setYMinusError(*it);
+      }
     }
 
     DP->setColor(_curveAppearance->color());
@@ -293,11 +369,12 @@ bool KstCurveDialogI::edit_I() {
     DP->setLineStyle(_curveAppearance->lineStyle());
     DP->Point.setType(_curveAppearance->pointType());
     DP->setBarStyle(_curveAppearance->barStyle());
-    DP->update(-1);
+    DP->setPointDensity(_curveAppearance->pointDensity());
+    DP->setIgnoreAutoScale(_checkBoxIgnoreAutoscale->isChecked());
 
     rc = true;
   }
-  
+
   DP->writeUnlock();
 
   if (rc) {
