@@ -20,17 +20,20 @@
 #include <assert.h>
 
 #include <kdebug.h>
+#include <kio/netaccess.h>
 #include <klibloader.h>
 #include <klocale.h>
 #include <kservicetype.h>
 
 #include <qdeepcopy.h>
 #include <qfile.h>
+#include <qfileinfo.h>
 #include <qstylesheet.h>
 
 #include "kst.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
+#include "kstscalar.h"
 #include "stdinsource.h"
 
 
@@ -67,6 +70,30 @@ namespace KST {
         return 0L;
       }
 
+      KstDataSource *create(KConfig *cfg, const QString& filename, const QString& type, const QDomElement& e) const {
+        KstDataSource *(*sym)(KConfig*, const QString&, const QString&, const QDomElement&) = (KstDataSource*(*)(KConfig*, const QString&, const QString&, const QDomElement&))symbol("load");
+        if (sym) {
+          //kdDebug() << "Trying to create " << filename << " type=" << type << " with " << service->property("Name").toString() << endl;
+          KstDataSource *ds = (sym)(cfg, filename, type, e);
+          if (ds) {
+            ds->_source = service->property("Name").toString();
+          }
+          //kdDebug() << (ds ? "SUCCESS" : "FAILED") << endl;
+          return ds;
+        } else {
+          KstDataSource *(*sym)(KConfig*, const QString&, const QString&) = (KstDataSource*(*)(KConfig*, const QString&, const QString&))symbol("create");
+          if (sym) {
+            KstDataSource *ds = (sym)(cfg, filename, type);
+            if (ds) {
+              ds->_source = service->property("Name").toString();
+            }
+            return ds;
+          }
+        }
+
+        return 0L;
+      }
+
       QStringList fieldList(KConfig *cfg, const QString& filename, const QString& type = QString::null, QString *typeSuggestion = 0L, bool *complete = 0L) const {
         QStringList (*sym)(KConfig*, const QString&, const QString&, QString*, bool*) = (QStringList(*)(KConfig*, const QString&, const QString&, QString*, bool*))symbol("fieldList");
         if (sym) {
@@ -99,6 +126,16 @@ namespace KST {
           return rc;
         }
 
+        return 0;
+      }
+
+      bool supportsTime(KConfig *cfg, const QString& filename) const {
+        bool (*sym)(KConfig*, const QString&) = (bool(*)(KConfig*, const QString&))symbol("supportsTime");
+        if (sym) {
+          bool rc = (sym)(cfg, filename);
+          return rc;
+        }
+
         return false;
       }
 
@@ -116,6 +153,10 @@ namespace KST {
         return QStringList();
       }
 
+      bool hasConfigWidget() const {
+        return 0L != symbol("widget");
+      }
+
       KstDataSourceConfigWidget *configWidget(KConfig *cfg, const QString& filename) const {
         QWidget *(*sym)(const QString&) = (QWidget *(*)(const QString&))symbol("widget");
         if (sym) {
@@ -140,8 +181,6 @@ namespace KST {
           return 0L;
         }
 
-        // FIXME: might be a good idea to cache this per-symbol
-
         QCString s = QFile::encodeName(sym + "_" + _plugLib);
         if (_lib->hasSymbol(s)) {
           return _lib->symbol(s);
@@ -174,6 +213,7 @@ namespace KST {
 
 
 static KConfig *kConfigObject = 0L;
+static QMap<QString,QString> urlMap;
 void KstDataSource::setupOnStartup(KConfig *cfg) {
   kConfigObject = cfg;
 }
@@ -183,6 +223,54 @@ static KST::PluginInfoList pluginInfo;
 void KstDataSource::cleanupForExit() {
   pluginInfo.clear();
   kConfigObject = 0L;
+  for (QMap<QString,QString>::Iterator i = urlMap.begin(); i != urlMap.end(); ++i) {
+    QFile::remove(i.data());
+  }
+  urlMap.clear();
+}
+
+
+static QString obtainFile(const QString& source) {
+  KURL url;
+  
+  if (QFile::exists(source) && QFileInfo(source).isRelative()) {
+    url.setPath(source);
+  } else {
+    url = KURL::fromPathOrURL(source);
+  }
+
+  if (url.isLocalFile() || url.protocol().isEmpty()) {
+    return source;
+  }
+
+  if (urlMap.contains(source)) {
+    return urlMap[source];
+  }
+
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+  // FIXME: come up with a way to indicate the "widget" and fill it in here so
+  //        that KIO dialogs are associated with the proper window
+  if (!KIO::NetAccess::exists(url, true, 0L)) {
+#else
+  if (!KIO::NetAccess::exists(url, true)) {
+#endif
+    return QString::null;
+  }
+
+  QString tmpFile;
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+  // FIXME: come up with a way to indicate the "widget" and fill it in here so
+  //        that KIO dialogs are associated with the proper window
+  if (!KIO::NetAccess::download(url, tmpFile, 0L)) {
+#else
+  if (!KIO::NetAccess::download(url, tmpFile)) {
+#endif
+    return QString::null;
+  }
+
+  urlMap[source] = tmpFile;
+
+  return tmpFile;
 }
 
 
@@ -273,12 +361,12 @@ static QValueList<PluginSortContainer> bestPluginsForSource(const QString& filen
 }
 
 
-static KstDataSourcePtr findPluginFor(const QString& filename, const QString& type) {
+static KstDataSourcePtr findPluginFor(const QString& filename, const QString& type, const QDomElement& e = QDomElement()) {
 
   QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(filename, type);
 
   for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
-    KstDataSourcePtr plugin = (*i).plugin->create(kConfigObject, filename);
+    KstDataSourcePtr plugin = (*i).plugin->create(kConfigObject, filename, QString::null, e);
     if (plugin) {
       return plugin;
     }
@@ -293,21 +381,48 @@ KstDataSourcePtr KstDataSource::loadSource(const QString& filename, const QStrin
     return new KstStdinSource(kConfigObject);
   }
 
-  if (filename.isEmpty()) {
+  QString fn = obtainFile(filename);
+  if (fn.isEmpty()) {
     return 0L;
   }
 
-  return findPluginFor(filename, type);
+  return findPluginFor(fn, type);
 }
 
 
 KstDataSourceConfigWidget* KstDataSource::configWidget() const {
-  return configWidgetForSource(_filename, fileType());
+  KstDataSourceConfigWidget *w = configWidgetForSource(_filename, fileType());
+  if (w) {
+    // FIXME: what to do here?  This is ugly, but the method is const and we
+    //        can't put a const shared pointer in the config widget.  Fix for
+    //        Kst 2.0 by making this non-const?
+    w->_instance = const_cast<KstDataSource*>(this);
+  }
+  return w;
+}
+
+
+bool KstDataSource::pluginHasConfigWidget(const QString& plugin) {
+  if (pluginInfo.isEmpty()) {
+    scanPlugins();
+  }
+
+  KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
+
+  for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
+    if ((*it)->service->property("Name").toString() == plugin) {
+      return (*it)->hasConfigWidget();
+    }
+  }
+
+  return false;
 }
 
 
 KstDataSourceConfigWidget* KstDataSource::configWidgetForPlugin(const QString& plugin) {
-  scanPlugins();
+  if (pluginInfo.isEmpty()) {
+    scanPlugins();
+  }
 
   KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
 
@@ -322,13 +437,18 @@ KstDataSourceConfigWidget* KstDataSource::configWidgetForPlugin(const QString& p
 
 
 KstDataSourceConfigWidget* KstDataSource::configWidgetForSource(const QString& filename, const QString& type) {
-  if (filename.isEmpty() || filename == "stdin" || filename == "-") {
+  if (filename == "stdin" || filename == "-") {
     return 0L;
   }
 
-  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(filename, type);
+  QString fn = obtainFile(filename);
+  if (fn.isEmpty()) {
+    return 0L;
+  }
+
+  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(fn, type);
   for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
-    KstDataSourceConfigWidget *w = (*i).plugin->configWidget(kConfigObject, filename);
+    KstDataSourceConfigWidget *w = (*i).plugin->configWidget(kConfigObject, fn);
     // Don't iterate.
     return w;
   }
@@ -338,16 +458,36 @@ KstDataSourceConfigWidget* KstDataSource::configWidgetForSource(const QString& f
 }
 
 
+bool KstDataSource::supportsTime(const QString& filename, const QString& type) {
+  if (filename == "stdin" || filename == "-") {
+    return false;
+  }
+
+  QString fn = obtainFile(filename);
+  if (fn.isEmpty()) {
+    return false;
+  }
+
+  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(fn, type);
+  return (*bestPlugins.begin()).plugin->supportsTime(kConfigObject, fn);
+}
+
+
 QStringList KstDataSource::fieldListForSource(const QString& filename, const QString& type, QString *outType, bool *complete) {
-  if (filename.isEmpty() || filename == "stdin" || filename == "-") {
+  if (filename == "stdin" || filename == "-") {
     return QStringList();
   }
 
-  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(filename, type);
+  QString fn = obtainFile(filename);
+  if (fn.isEmpty()) {
+    return QStringList();
+  }
+
+  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(fn, type);
   QStringList rc;
   for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
     QString typeSuggestion;
-    rc = (*i).plugin->fieldList(kConfigObject, filename, QString::null, &typeSuggestion, complete);
+    rc = (*i).plugin->fieldList(kConfigObject, fn, QString::null, &typeSuggestion, complete);
     if (!rc.isEmpty()) {
       if (outType) {
         if (typeSuggestion.isEmpty()) {
@@ -372,7 +512,7 @@ KstDataSourcePtr KstDataSource::loadSource(QDomElement& e) {
     QDomElement e = n.toElement();
     if (!e.isNull()) {
       if (e.tagName() == "filename") {
-        filename = e.text();
+        filename = obtainFile(e.text());
       } else if (e.tagName() == "type") {
         type = e.text();
       }
@@ -388,14 +528,15 @@ KstDataSourcePtr KstDataSource::loadSource(QDomElement& e) {
     return new KstStdinSource(kConfigObject);
   }
 
-  return findPluginFor(filename, type);
+  return findPluginFor(filename, type, e);
 }
 
 
 class KstDataSourcePrivate {
   public:
-    KstDataSourcePrivate() {}
+    KstDataSourcePrivate() { _reusable = true; }
     ~KstDataSourcePrivate() {}
+    bool _reusable;
 };
 
 
@@ -403,8 +544,9 @@ KstDataSource::KstDataSource(KConfig *cfg, const QString& filename, const QStrin
 : KstObject(), _filename(filename), _cfg(cfg) {
   Q_UNUSED(type)
   _valid = false;
-  //d = new KstDataSourcePrivate;
+  d = new KstDataSourcePrivate;
   _numFramesScalar = new KstScalar(filename + i18n("-frames"));
+  // Don't set provider - this is always up-to-date
 }
 
 
@@ -413,7 +555,8 @@ KstDataSource::~KstDataSource() {
   KST::scalarList.remove(_numFramesScalar);
   KST::scalarList.lock().writeUnlock();
   _numFramesScalar = 0L;
-  //delete d;
+  delete d;
+  d = 0L;
 }
 
 
@@ -472,6 +615,12 @@ int KstDataSource::frameCount(const QString& field) const {
 
 
 QString KstDataSource::fileName() const {
+  // Look to see if it was a URL and save the URL instead
+  for (QMap<QString,QString>::ConstIterator i = urlMap.begin(); i != urlMap.end(); ++i) {
+    if (i.data() == _filename) {
+      return i.key();
+    }
+  }
   return _filename;
 }
 
@@ -487,7 +636,15 @@ QString KstDataSource::fileType() const {
 
 
 void KstDataSource::save(QTextStream &ts, const QString& indent) {
-  ts << indent << "<filename>" << QStyleSheet::escape(_filename) << "</filename>" << endl;
+  QString name = QStyleSheet::escape(_filename);
+  // Look to see if it was a URL and save the URL instead
+  for (QMap<QString,QString>::ConstIterator i = urlMap.begin(); i != urlMap.end(); ++i) {
+    if (i.data() == _filename) {
+      name = QStyleSheet::escape(i.key());
+      break;
+    }
+  }
+  ts << indent << "<filename>" << name << "</filename>" << endl;
   ts << indent << "<type>" << QStyleSheet::escape(fileType()) << "</type>" << endl;
 }
 
@@ -568,8 +725,8 @@ int KstDataSource::sampleForTime(const QDateTime& time, bool *ok) {
 
 
 
-int KstDataSource::sampleForTime(int milliseconds, bool *ok) {
-  Q_UNUSED(milliseconds)
+int KstDataSource::sampleForTime(int seconds, bool *ok) {
+  Q_UNUSED(seconds)
   if (ok) {
     *ok = false;
   }
@@ -597,9 +754,19 @@ int KstDataSource::relativeTimeForSample(int sample, bool *ok) {
 }
 
 
+bool KstDataSource::reusable() const {
+  return d->_reusable;
+}
+
+
+void KstDataSource::disableReuse() {
+  d->_reusable = false;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 KstDataSourceConfigWidget::KstDataSourceConfigWidget()
-: QWidget(0L), _cfg(0L) {
+: QWidget(0L), _cfg(0L), d(0L) {
 }
 
 
@@ -611,9 +778,25 @@ void KstDataSourceConfigWidget::save() {
 }
 
 
+void KstDataSourceConfigWidget::load() {
+}
+
+
 void KstDataSourceConfigWidget::setConfig(KConfig *cfg) {
   _cfg = cfg;
 }
 
+
+void KstDataSourceConfigWidget::setInstance(KstDataSourcePtr inst) {
+  _instance = inst;
+}
+
+
+KstDataSourcePtr KstDataSourceConfigWidget::instance() const {
+  return _instance;
+}
+
+
+
 #include "kstdatasource.moc"
 // vim: ts=2 sw=2 et
