@@ -19,45 +19,64 @@
 #define KSTDEBUG_H
 
 #include <qdatetime.h>
+#include <qguardedptr.h>
 #include <qobject.h>
 #include <qmutex.h>
 
 #include <kstaticdeleter.h>
+#include <ksttimers.h>
 
-class KstDebug : public QObject {
+#include "kst_export.h"
+
+// This class has to be threadsafe
+class KST_EXPORT KstDebug : public QObject {
   Q_OBJECT
   friend class KStaticDeleter<KstDebug>;
   public:
-    enum LogLevel { Notice = 0, Warning, Error, Debug, None = 9999 };
+    enum LogLevel { Unknown = 0, Notice = 1, Warning = 2, Error = 4, Debug = 8, None = 16384 };
     struct LogMessage {
       QDateTime	date;
       QString 	msg;
       LogLevel 	level;
     };
-    static KstDebug* self();
+    static KstDebug *self();
 
     void clear();
     void log(const QString& msg, LogLevel level = Notice);
     void setLimit(bool applyLimit, int limit);   
     QString text();
     void sendEmail();
-    
-    const QValueList<LogMessage>& messages() const { return _messages; }
+
+    int logLength() const;
+    QValueList<LogMessage> messages() const;
+    KstDebug::LogMessage message(unsigned n) const;
     QStringList dataSourcePlugins() const;
     QString label(LogLevel level) const;
 
-  signals: 
-    void logAdded();
+    int limit() const;
+
+#ifdef BENCHMARK
+    QMap<QString,int>& drawCounter() { return _drawCounter; }
+#endif
+
+  protected:
+    friend class KstApp;
+    void setHandler(QObject *handler);
 
   private:
     KstDebug();
     ~KstDebug();
-    
+
     static KstDebug *_self;
     QValueList<LogMessage> _messages;
-    bool  _applyLimit;
-    int 	_limit;
-    QMutex _lock;
+    bool _applyLimit;
+    int _limit;
+    mutable QMutex _lock;
+#ifdef BENCHMARK
+    // If this is ever public we can't do this
+    QMap<QString,int> _drawCounter;
+#endif
+    QGuardedPtr<QObject> _handler;
 };
 
 
