@@ -19,7 +19,7 @@
 
 #include "debugdialog.h"
 
-class KstLogTable;
+class KstLogWidget;
 
 class KstDebugDialogI : public DebugDialog {
   Q_OBJECT
@@ -29,19 +29,12 @@ class KstDebugDialogI : public DebugDialog {
     virtual ~KstDebugDialogI();
 
   public slots:
-    void messagesDebug();
-    void messagesWarning();
-    void messagesNotice();
-    void messagesOther();
-    void messagesError();
     void email();
     void clear();
     void show_I();
-    void logAdded();
 
   private:  
-    KstLogTable *_table;
-    bool _firstNotification;
+    KstLogWidget *_log;
 };
 
 #endif
