@@ -38,19 +38,18 @@
 #include "stringselector.h"
 #include "vectorselector.h"
 
-KstFilterDialogI *KstFilterDialogI::_inst = 0L;
-static KStaticDeleter<KstFilterDialogI> _plFilterInst;
+QGuardedPtr<KstFilterDialogI> KstFilterDialogI::_inst;
 
 KstFilterDialogI *KstFilterDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _plFilterInst.setObject(new KstFilterDialogI);
+    _inst = new KstFilterDialogI(KstApp::inst());
   }
 
   return _inst;
 }
 
 
-const QString newFilterPluginString = i18n("Auto Name");
+const QString newFilterPluginString = i18n("<Auto Name>");
 
 KstFilterDialogI::KstFilterDialogI(QWidget* parent, const char* name,
                              bool modal, WFlags fl)
@@ -89,8 +88,12 @@ void KstFilterDialogI::show_setCurve(const QString& curveName,
   _window   = window;
   _plotName = plotName;
   _curve    = curveName;
-  if (curves.findTag(curveName) != curves.end()) {
-    KstBaseCurvePtr curve = *curves.findTag(curveName);
+
+  // it should be impossible for the curve not to exist so this should
+  // always be true.  If it is false, we do not properly take care of it, 
+  // here and bad things will happen....
+  KstBaseCurvePtr curve = *curves.findTag(curveName);
+  if (curve) {
     curve->readLock();
     _xvector = curve->xVTag();
     _yvector = curve->yVTag();
@@ -125,7 +128,7 @@ void KstFilterDialogI::updatePluginList() {
       if (it.data()._name == previous) {
         newFocus = cnt;
       }
-      cnt++;
+      ++cnt;
     }
   }
 
@@ -165,13 +168,14 @@ void KstFilterDialogI::update() {
         }
       }
     }
+    pluginChanged(0);
   }
 }
 
 
 void KstFilterDialogI::fixupLayout() {
-  _frameWidget->resize(_frameWidget->sizeHint());
-  _pluginFrame->resize(_pluginFrame->sizeHint());
+  _frameWidget->updateGeometry();
+  _pluginFrame->updateGeometry();
   resize(sizeHint());
   setMinimumSize(size());
   updateGeometry();
@@ -288,70 +292,34 @@ bool KstFilterDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
       if (tagName == newFilterPluginString) {
         tagName = plugin->tagName();
       }
-      li->setText(tagName + "-" + (*it)._name);
+      li->setText(tagName); // this will be the name of the output vec
     }
 
-    QString nt = li->text();
+    QString nt = KST::suggestVectorName(li->text());
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      if (!KST::vectorTagNameNotUnique(nt, false)) {
-        KstVectorPtr v;
-        if (plugin->outputVectors().contains((*it)._name)) {
-          v = plugin->outputVectors()[(*it)._name];
-        } else {
-          v = new KstVector;
-          v->writeLock();
-          plugin->outputVectors().insert((*it)._name, v);
-          KST::addVectorToList(v);
-        }
-        v->setTagName(nt);
-        v->setProvider(plugin.data());
-      } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
-        while (KST::vectorTagNameNotUnique(nt, false)) {
-          nt += "'";
-        }
-        KstVectorPtr v;
-        if (plugin->outputVectors().contains((*it)._name)) {
-          v = plugin->outputVectors()[(*it)._name];
-        } else {
-          v = new KstVector;
-          v->writeLock();
-          plugin->outputVectors().insert((*it)._name, v);
-          KST::addVectorToList(v);
-        }
-        v->setTagName(nt);
-        v->setProvider(plugin.data());
+      KstVectorPtr v = plugin->outputVectors()[(*it)._name];
+      if (!v) {
+        v = new KstVector;
+        v->writeLock();
+        plugin->outputVectors().insert((*it)._name, v);
+        KST::addVectorToList(v);
       }
+      v->setTagName(nt);
+      v->setProvider(plugin.data());
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      if (!KST::vectorTagNameNotUnique(nt, false)) {
-        KstScalarPtr s = plugin->outputScalars()[(*it)._name];
-        s->writeLock();
-        s->setTagName(nt);
-      } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
-        while (KST::vectorTagNameNotUnique(nt, false)) {
-          nt += "'";
-        }
-        KstScalarPtr s = plugin->outputScalars()[(*it)._name];
-        s->writeLock();
-        s->setTagName(nt);
-      }
+      KstScalarPtr s = plugin->outputScalars()[(*it)._name];
+      s->writeLock();
+      s->setTagName(nt);
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      if (!KST::vectorTagNameNotUnique(nt, false)) {
         KstStringPtr s = plugin->outputStrings()[(*it)._name];
         s->writeLock();
         s->setTagName(nt);
-      } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
-        while (KST::vectorTagNameNotUnique(nt, false)) {
-          nt += "'";
-        }
-        KstStringPtr s = plugin->outputStrings()[(*it)._name];
-        s->writeLock();
-        s->setTagName(nt);
-      }
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       // Nothing
     }
   }
 
+
   return true;
 }
 
@@ -363,26 +331,20 @@ bool KstFilterDialogI::createCurve(KstPluginPtr plugin) {
   bool rc = false;
 
   // Generate new fit name
-  QString c_name = "F" + QString::number(curves.count()+1) + "-" + _curve;
-  while (curves.findTag(c_name) != curves.end()) {
-    c_name += "'";
-  }
-  while (KST::vectorList.findTag(c_name) != KST::vectorList.end()) {
-    c_name += "'";
-  }
+  QString c_name = KST::suggestCurveName(plugin->tagName(), true);
 
   KstVectorList::Iterator it = KST::vectorList.findTag(_xvector);
   if (it != KST::vectorList.end()) {
     xVector = *it;
   }
 
-  if (plugin->outputVectors().find(plugin->plugin()->data()._filterOutputVector) != plugin->outputVectors().end()) {
+  if (plugin->outputVectors().contains(plugin->plugin()->data()._filterOutputVector)) {
     yVector = plugin->outputVectors()[plugin->plugin()->data()._filterOutputVector];
   }
 
   if (xVector && yVector) {
-    plugin->update(-1);
-    KstVCurvePtr fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
+    plugin->setDirty();
+    KstVCurvePtr fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
 
     fit->setHasPoints(_curveAppearance->showPoints());
     fit->setHasLines(_curveAppearance->showLines());
@@ -391,6 +353,7 @@ bool KstFilterDialogI::createCurve(KstPluginPtr plugin) {
     fit->setLineStyle(_curveAppearance->lineStyle());
     fit->Point.setType(_curveAppearance->pointType());
     fit->setBarStyle(_curveAppearance->barStyle());
+    fit->setPointDensity(_curveAppearance->pointDensity());
 
     //
     // add the curve to the plot...
@@ -400,7 +363,7 @@ bool KstFilterDialogI::createCurve(KstPluginPtr plugin) {
       Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_plotName));
       if (plot) {
         plot->addCurve(fit.data());
-        w->view()->paint(P_DATA);
+        w->view()->paint(P_PAINT);
       }
     }
 
@@ -430,35 +393,30 @@ bool KstFilterDialogI::new_I() {
       if (pPtr) {
         KstPluginPtr plugin = new KstPlugin;
         plugin->writeLock();
+        plugin->setDirty();
         if (saveInputs(plugin, pPtr)) {
           plugin->setPlugin(pPtr);
 
           if (tagName == newFilterPluginString) {
-            tagName = KST::suggestPluginName(_pluginList[pitem], _curve);
+            tagName = KST::suggestPluginName(_pluginList[pitem], _yvector);
           }
 
           plugin->setTagName(tagName);
           if (saveOutputs(plugin, pPtr)) {
-            plugin->writeUnlock();
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(plugin.data());
-            KST::dataObjectList.lock().writeUnlock();
-
-            plugin->readLock();
             if (plugin->isValid()) {
               if (!createCurve(plugin)) {
                 KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
+              } else {
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(plugin.data());
+                KST::dataObjectList.lock().writeUnlock();
               }
             } else {
               KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
             }
-            plugin->readUnlock();
-          } else {
-            plugin->writeUnlock();
           }
-        } else {
-          plugin->writeUnlock();
         }
+        plugin->writeUnlock();
       }
     }
     emit modified();
@@ -570,7 +528,7 @@ void KstFilterDialogI::generateEntries(const QString& fixedVector, bool input, i
       QWhatsThis::add(widget, (*it)._description);
     }
 
-    cnt++;
+    ++cnt;
   }
 }
 
@@ -583,7 +541,6 @@ void KstFilterDialogI::pluginChanged(int idx) {
   // Create a new one
   QGridLayout *topGrid = dynamic_cast<QGridLayout*>(_pluginFrame->layout());
   if (topGrid) {
-    topGrid->invalidate();
     topGrid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
   } else {
     kdError() << "Somehow we lost the grid!" << endl;
@@ -612,11 +569,11 @@ void KstFilterDialogI::pluginChanged(int idx) {
     QFrame* line = new QFrame(_frameWidget);
     line->setFrameShadow(QFrame::Sunken);
     line->setFrameShape(QFrame::HLine);
-    cnt++;
+    ++cnt;
     grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
 
     // Generate the input values
-    cnt++;
+    ++cnt;
     generateEntries(pluginData._filterInputVector, true, cnt, _frameWidget, grid, pluginData._inputs);
 
     if (!pluginData._inputs.isEmpty() && !pluginData._outputs.isEmpty()) {
@@ -624,12 +581,12 @@ void KstFilterDialogI::pluginChanged(int idx) {
       line->setFrameShape(QFrame::HLine);
       line->setFrameShadow(QFrame::Sunken);
       line->setFrameShape(QFrame::HLine);
-      cnt++;
+      ++cnt;
       grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
     }
 
     // Generate the output values
-    cnt++;
+    ++cnt;
     generateEntries(pluginData._filterOutputVector, false, cnt, _frameWidget, grid, pluginData._outputs);
   }
 
