@@ -39,19 +39,19 @@
 #include "stringselector.h"
 #include "vectorselector.h"
 
-KstFitDialogI *KstFitDialogI::_inst = 0L;
-static KStaticDeleter<KstFitDialogI> _plFitInst;
+QGuardedPtr<KstFitDialogI> KstFitDialogI::_inst;
 
 KstFitDialogI *KstFitDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _plFitInst.setObject(new KstFitDialogI);
+    _inst = new KstFitDialogI(KstApp::inst());
   }
 
   return _inst;
 }
 
 
-const QString newFitPluginString = "<Auto Name>";
+//const QString newFitPluginString = "<Auto Name>";
+const QString fit_defaultTag = "<Auto Name>";
 
 KstFitDialogI::KstFitDialogI(QWidget* parent, const char* name,
                              bool modal, WFlags fl)
@@ -86,8 +86,7 @@ void KstFitDialogI::OK() {
 void KstFitDialogI::show_setCurve(const QString& strCurve,
                                   const QString& strPlotName,
                                   const QString& strWindow) {
-
-  KstBaseCurvePtr  curve;
+  KstBaseCurvePtr curve;
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
   KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
   QString new_label;
@@ -95,17 +94,19 @@ void KstFitDialogI::show_setCurve(const QString& strCurve,
   _strWindow   = strWindow;
   _strPlotName = strPlotName;
   _strCurve    = strCurve;
-  if (curves.findTag(strCurve) != curves.end()) {
-    curve = *(curves.findTag(strCurve));
+  curve = *curves.findTag(strCurve);
+  if (curve) {
+    KstReadLocker rl(curve);
     _xvector = curve->xVTag();
     _yvector = curve->yVTag();
     _evector = curve->yETag();
   }
 
-  if ( _xvector=="<None>" ) return;
-  if ( _yvector=="<None>" ) return;
+  if (_xvector == "<None>" || _yvector == "<None>") {
+    return;
+  }
 
-   _tagName->setText(newFitPluginString);
+  _tagName->setText(fit_defaultTag);
 
   updatePluginList();
   pluginChanged(PluginCombo->currentItem());
@@ -135,7 +136,7 @@ void KstFitDialogI::updatePluginList() {
         if (it.data()._name == previous) {
           newFocus = cnt;
         }
-        cnt++;
+        ++cnt;
       }
     }
   }
@@ -150,9 +151,6 @@ void KstFitDialogI::updatePluginList() {
 
 
 void KstFitDialogI::update() {
-  QString selectedVector;
-  QString selectedScalar;
-  QString selectedString;
   KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
   if (pPtr) {
     const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
@@ -184,8 +182,8 @@ void KstFitDialogI::update() {
 
 
 void KstFitDialogI::fixupLayout() {
-  _frameWidget->resize(_frameWidget->sizeHint());
-  _pluginFrame->resize(_pluginFrame->sizeHint());
+  _frameWidget->updateGeometry();
+  _pluginFrame->updateGeometry();
   resize(sizeHint());
   setMinimumSize(size());
   updateGeometry();
@@ -315,7 +313,7 @@ bool KstFitDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
 
     if (li->text().isEmpty()) {
       QString tagName = _tagName->text();
-      if (tagName == newFitPluginString) {
+      if (tagName == fit_defaultTag) {
         tagName = plugin->tagName();
       }
       li->setText(tagName+"-"+(*it)._name);
@@ -373,21 +371,18 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
   bool bRetVal = false;
 
   // Generate new fit name
-  QString c_name = KST::suggestCurveName(plugin->tagName());
+  QString c_name = KST::suggestCurveName(plugin->tagName(), true);
 
-  if (plugin->inputVectors().find("X Array") != plugin->inputVectors().end()) {
+  if (plugin->inputVectors().contains("X Array")) {
     xVector = plugin->inputVectors()["X Array"];
   }
-  if (plugin->outputVectors().find("Y Fitted") != plugin->outputVectors().end()) {
+  if (plugin->outputVectors().contains("Y Fitted")) {
     yVector = plugin->outputVectors()["Y Fitted"];
   }
 
   if (xVector && yVector) {
-    plugin->update(-1);
-    fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
+    fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
     if (fit) {
-      Kst2DPlotPtr plot;
-
       fit->setHasPoints(_curveAppearance->showPoints());
       fit->setHasLines(_curveAppearance->showLines());
       fit->setHasBars(_curveAppearance->showBars());
@@ -395,26 +390,24 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
       fit->setLineStyle(_curveAppearance->lineStyle());
       fit->Point.setType(_curveAppearance->pointType());
       fit->setBarStyle(_curveAppearance->barStyle());
+      fit->setPointDensity(_curveAppearance->pointDensity());
 
-      //
-      // create the necessary label...
-      //
-      strLabel = i18n("%2: %1\\n").
+      // create the necessary label
+      strLabel = i18n("curvename: pluginname\\n", "%2: %1\\n").
                  arg(_strCurve.replace( '_',  "\\_" ).remove( QRegExp( "^[C][0-9]{1,2}\\-" ) )).
                  arg(plugin->plugin()->data()._readableName);
 
       KstVectorPtr vectorParam = plugin->outputVectors()["Parameters"];
       if (vectorParam) {
-        QString strParamName;
-
+        plugin->createFitScalars();
         for (int i = 0; i < vectorParam->length(); i++) {
-          strParamName = plugin->plugin()->parameterName(i);
+          QString strParamName = plugin->plugin()->parameterName(i);
           if (!strParamName.isEmpty()) {
             if (plugin->outputScalars().contains(strParamName)) {
               KstScalarPtr scalar = plugin->outputScalars()[strParamName];
 
               if (scalar) {
-                strLabel += i18n("%1: [%2]\\n").arg(strParamName).arg(scalar->tagName());
+                strLabel += i18n("parametername: [scalarname]\\n", "%1: [%2]\\n").arg(strParamName).arg(scalar->tagName());
               }
             }
           }
@@ -426,12 +419,11 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
       //
       KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_strWindow));
       if (w && w->view()->findChild(_strPlotName)) {
-        plot = kst_cast<Kst2DPlot>(w->view()->findChild(_strPlotName));
+        Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_strPlotName));
         if (plot) {
           label = new KstLabel(strLabel, LxTy, 0.0, 0.05, 0.05);
           plot->addLabel(label);
           plot->addCurve(fit.data());
-          w->view()->paint(P_DATA);
         }
       }
 
@@ -450,7 +442,7 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
 bool KstFitDialogI::new_I() {
   QString tagName = _tagName->text();
 
-  if (tagName != newFitPluginString) {
+  if (tagName != fit_defaultTag) {
     if (KST::dataTagNameNotUnique(tagName, true, this)) {
       _tagName->setFocus();
       return false;
@@ -465,33 +457,29 @@ bool KstFitDialogI::new_I() {
     if (pPtr) {
       KstPluginPtr plugin = new KstPlugin;
       plugin->writeLock();
+      plugin->setDirty();
       if (saveInputs(plugin, pPtr)) {
         plugin->setPlugin(pPtr);
 
-        if (tagName == newFitPluginString) {
+        if (tagName == fit_defaultTag) {
           tagName = KST::suggestPluginName(_pluginList[pitem], _strCurve);
         }
 
         plugin->setTagName(tagName);
         if (saveOutputs(plugin, pPtr)) {
-          plugin->writeUnlock();
-          KST::dataObjectList.lock().writeLock();
-          KST::dataObjectList.append(plugin.data());
-          KST::dataObjectList.lock().writeUnlock();
-
-          plugin->readLock();
           if (plugin->isValid()) {
             if (!createCurve(plugin)) {
               KMessageBox::sorry(this, i18n("Could not create curve from fit, possibly due to a bad plugin."));
-              plugin->readUnlock();
-              plugin = 0l;
+              plugin->writeUnlock();
               return false;
             }
-            plugin->readUnlock();
+            plugin->writeUnlock();
+            KST::dataObjectList.lock().writeLock();
+            KST::dataObjectList.append(plugin.data());
+            KST::dataObjectList.lock().writeUnlock();
           } else {
             KMessageBox::sorry(this, i18n("There is something wrong (i.e, there is a bug) with the creation of the fit plugin."));
-            plugin->readUnlock();
-            plugin = 0l;
+            plugin->writeUnlock();
             return false;
           }
         } else {
@@ -623,7 +611,7 @@ void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidg
       QWhatsThis::add(widget, (*it)._description);
     }
 
-    cnt++;
+    ++cnt;
   }
 }
 
@@ -636,7 +624,6 @@ void KstFitDialogI::pluginChanged(int idx) {
   // Create a new one
   QGridLayout *topGrid = dynamic_cast<QGridLayout*>(_pluginFrame->layout());
   if (topGrid) {
-    topGrid->invalidate();
     topGrid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
   } else {
     kdError() << "Somehow we lost the grid!" << endl;
@@ -665,11 +652,11 @@ void KstFitDialogI::pluginChanged(int idx) {
     QFrame* line = new QFrame(_frameWidget);
     line->setFrameShadow(QFrame::Sunken);
     line->setFrameShape(QFrame::HLine);
-    cnt++;
+    ++cnt;
     grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
 
     // Generate the input values
-    cnt++;
+    ++cnt;
     generateEntries(pluginData._isFitWeighted, true, cnt, _frameWidget, grid, pluginData._inputs);
 
     if (!pluginData._inputs.isEmpty() && !pluginData._outputs.isEmpty()) {
@@ -677,12 +664,12 @@ void KstFitDialogI::pluginChanged(int idx) {
       line->setFrameShape(QFrame::HLine);
       line->setFrameShadow(QFrame::Sunken);
       line->setFrameShape(QFrame::HLine);
-      cnt++;
+      ++cnt;
       grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
     }
 
     // Generate the output values
-    cnt++;
+    ++cnt;
     generateEntries(pluginData._isFitWeighted, false, cnt, _frameWidget, grid, pluginData._outputs);
   }
 
