@@ -17,25 +17,16 @@
 
 // include files for Qt
 #include <qcheckbox.h>
-#include <qcolor.h>
-#include <qcombobox.h>
-#include <qfontdatabase.h>
-#include <qlistbox.h>
-#include <qpushbutton.h>
 #include <qradiobutton.h>
-#include <qstring.h>
-#include <qwidget.h>
 
 // include files for KDE
 #include <kcolorbutton.h>
 #include <kfontcombo.h>
 #include <klineedit.h>
-#include <kmdimainfrm.h>
 #include <knuminput.h>
 
 // application specific includes
 #include "kst.h"
-#include "kstdatacollection.h"
 #include "kstlabeldialog_i.h"
 #include "kstviewwindow.h"
 #include "scalarselector.h"
@@ -47,11 +38,12 @@ KstLabelDialogI::KstLabelDialogI(QWidget *parent, const char* name, bool modal,
   _i_plot = 0;
 
   FontComboBox->setCurrentFont(this->font().family());
-
+  FontComboBox->setEditable(false);
   connect(Apply, SIGNAL(clicked()), this, SLOT(apply()));
   connect(OK, SIGNAL(clicked()), this, SLOT(ok()));
   connect(Delete, SIGNAL(clicked()), this, SLOT(deleteL()));
   connect(checkBoxPlotColor, SIGNAL(clicked()), this, SLOT(usePlotColorChange()));
+  setFixedHeight(height());  
 }
 
 
@@ -91,7 +83,7 @@ void KstLabelDialogI::updateI() {
       _editing = false;
       _i_label = 0;
     } else {
-      KstLabel *label = _i_plot->_labelList.at(_i_label);
+      KstLabelPtr label = _i_plot->_labelList[_i_label];
       LabelText->setText(label->text());
       FontSize->setValue(label->size());
       switch (label->justification()) {
@@ -106,6 +98,7 @@ void KstLabelDialogI::updateI() {
           break;
         default:
           LeftJustify->setChecked(true);
+          break;
       }
       FontComboBox->setCurrentFont(label->fontName());
       Rotation->setValue((int)label->rotation());
@@ -147,17 +140,17 @@ void KstLabelDialogI::applyAsNew() {
     J = RxBy;
   }
 
-  KstLabel *new_label = new KstLabel(LabelText->text(), J,
+  KstLabelPtr new_label = new KstLabel(LabelText->text(), J,
                            (float) Rotation->value(), _x, _y, false);
   new_label->setFontName(FontComboBox->currentText());
   new_label->setSize(FontSize->value());
   new_label->setUsePlotColor(checkBoxPlotColor->isChecked());
   new_label->setColor(kColorButton->color());
-  
+
   _i_plot->_labelList.append(new_label);
   _i_label = _i_plot->_labelList.count() - 1;
   _editing = true;
-  _i_plot->update();
+  _i_plot->setDirty();
 }
 
 
@@ -169,7 +162,7 @@ void KstLabelDialogI::applyEdits() {
       _i_label = 0;
       return;
     } else {
-      KstLabel *label = _i_plot->_labelList.at(_i_label);
+      KstLabelPtr label = _i_plot->_labelList[_i_label];
       KstJustifyType J;
 
       if (LeftJustify->isChecked()) {
@@ -188,7 +181,7 @@ void KstLabelDialogI::applyEdits() {
       label->setUsePlotColor(checkBoxPlotColor->isChecked());
       label->setColor(kColorButton->color());
     }
-    _i_plot->update();
+    _i_plot->setDirty();
   }
 }
 
@@ -213,10 +206,10 @@ void KstLabelDialogI::deleteL() {
       close();
       return;
     } else {
-      _i_plot->_labelList.remove(_i_label);
+      _i_plot->_labelList.remove(_i_plot->_labelList[_i_label]);
       emit applied();
     }
-    _i_plot->update();
+    _i_plot->setDirty();
   }
   _editing = false;
   close();
@@ -224,11 +217,7 @@ void KstLabelDialogI::deleteL() {
 
 
 void KstLabelDialogI::usePlotColorChange() {
-  if (checkBoxPlotColor->isChecked()) {
-    kColorButton->setEnabled(false);  
-  } else {
-    kColorButton->setEnabled(true);
-  }
+    kColorButton->setEnabled(!checkBoxPlotColor->isChecked());  
 }
 #include "kstlabeldialog_i.moc"
 // vim: ts=2 sw=2 et
