@@ -23,9 +23,10 @@
 enum KstLegendLayoutType { LeftTop, CentreTop, RightTop, LeftCentre, CentreCentre, RightCentre, LeftBottom, CentreBottom, RightBottom };
 enum KstLegendAlignmentType { AlignmentLeft, AlignmentCentre, AlignmentRight };
 
-class KstLegend {
+class KstLegend : public KstShared, public KstRWLock {
   public:
     KstLegend(KstJustifyType in_j = CxBy, float in_X = 0.015, float in_Y = 0.02, bool is_sample = false);
+    KstLegend(const KstLegend *legend);
     ~KstLegend();
 
     void setRelPosition(float new_x, float new_y);
@@ -53,7 +54,7 @@ class KstLegend {
     int size() const;
 
     void save(QTextStream &ts, const QString& indent = QString::null);
-    void read(QDomElement &e);
+    void read(const QDomElement &e);
 
     void setDoScalarReplacement(bool in_do);
 
