@@ -22,7 +22,11 @@ KstMetaPlot::KstMetaPlot(const QString& type) : KstBorderedViewObject(type) {
 }
 
 
-KstMetaPlot::KstMetaPlot(QDomElement& e) : KstBorderedViewObject(e) {
+KstMetaPlot::KstMetaPlot(const QDomElement& e) : KstBorderedViewObject(e) {
+}
+
+
+KstMetaPlot::KstMetaPlot(const KstMetaPlot& metaPlot) : KstBorderedViewObject(metaPlot) {
 }
 
 
@@ -30,11 +34,6 @@ KstMetaPlot::~KstMetaPlot() {
 }
 
 
-KstObject::UpdateType KstMetaPlot::update() {
-  return NO_CHANGE;
-}
-
-
 void KstMetaPlot::save(QTextStream& ts, const QString& indent) {
   KstBorderedViewObject::save(ts, indent);
 }
@@ -45,9 +44,5 @@ void KstMetaPlot::saveTag(QTextStream& ts, const QString& indent) {
 }
 
 
-void KstMetaPlot::paint(KstPaintType type, QPainter& p) {
-  KstBorderedViewObject::paint(type, p);
-}
-
 #include "kstmetaplot.moc"
 // vim: ts=2 sw=2 et
