@@ -25,16 +25,12 @@ class KstMetaPlot : public KstBorderedViewObject {
   protected:
     KstMetaPlot(const QString& type);
   public:
-    KstMetaPlot(QDomElement& e);
+    KstMetaPlot(const QDomElement& e);
+    KstMetaPlot(const KstMetaPlot& metaPlot);
     virtual ~KstMetaPlot();
 
-    virtual UpdateType update();
     virtual void save(QTextStream& ts, const QString& indent = QString::null);
     virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
-
-  public slots:
-    virtual void paint(KstPaintType type, QPainter& p);
-
 };
 
 typedef KstSharedPtr<KstMetaPlot> KstMetaPlotPtr;
