@@ -26,10 +26,10 @@
 #include <klocale.h>
 
 // application specific includes
+#include "dialoglauncher.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstplugin.h"
-#include "kstplugindialog_i.h"
 #include "plugincollection.h"
 
 
@@ -40,7 +40,7 @@ KstPlugin::KstPlugin() : KstDataObject() {
 }
 
 
-KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
+KstPlugin::KstPlugin(const QDomElement &e) : KstDataObject(e) {
   QString pluginName;
 
   _inStringCnt = 0;
@@ -60,20 +60,30 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
         _inputVectorLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
       } else if (e.tagName() == "iscalar") {
         _inputScalarLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
+      } else if (e.tagName() == "istring") {
+        _inputStringLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
       } else if (e.tagName() == "ovector") {
         KstVectorPtr v;
-
-        // FIXME: these might already exist!!
         if (e.attribute("scalarList", "0").toInt()) {
           v = new KstVector(e.text(), 0, true);
         } else {
           v = new KstVector(e.text(), 0, false);
         }
+        v->setProvider(this);
         _outputVectors.insert(e.attribute("name"), v);
         KST::addVectorToList(v);
       } else if (e.tagName() == "oscalar") {
-        // FIXME: these might already exist!!
-        _outputScalars.insert(e.attribute("name"), new KstScalar(e.text()));
+        KstScalarPtr sp = new KstScalar(e.text());
+        sp->writeLock();
+        sp->setProvider(this);
+        sp->writeUnlock();
+        _outputScalars.insert(e.attribute("name"), sp);
+      } else if (e.tagName() == "ostring") {
+        KstStringPtr sp = new KstString(e.text());
+        sp->writeLock();
+        sp->setProvider(this);
+        sp->writeUnlock();
+        _outputStrings.insert(e.attribute("name"), sp);
       }
     }
     n = n.nextSibling();
@@ -101,19 +111,43 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
           } else {
             v = new KstVector(tagName() + " vector - " + (*it)._name, 0, false);
           }
+          v->setProvider(this);
           _outputVectors.insert((*it)._name, v);
           KST::addVectorToList(v);
         }
+      } else if ((*it)._type == Plugin::Data::IOValue::MatrixType) {
+          abort(); // FIXME:
+#if 0
+        _outArrayCnt += 2;
+        if (!_outputMatrices.contains((*it)._name)) {
+          KstMatrixPtr m;
+
+          if ((*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
+            m = new KstMatrix(tagName() + " matrix - " + (*it)._name, 0, true);
+          } else {
+            m = new KstMatrix(tagName() + " matrix - " + (*it)._name, 0, false);
+          }
+          m->setProvider(this);
+          _outputMatrices.insert((*it)._name, m);
+          KST::addVectorToList(v);
+        }
+#endif
       } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
         _outScalarCnt++;
         if (!_outputScalars.contains((*it)._name)) {
           KstScalarPtr s = new KstScalar(tagName() + " scalar - " + (*it)._name);
+          s->writeLock();
+          s->setProvider(this);
+          s->writeUnlock();
           _outputScalars.insert((*it)._name, s);
         }
       } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
         _outStringCnt++;
         if (!_outputStrings.contains((*it)._name)) {
           KstStringPtr s = new KstString(tagName() + " string - " + (*it)._name);
+          s->writeLock();
+          s->setProvider(this);
+          s->writeUnlock();
           _outputStrings.insert((*it)._name, s);
         }
       }
@@ -226,16 +260,14 @@ void KstPlugin::freeParameters() {
 
 KstObject::UpdateType KstPlugin::update(int update_counter) {
   if (!isValid()) {
-    return NO_CHANGE;
+    return setLastUpdateResult(NO_CHANGE);
   }
 
-  if (KstObject::checkUpdateCounter(update_counter)) {
-    return NO_CHANGE;
-  }
+  bool force = dirty();
+  setDirty(false);
 
-  if (update_counter != -1) {
-    //bool force = false;
-    // FIXME: we should only update if one of our dependencies was updated
+  if (KstObject::checkUpdateCounter(update_counter) && !force) {
+    return lastUpdateResult();
   }
 
 #define CLEANUP() do {\
@@ -256,9 +288,8 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
 
   const QValueList<Plugin::Data::IOValue>& itable = _plugin->data()._inputs;
   const QValueList<Plugin::Data::IOValue>& otable = _plugin->data()._outputs;
-  bool bNew = false; // what does this variable name mean?  That's why we don't
-                    // use HN in Kst!!
   int itcnt = 0, vitcnt = 0, sitcnt = 0;
+  bool doUpdate = force;
 
   // Populate the input scalars and vectors
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin();
@@ -268,22 +299,31 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
       if (!_inputVectors.contains((*it)._name)) {
         KstDebug::self()->log(i18n("Input vector [%1] for plugin %2 not found.  Unable to continue.").arg((*it)._name).arg(tagName()), KstDebug::Error);
         CLEANUP();
-        return NO_CHANGE;
+        return setLastUpdateResult(NO_CHANGE);
       }
-      if (_inputVectors[(*it)._name]->sampleCount() == _inputVectors[(*it)._name]->numNew()) {
-        bNew = true;
-      }
-      _inVectors[vitcnt] = _inputVectors[(*it)._name]->value();
-      _inArrayLens[vitcnt++] = _inputVectors[(*it)._name]->sampleCount();
+      KstVectorPtr iv = _inputVectors[(*it)._name];
+      doUpdate = (UPDATE == iv->update(update_counter)) || doUpdate;
+      _inVectors[vitcnt] = iv->value();
+      _inArrayLens[vitcnt++] = iv->length();
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      _inScalars[itcnt++] = _inputScalars[(*it)._name]->value();
+      KstScalarPtr is = _inputScalars[(*it)._name];
+      doUpdate = (UPDATE == is->update(update_counter)) || doUpdate;
+      _inScalars[itcnt++] = is->value();
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      _inStrings[sitcnt++] = strdup(_inputStrings[(*it)._name]->value().latin1());
+      KstStringPtr is = _inputStrings[(*it)._name];
+      doUpdate = (UPDATE == is->update(update_counter)) || doUpdate;
+      // Maybe we should use UTF-8 instead?
+      _inStrings[sitcnt++] = strdup(is->value().latin1());
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       _inScalars[itcnt++] = getpid();
     }
   }
 
+  if (!doUpdate) {
+    CLEANUP();
+    return setLastUpdateResult(NO_CHANGE);
+  }
+
   vitcnt = 0;
   // Populate the output vectors
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin();
@@ -293,7 +333,7 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
       if (!_outputVectors.contains((*it)._name)) {
         KstDebug::self()->log(i18n("Output vector [%1] for plugin %2 not found.  Unable to continue.").arg((*it)._name).arg(tagName()), KstDebug::Error);
         CLEANUP();
-        return NO_CHANGE;
+        return setLastUpdateResult(NO_CHANGE);
       }
       _outVectors[vitcnt] = _outputVectors[(*it)._name]->value();
       _outArrayLens[vitcnt++] = _outputVectors[(*it)._name]->length();
@@ -334,33 +374,24 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
         it != otable.end();
         ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) {
-        vectorRealloced(_outputVectors[(*it)._name], _outVectors[vitcnt], _outArrayLens[vitcnt], bNew);
+        KstVectorPtr vp = _outputVectors[(*it)._name];
+        vectorRealloced(vp, _outVectors[vitcnt], _outArrayLens[vitcnt]);
+        vp->setDirty();
+        vp->update(update_counter);
         vitcnt++;
       } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-        _outputScalars[(*it)._name]->setValue(_outScalars[itcnt++]);
+        KstScalarPtr sp = _outputScalars[(*it)._name];
+        sp->setValue(_outScalars[itcnt++]);
+        sp->update(update_counter);
       } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-        _outputStrings[(*it)._name]->setValue(_outStrings[sitcnt++]);
+        KstStringPtr sp = _outputStrings[(*it)._name];
+        sp->setValue(_outStrings[sitcnt++]);
+        sp->update(update_counter);
       }
     }
 
-    // if we have a fit plugin then create the necessary scalars from the parameter vector...
-    if (_plugin->data()._isFit) {
-      KstVectorPtr vectorParam = _outputVectors["Parameters"];
-      if (vectorParam) {
-        for (int i = 0; i < vectorParam->length(); i++) {
-          const QString& paramName = _plugin->parameterName(i);
-          if (!paramName.isEmpty()) {
-            QString scalarName = i18n("%1-%2").arg(tagName()).arg(paramName);
-            double scalarValue = vectorParam->value(i);
-            if (!_outputScalars.contains(paramName)) {
-              KstScalarPtr s = new KstScalar(scalarName);
-              _outputScalars.insert(paramName, s);
-            }
-            _outputScalars[paramName]->setValue(scalarValue);
-          }
-        }
-      }
-    }
+    // if we have a fit plugin then create the necessary scalars from the parameter vector
+    createFitScalars();
     _lastError = QString::null;
   } else if (rc > 0) {
     if (_lastError.isEmpty()) {
@@ -394,10 +425,10 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
       KstDebug::self()->log(i18n("Plugin %2 produced error: %1.").arg(_lastError).arg(tagName()), KstDebug::Error);
     }
   }
-  
+
   CLEANUP();
 #undef CLEANUP
-  return UPDATE;
+  return setLastUpdateResult(UPDATE);
 }
 
 
@@ -523,17 +554,20 @@ bool KstPlugin::setPlugin(KstSharedPtr<Plugin> plugin) {
         v = new KstVector;
       }
       v->writeLock();
+      v->setProvider(this);
       _outputVectors.insert((*it)._name, v);
       _outArrayCnt++;
       KST::addVectorToList(v);
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       KstScalarPtr s = new KstScalar;
       s->writeLock();
+      s->setProvider(this);
       _outputScalars.insert((*it)._name, s);
       _outScalarCnt++;
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
       KstStringPtr s = new KstString;
       s->writeLock();
+      s->setProvider(this);
       _outputStrings.insert((*it)._name, s);
       _outStringCnt++;
     }
@@ -551,7 +585,7 @@ KstSharedPtr<Plugin> KstPlugin::plugin() const {
 
 
 void KstPlugin::_showDialog() {
-  KstPluginDialogI::globalInstance()->show_Edit(tagName());
+  KstDialogs::showPluginDialog(tagName());
 }
 
 
@@ -570,4 +604,28 @@ QString KstPlugin::lastError() const {
   return _lastError;
 }
 
+
+void KstPlugin::createFitScalars() {
+  if (_plugin->data()._isFit && _outputVectors.contains("Parameters")) {
+    KstVectorPtr vectorParam = _outputVectors["Parameters"];
+    if (vectorParam) {
+      for (int i = 0; i < vectorParam->length(); i++) {
+        const QString& paramName = _plugin->parameterName(i);
+        if (!paramName.isEmpty()) {
+          QString scalarName = i18n("%1-%2").arg(tagName()).arg(paramName);
+          double scalarValue = vectorParam->value(i);
+          if (!_outputScalars.contains(paramName)) {
+            KstScalarPtr s = new KstScalar(scalarName);
+            s->writeLock();
+            s->setProvider(this);
+            s->writeUnlock();
+            _outputScalars.insert(paramName, s);
+          }
+          _outputScalars[paramName]->setValue(scalarValue);
+        }
+      }
+    }
+  }
+}
+
 // vim: ts=2 sw=2 et
