@@ -16,59 +16,48 @@
  ***************************************************************************/
 
 #include <assert.h>
-#include <stdlib.h>
 
 // include files for Qt
-#include <qcheckbox.h>
 #include <qcombobox.h>
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qlineedit.h>
-#include <qlistview.h>
-#include <qobjectlist.h>
-#include <qpushbutton.h>
-#include <qspinbox.h>
-#include <qstring.h>
-#include <qstringlist.h>
 #include <qtimer.h>
 #include <qtooltip.h>
 #include <qwhatsthis.h>
-#include <qwidget.h>
 
 // include files for KDE
 #include <kcolorbutton.h>
-#include <kcolordialog.h>
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <kstaticdeleter.h>
 
 // application specific includes
-#include "kstdatacollection.h"
-#include "kstdoc.h"
+#include "kst.h"
 #include "kstplugindialog_i.h"
-#include "plugin.h"
+#include "kstdefaultnames.h"
 #include "plugincollection.h"
 #include "pluginmanager.h"
 #include "scalarselector.h"
+#include "stringselector.h"
 #include "vectorselector.h"
 
 #define DIALOGTYPE KstPluginDialogI
 #define DTYPE "Plugin"
 #include "dataobjectdialog.h"
 
-KstPluginDialogI *KstPluginDialogI::_inst = 0L;
-static KStaticDeleter<KstPluginDialogI> _plPluginInst;
+
+QGuardedPtr<KstPluginDialogI> KstPluginDialogI::_inst;
 
 KstPluginDialogI *KstPluginDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _plPluginInst.setObject(new KstPluginDialogI);
+    _inst = new KstPluginDialogI(KstApp::inst());
   }
   return _inst;
 }
 
 
-static const QString newPluginString = "<New_Plugin>";
+static const QString& plugin_defaultTag = KGlobal::staticQString("<Auto Name>");
 
 KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name,
                                    bool modal, WFlags fl)
@@ -86,35 +75,46 @@ KstPluginDialogI::~KstPluginDialogI() {
   DP = 0L;
 }
 
+
 KstPluginPtr KstPluginDialogI::_getPtr(const QString &tagin) {
   KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
 
   return *c.findTag(tagin);
 }
 
+
 void KstPluginDialogI::updatePluginList() {
   PluginCollection *pc = PluginCollection::self();
   QString previous = _pluginList[PluginCombo->currentItem()];
   int newFocus = -1;
 
-  const QMap<QString,Plugin::Data>& _pluginMap = pc->pluginList();
+  const QMap<QString,Plugin::Data>& pluginMap = pc->pluginList();
+
+  QMap<QString,QString> oldIEntries, oldOEntries;
 
   PluginCombo->clear();
   _pluginList.clear();
   int cnt = 0;
-  for (QMap<QString,Plugin::Data>::ConstIterator it = _pluginMap.begin();
-                                                  it != _pluginMap.end();
-                                                                     ++it) {
+  Plugin::Data restoreEntry;
+  for (QMap<QString,Plugin::Data>::ConstIterator it = pluginMap.begin();
+                                                  it != pluginMap.end();
+                                                                   ++it) {
     _pluginList += it.data()._name;
     PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName).arg(it.data()._version).arg(it.data()._description));
     if (it.data()._name == previous) {
       newFocus = cnt;
+      oldIEntries = cacheInputs(it.data()._inputs);
+      oldOEntries = cacheInputs(it.data()._outputs);
+      restoreEntry = it.data();
     }
-    cnt++;
+    ++cnt;
   }
 
   if (newFocus != -1) {
     PluginCombo->setCurrentItem(newFocus);
+    pluginChanged(PluginCombo->currentItem());
+    restoreInputs(restoreEntry._inputs, oldIEntries);
+    restoreInputs(restoreEntry._outputs, oldOEntries);
   } else {
     PluginCombo->setCurrentItem(0);
     pluginChanged(0);
@@ -134,6 +134,15 @@ void KstPluginDialogI::updateForm() {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           vs->update();
         }
+      } else if ((*it)._type == Plugin::Data::IOValue::StringType) { // string
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+        assert(field);
+        if (field) {
+          StringSelector *ss = static_cast<StringSelector*>(field);
+          ss->update();
+        }
+      } else if ((*it)._type == Plugin::Data::IOValue::PidType) { 
+        // Nothing
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
         assert(field);
@@ -146,6 +155,7 @@ void KstPluginDialogI::updateForm() {
   }
 }
 
+
 void KstPluginDialogI::_fillFieldsForEdit() {
   if (DP == 0L) {
     return; // shouldn't be needed
@@ -160,34 +170,34 @@ void KstPluginDialogI::_fillFieldsForEdit() {
   pluginChanged(PluginCombo->currentItem());
 
   fillVectorScalarCombos(DP->plugin());
+  PluginCombo->setEnabled(DP->getUsage() < 2);
   DP->readUnlock();
 
   show();
   fixupLayout();
 }
 
+
 void KstPluginDialogI::_fillFieldsForNew() {
   updatePluginList();
 
   KstPluginList plugins = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
 
   /* set tag name */
-  QString new_label;
-  new_label.sprintf("PL%d-", signed(plugins.count())+1);
-  new_label += newPluginString;
-  _tagName->setText(new_label);
+  _tagName->setText(plugin_defaultTag);
 
   updatePluginList();
 
   PluginCombo->setCurrentItem(0);
   pluginChanged(PluginCombo->currentItem());
+  PluginCombo->setEnabled(true);
 
   KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
 
   fillVectorScalarCombos(plugin);
-
 }
 
+
 void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
   bool DPvalid = false;
 
@@ -216,6 +226,21 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
             vs->setSelection(selectedVector);
           }
         }
+      } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+        assert(field);
+        if (field) {
+          StringSelector *ss = static_cast<StringSelector*>(field);
+          QString selectedString = ss->selectedString();
+          ss->update();
+          if (DPvalid) {
+            ss->setSelection(DP->inputStrings()[(*it)._name]->tagName());
+          } else {
+            ss->setSelection(selectedString);
+          }
+        }
+      } else if ((*it)._type == Plugin::Data::IOValue::PidType) { 
+        // Nothing
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
         assert(field);
@@ -243,6 +268,9 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
           QString ts;
           if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
             ts = DP->outputVectors()[(*it)._name]->tagName();
+          } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+          } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+            ts = DP->outputStrings()[(*it)._name]->tagName();
           } else { // scalar
             ts = DP->outputScalars()[(*it)._name]->tagName();
           }
@@ -264,22 +292,109 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
   }
 }
 
+
 void KstPluginDialogI::update() {
   updatePluginList();
 }
 
 
 void KstPluginDialogI::fixupLayout() {
-  _frameWidget->resize(_frameWidget->sizeHint());
-  _pluginFrame->resize(_pluginFrame->sizeHint());
+  _frameWidget->updateGeometry();
+  _pluginFrame->updateGeometry();
   resize(sizeHint());
+  setMinimumSize(size());
   updateGeometry();
 }
 
 
+QMap<QString,QString> KstPluginDialogI::cacheInputs(const QValueList<Plugin::Data::IOValue>& table) {
+  QMap<QString,QString> rc;
+  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
+    if ((*it)._type == Plugin::Data::IOValue::TableType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+      if (field) {
+        rc[(*it)._name] = static_cast<VectorSelector*>(field)->selectedVector();
+      } else {
+        field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          rc[(*it)._name] = static_cast<QLineEdit*>(field)->text();
+        }
+      }
+
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+      if (field) {
+        rc[(*it)._name] = static_cast<StringSelector*>(field)->selectedString();
+      } else {
+        field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          rc[(*it)._name] = static_cast<QLineEdit*>(field)->text();
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
+    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
+      if (field) {
+        rc[(*it)._name] = static_cast<ScalarSelector*>(field)->selectedScalar();
+      } else {
+        field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          rc[(*it)._name] = static_cast<QLineEdit*>(field)->text();
+        }
+      }
+    }
+  }
+  return rc;
+}
+
+
+void KstPluginDialogI::restoreInputs(const QValueList<Plugin::Data::IOValue>& table, const QMap<QString,QString>& v) {
+  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
+    if (!v.contains((*it)._name)) {
+      continue;
+    }
+    if ((*it)._type == Plugin::Data::IOValue::TableType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+      if (field) {
+        static_cast<VectorSelector*>(field)->setSelection(v[(*it)._name]);
+      } else {
+        field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          static_cast<QLineEdit*>(field)->setText(v[(*it)._name]);
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+      if (field) {
+        static_cast<StringSelector*>(field)->setSelection(v[(*it)._name]);
+      } else {
+        field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          static_cast<QLineEdit*>(field)->setText(v[(*it)._name]);
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
+    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
+      if (field) {
+        static_cast<ScalarSelector*>(field)->setSelection(v[(*it)._name]);
+      } else {
+        field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
+        if (field) {
+          static_cast<QLineEdit*>(field)->setText(v[(*it)._name]);
+        }
+      }
+    }
+  }
+}
+
+
 bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
   KST::vectorList.lock().readLock();
   KST::scalarList.lock().readLock();
+  KST::stringList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
@@ -294,7 +409,29 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         plugin->inputVectors()[(*it)._name]->writeUnlock();
       }
       plugin->inputVectors().insert((*it)._name, v);
-    } else {
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+      assert(field);
+      StringSelector *ss = static_cast<StringSelector*>(field);
+      KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
+      if (s == *KST::stringList.end()) {
+        QString val = ss->_string->currentText();
+        KstStringPtr newString = new KstString(ss->_string->currentText(), val, true, false);
+        newString->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
+          plugin->inputStrings()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputStrings().insert((*it)._name, newString);
+      } else {
+        s->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
+          plugin->inputStrings()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputStrings().insert((*it)._name, s);
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
+    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
       assert(field);
       ScalarSelector *ss = static_cast<ScalarSelector*>(field);
@@ -324,8 +461,10 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         }
         plugin->inputScalars().insert((*it)._name, s);
       }
+    } else {
     }
   }
+  KST::stringList.lock().readUnlock();
   KST::scalarList.lock().readUnlock();
   KST::vectorList.lock().readUnlock();
   return true;
@@ -337,13 +476,15 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
     QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
-    assert(field);
+    if (!field) {
+      continue; // Some are unsupported
+    }
 
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
     if (li->text().isEmpty()) {
       QString tagName = _tagName->text();
-      if (tagName.endsWith(newPluginString)) {
+      if (tagName==plugin_defaultTag) {
         tagName = plugin->tagName();
       }
       li->setText(tagName + "-" + (*it)._name);
@@ -352,7 +493,16 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
     QString nt = li->text();
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputVectors()[(*it)._name]->setTagName(nt);
+        // Implicitly creates it if it doesn't exist
+        KstVectorPtr v = plugin->outputVectors()[(*it)._name];
+        if (!v) {
+          v = new KstVector;
+          v->writeLock();
+          plugin->outputVectors().insert((*it)._name, v);
+          KST::addVectorToList(v);
+        }
+        v->setTagName(nt);
+        v->setProvider(plugin.data()); // in case it's implicitly created
       } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -363,10 +513,41 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         } else {
           v = new KstVector;
           v->writeLock();
+          v->setProvider(plugin.data());
           plugin->outputVectors().insert((*it)._name, v);
+          KST::addVectorToList(v);
         }
         v->setTagName(nt);
       }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      if (!KST::vectorTagNameNotUnique(nt, false)) {
+        KstStringPtr s;
+        if (plugin->outputStrings().contains((*it)._name)) {
+          s = plugin->outputStrings()[(*it)._name];
+        } else {
+          s = new KstString;
+          s->writeLock();
+          plugin->outputStrings().insert((*it)._name, s);
+        }
+        s->setTagName(nt);
+        s->setProvider(plugin.data());
+      } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
+        while (KST::vectorTagNameNotUnique(nt, false)) {
+          nt += "'";
+        }
+        KstStringPtr s;
+        if (plugin->outputStrings().contains((*it)._name)) {
+          s = plugin->outputStrings()[(*it)._name];
+        } else {
+          s = new KstString;
+          s->writeLock();
+          plugin->outputStrings().insert((*it)._name, s);
+        }
+        s->setTagName(nt);
+        s->setProvider(plugin.data());
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
         KstScalarPtr s;
@@ -378,6 +559,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           plugin->outputScalars().insert((*it)._name, s);
         }
         s->setTagName(nt);
+        s->setProvider(plugin.data());
       } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -391,6 +573,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           plugin->outputScalars().insert((*it)._name, s);
         }
         s->setTagName(nt);
+        s->setProvider(plugin.data());
       }
     }
   }
@@ -400,14 +583,14 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
 
 
 bool KstPluginDialogI::new_I() {
-  KstPluginPtr plugin;
   QString tagName = _tagName->text();
 
-  if (!tagName.endsWith(newPluginString) && KST::dataTagNameNotUnique(tagName, true, this)) {
+  if (tagName != plugin_defaultTag && KST::dataTagNameNotUnique(tagName, true, this)) {
     _tagName->setFocus();
     return false;
   }
 
+  KstPluginPtr plugin;
   int pitem = PluginCombo->currentItem();
   if (pitem >= 0 && PluginCombo->count() > 0) {
     KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
@@ -422,13 +605,8 @@ bool KstPluginDialogI::new_I() {
 
       plugin->setPlugin(pPtr);
 
-      if (tagName.endsWith(newPluginString)) {
-        QString tmpTagName;
-        int i = 0;
-        do {
-          tmpTagName = i18n("P%1-%2").arg(++i).arg(_pluginList[pitem]);
-        } while (KST::dataTagNameNotUnique(tmpTagName, false));
-        tagName = tmpTagName;
+      if (tagName == plugin_defaultTag) {
+        tagName = KST::suggestPluginName(_pluginList[pitem]);
       }
       plugin->setTagName(tagName);
       if (!saveOutputs(plugin, pPtr)) {
@@ -441,12 +619,11 @@ bool KstPluginDialogI::new_I() {
   }
 
   if (!plugin || !plugin->isValid()) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. "
-                                  "Please fix it."));
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered.  Please fix the plugin."));
     return false;
   }
 
-  plugin->update(-1);
+  plugin->setDirty();
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(plugin.data());
   KST::dataObjectList.lock().writeUnlock();
@@ -481,8 +658,12 @@ bool KstPluginDialogI::edit_I() {
   for (KstScalarMap::Iterator i = DP->inputScalars().begin(); i != DP->inputScalars().end(); ++i) {
     (*i)->writeUnlock();
   }
+  for (KstStringMap::Iterator i = DP->inputStrings().begin(); i != DP->inputStrings().end(); ++i) {
+    (*i)->writeUnlock();
+  }
   DP->inputVectors().clear();
   DP->inputScalars().clear();
+  DP->inputStrings().clear();
 
   // Save the vectors and scalars
   if (!saveInputs(DP, pPtr)) {
@@ -508,7 +689,7 @@ bool KstPluginDialogI::edit_I() {
     DP->writeUnlock();
     return false;
   }
-  DP->update(-1);
+  DP->setDirty();
   DP->writeUnlock();
 
   emit modified();
@@ -516,12 +697,14 @@ bool KstPluginDialogI::edit_I() {
 }
 
 void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
-QString scalarLabelTemplate, vectorLabelTemplate;
+  QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
 
   if (input) {
+    stringLabelTemplate = i18n("Input string - %1:");
     scalarLabelTemplate = i18n("Input scalar - %1:");
     vectorLabelTemplate = i18n("Input vector - %1:");
   } else {
+    stringLabelTemplate = i18n("Output string - %1:");
     scalarLabelTemplate = i18n("Output scalar - %1:");
     vectorLabelTemplate = i18n("Output vector - %1:");
   }
@@ -529,7 +712,14 @@ QString scalarLabelTemplate, vectorLabelTemplate;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
     QString labellabel;
     bool scalar = false;
+    bool string = false;
     switch ((*it)._type) {
+      case Plugin::Data::IOValue::PidType:
+        continue;
+      case Plugin::Data::IOValue::StringType:
+        labellabel = stringLabelTemplate.arg((*it)._name);
+        string = true;
+        break;
       case Plugin::Data::IOValue::FloatType:
         labellabel = scalarLabelTemplate.arg((*it)._name);
         scalar = true;
@@ -558,13 +748,33 @@ QString scalarLabelTemplate, vectorLabelTemplate;
         widget = w;
         connect(w->_scalar, SIGNAL(activated(const QString&)), this, SLOT(updateScalarTooltip(const QString&)));
         connect(widget, SIGNAL(newScalarCreated()), this, SIGNAL(modified()));
+        if (!(*it)._default.isEmpty()) {
+          w->_scalar->insertItem((*it)._default);
+          w->_scalar->setCurrentText((*it)._default);
+        }
         KstScalarPtr p = *KST::scalarList.findTag(w->_scalar->currentText());
-        w->allowDirectEntry( true );
+        w->allowDirectEntry(true);
         if (p) {
           p->readLock();
           QToolTip::add(w->_scalar, QString::number(p->value()));
           p->readUnlock();
         }
+      } else if (string) {
+        StringSelector *w = new StringSelector(parent, (*it)._name.latin1());
+        widget = w;
+        connect(w->_string, SIGNAL(activated(const QString&)), this, SLOT(updateStringTooltip(const QString&)));
+        connect(widget, SIGNAL(newStringCreated()), this, SIGNAL(modified()));
+        if (!(*it)._default.isEmpty()) {
+          w->_string->insertItem((*it)._default);
+          w->_string->setCurrentText((*it)._default);
+        }
+        KstStringPtr p = *KST::stringList.findTag(w->_string->currentText());
+        w->allowDirectEntry(true);
+        if (p) {
+          p->readLock();
+          QToolTip::add(w->_string, p->value());
+          p->readUnlock();
+        }
       } else {
         widget = new VectorSelector(parent, (*it)._name.latin1());
         connect(widget, SIGNAL(newVectorCreated(const QString&)), this, SIGNAL(modified()));
@@ -594,7 +804,6 @@ void KstPluginDialogI::pluginChanged(int idx) {
   // Create a new one
   QGridLayout *topGrid = dynamic_cast<QGridLayout*>(_pluginFrame->layout());
   if (topGrid) {
-    topGrid->invalidate();
     topGrid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
   } else {
     kdError() << "Somehow we lost the grid!" << endl;
@@ -672,6 +881,20 @@ void KstPluginDialogI::updateScalarTooltip(const QString& n) {
 }
 
 
+void KstPluginDialogI::updateStringTooltip(const QString& n) {
+  KstStringPtr s = *KST::stringList.findTag(n);
+  QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
+  if (s) {
+    s->readLock();
+    QToolTip::remove(w);
+    QToolTip::add(w, s->value());
+    s->readUnlock();
+  } else {
+    QToolTip::remove(w);
+  }
+}
+
+
 #include "kstplugindialog_i.moc"
 
 // vim: ts=2 sw=2 et
