@@ -36,7 +36,7 @@ int KstPoint::type() const {
   return Type;
 }
 
-void KstPoint::draw(QPainter *p, int x, int y, int size) {
+void KstPoint::draw(QPainter *p, int x, int y, int lineSize, int size) {
   int s;
 
   if (size == -1) {
@@ -49,6 +49,8 @@ void KstPoint::draw(QPainter *p, int x, int y, int size) {
     s = 1;
   }
 
+  s += lineSize;
+
   switch (Type) {
     case 0:
       p->drawLine(x-s, y-s, x+s, y+s);
@@ -67,14 +69,22 @@ void KstPoint::draw(QPainter *p, int x, int y, int size) {
       p->drawEllipse(x-s, y-s, 2*s+1, 2*s+1);
       break;
     case 4:
-      p->drawLine(x-s, y-s, x+s, y-s);
-      p->drawLine(x-s, y-s, x, y+s);
-      p->drawLine(x, y+s, x+s, y-s);
+      {
+        QPointArray pts(3);
+
+        pts.putPoints( 0, 3, x-s, y-s, x, y+s, x+s, y-s );
+        p->setBrush(QBrush::NoBrush);
+        p->drawPolygon(pts);
+      }
       break;
     case 5:
-      p->drawLine(x-s, y+s, x+s, y+s);
-      p->drawLine(x-s, y+s, x, y-s);
-      p->drawLine(x, y-s, x+s, y+s);
+      {
+        QPointArray pts(3);
+
+        pts.putPoints( 0, 3, x-s, y+s, x, y-s, x+s, y+s );
+        p->setBrush(QBrush::NoBrush);
+        p->drawPolygon(pts);
+      }
       break;
     case 6:
       p->setBrush(QBrush::SolidPattern);
@@ -84,7 +94,56 @@ void KstPoint::draw(QPainter *p, int x, int y, int size) {
       p->drawLine(x-s, y, x+s, y);
       p->drawLine(x, y-s, x, y+s);
       break;
+    case 8:
+      p->drawLine(x-s, y-s, x+s, y+s);
+      p->drawLine(x-s, y+s, x+s, y-s);
+      p->drawLine(x-s, y, x+s, y);
+      p->drawLine(x, y-s, x, y+s);
+      break;    
+    case 9:
+      {
+        QPointArray pts(3);
+
+        pts.putPoints( 0, 3, x-s, y-s, x, y+s, x+s, y-s );
+        p->setBrush(QBrush::SolidPattern);
+        p->drawPolygon(pts);
+      }
+      break;
+    case 10:
+      {
+        QPointArray pts(3);
+
+        pts.putPoints( 0, 3, x-s, y+s, x, y-s, x+s, y+s );
+        p->setBrush(QBrush::SolidPattern);
+        p->drawPolygon(pts);
+      }
+      break;
+    case 11:
+      {
+        QPointArray pts(4);
+
+        pts.putPoints( 0, 4,   x+s, y,
+                               x, y+s,
+                               x-s, y,
+                               x, y-s );
+        p->setBrush(QBrush::NoBrush);
+        p->drawPolygon(pts);
+      }
+      break;
+    case 12:
+      {    
+        QPointArray pts(4);
+
+        pts.putPoints( 0, 4,   x+s, y,
+                               x, y+s,
+                               x-s, y,
+                               x, y-s );
+        p->setBrush(QBrush::SolidPattern);
+        p->drawPolygon(pts);
+      }
+      break;
   }
+
   p->setBrush(QBrush::NoBrush);
 }
 
@@ -94,7 +153,7 @@ int KstPoint::dim(QPainter *p) const {
   if (s < 1) {
     s = 1;
   }
-  
+
   return s;
 }
 
