@@ -30,6 +30,7 @@
 #include "curveappearancewidget.h"
 #include "curveplacementwidget.h"
 #include "kstpsddialog_i.h"
+#include "kstuinames.h"
 #include "kstvcurve.h"
 #include "kstviewwindow.h"
 #include "vectorselector.h"
@@ -38,14 +39,13 @@
 #define DTYPE "Power Spectrum"
 #include "dataobjectdialog.h"
 
-static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
+const QString& KstPsdDialogI::defaultTag = KGlobal::staticQString("<Auto Name>");
 
-KstPsdDialogI *KstPsdDialogI::_inst = 0L;
-static KStaticDeleter<KstPsdDialogI> _psInst;
+QGuardedPtr<KstPsdDialogI> KstPsdDialogI::_inst = 0L;
 
 KstPsdDialogI *KstPsdDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _psInst.setObject(new KstPsdDialogI);
+    _inst = new KstPsdDialogI(KstApp::inst());
   }
   return _inst;
 }
@@ -81,12 +81,12 @@ void KstPsdDialogI::_fillFieldsForEdit() {
 
   _vector->setSelection(DP->vTag());
 
-  /* set sample rate, Units, FFT len, and vector units */
+  // set sample rate, Units, FFT len, and vector units
   _kstFFTOptions->FFTLen->setValue(DP->len());
   _kstFFTOptions->SampRate->setText(QString::number(DP->freq()));
 
-  _kstFFTOptions->VectorUnits->setText(DP->VUnits);
-  _kstFFTOptions->RateUnits->setText(DP->RUnits);
+  _kstFFTOptions->VectorUnits->setText(DP->vUnits());
+  _kstFFTOptions->RateUnits->setText(DP->rUnits());
   _kstFFTOptions->Apodize->setChecked(DP->apodize());
   _kstFFTOptions->RemoveMean->setChecked(DP->removeMean());
   _kstFFTOptions->Interleaved->setChecked(DP->average());
@@ -102,16 +102,11 @@ void KstPsdDialogI::_fillFieldsForEdit() {
 }
 
 void KstPsdDialogI::_fillFieldsForNew() {
-  /* set tag name */
   _tagName->setText(defaultTag);
-
-  /* set the curve placement window  */
   _curvePlacement->update();
-
-  /* set up the fft options */
   _kstFFTOptions->update();
 
-  //for some reason the lower widget needs to be shown first to prevent overlapping?
+  // for some reason the lower widget needs to be shown first to prevent overlapping?
   _curveAppearance->hide();
   _curvePlacement->show();
   _curveAppearance->show();
@@ -134,7 +129,7 @@ bool KstPsdDialogI::new_I() {
     tag_name = KST::suggestPSDName(_vector->selectedVector());
   }
 
-  /* verify that the curve name is unique */
+  // verify that the curve name is unique
   if (KST::dataTagNameNotUnique(tag_name)) {
     _tagName->setFocus();
     return false;
@@ -153,8 +148,7 @@ bool KstPsdDialogI::new_I() {
               << "a non existant vector...." << endl;
   }
 
-
-  /* create the psd curve */
+  // create the psd curve
   if (!_kstFFTOptions->checkValues()) {
     return false;
   } else {
@@ -168,7 +162,7 @@ bool KstPsdDialogI::new_I() {
                             _kstFFTOptions->VectorUnits->text(),
                             _kstFFTOptions->RateUnits->text());
 
-    KstVCurvePtr vc = new KstVCurve(tag_name + "-curve", psd->vX(), psd->vY(), 0L, 0L, _curveAppearance->color());
+    KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(tag_name,true), psd->vX(), psd->vY(), 0L, 0L, 0L, 0L, _curveAppearance->color());
     vc->setHasPoints(_curveAppearance->showPoints());
     vc->setHasLines(_curveAppearance->showLines());
     vc->setHasBars(_curveAppearance->showBars());
@@ -176,6 +170,7 @@ bool KstPsdDialogI::new_I() {
     vc->setLineWidth(_curveAppearance->lineWidth());
     vc->setLineStyle(_curveAppearance->lineStyle());
     vc->setBarStyle(_curveAppearance->barStyle());
+    vc->setPointDensity(_curveAppearance->pointDensity());
 
     Kst2DPlotPtr plot;
     KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_curvePlacement->_plotWindow->currentText()));
@@ -185,27 +180,27 @@ bool KstPsdDialogI::new_I() {
     }
     if (w) {
       if (_curvePlacement->existingPlot()) {
-        /* assign curve to plot */
+        // assign curve to plot
         plot = kst_cast<Kst2DPlot>(w->view()->findChild(_curvePlacement->plotName()));
         if (plot) {
           plot->addCurve(vc.data());
-          w->view()->paint(P_DATA);
         }
       }
 
       if (_curvePlacement->newPlot()) {
-        /* assign curve to plot */
+        // assign curve to plot
         QString name = w->createPlot<Kst2DPlot>(KST::suggestPlotName());
         if (_curvePlacement->reGrid()) {
           w->view()->cleanup(_curvePlacement->columns());
         }
         plot = kst_cast<Kst2DPlot>(w->view()->findChild(name));
         if (plot) {
+          plot->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
+          plot->setYAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
           _curvePlacement->update();
           _curvePlacement->setCurrentPlot(plot->tagName());
           plot->addCurve(vc.data());
           plot->GenerateDefaultLabels();
-          w->view()->paint(P_DATA);
         }
       }
     }
@@ -221,9 +216,9 @@ bool KstPsdDialogI::new_I() {
   return true;
 }
 
-/* returns true if succesful */
+// returns true if succesful
 bool KstPsdDialogI::edit_I() {
-  /* verify that the curve name is unique */
+  // verify that the curve name is unique
   if (_tagName->text() != DP->tagName() && KST::dataTagNameNotUnique(_tagName->text())) {
     return false;
   }
@@ -244,13 +239,12 @@ bool KstPsdDialogI::edit_I() {
   DP->setFreq(_kstFFTOptions->SampRate->text().toDouble());
   DP->setLen(_kstFFTOptions->FFTLen->text().toInt());
 
-  DP->VUnits = _kstFFTOptions->VectorUnits->text();
-  DP->RUnits = _kstFFTOptions->RateUnits->text();
+  DP->setVUnits(_kstFFTOptions->VectorUnits->text());
+  DP->setRUnits(_kstFFTOptions->RateUnits->text());
 
   DP->setApodize(_kstFFTOptions->Apodize->isChecked());
   DP->setRemoveMean(_kstFFTOptions->RemoveMean->isChecked());
   DP->setAverage(_kstFFTOptions->Interleaved->isChecked());
-  DP->update();
 
   DP->writeUnlock();
 
