@@ -29,7 +29,6 @@
 #include "kstdebug.h"
 #include "kstrvector.h"
 
-//
 // ReqNF <=0 means read from ReqF0 to end of File
 // ReqF0 < means start at EndOfFile-ReqNF.
 //
@@ -39,19 +38,18 @@
 //    1         ??       illegal: fixed in checkIntegrity
 //  > 1        < 0       read the last ReqNF frames from the file
 //  > 1        >=0       Read ReqNF frames starting at frame ReqF0
-//
 
 /** Create a KstRVector: raw data from a file */
 KstRVector::KstRVector(KstDataSourcePtr in_file, const QString &in_field,
                        const QString &in_tag, int in_f0, int in_n,
                        int skip, bool in_DoSkip, bool in_DoAve)
-: KstVector() {
-  setTagName(in_tag);
+: KstVector(in_tag) {
   commonRVConstructor(in_file, in_field, in_f0, in_n, skip,
-		      in_DoSkip, in_DoAve);
+      in_DoSkip, in_DoAve);
 }
 
-KstRVector::KstRVector(QDomElement &e, const QString &o_file,
+
+KstRVector::KstRVector(const QDomElement &e, const QString &o_file,
                        int o_n, int o_f, int o_s, bool o_ave)
 : KstVector() {
   KstDataSourcePtr in_file;
@@ -116,10 +114,13 @@ KstRVector::KstRVector(QDomElement &e, const QString &o_file,
   commonRVConstructor(in_file, in_field, in_f0, in_n, in_skip, in_DoSkip, in_DoAve);
 }
 
+
 void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
                                      const QString &in_field, int in_f0,
                                      int in_n, int in_skip, bool in_DoSkip,
                                      bool in_DoAve) {
+  _saveable = true;
+  _dontUseSkipAccel = false;
   _numSamples = 0;
   _scalars["sum"]->setValue(0.0);
   _scalars["sumsquared"]->setValue(0.0);
@@ -153,8 +154,6 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
   if (!in_file) {
     KstDebug::self()->log(i18n("Data file for vector %1 was not opened.").arg(tagName()), KstDebug::Warning);
   }
-
-  update(-1);
 }
 
 
@@ -170,6 +169,7 @@ void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
     Skip = 1;
   }
 
+  _dontUseSkipAccel = false;
   _file = in_file;
   ReqF0 = in_f0;
   ReqNF = in_n;
@@ -190,6 +190,7 @@ void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
 
 }
 
+
 void KstRVector::changeFile(KstDataSourcePtr in_file) {
   if (!in_file) {
     KstDebug::self()->log(i18n("Data file for vector %1 was not opened.").arg(tagName()), KstDebug::Warning);
@@ -204,6 +205,7 @@ void KstRVector::changeFile(KstDataSourcePtr in_file) {
   }
 }
 
+
 void KstRVector::changeFrames(int in_f0, int in_n,
                               int in_skip, bool in_DoSkip,
                               bool in_DoAve) {
@@ -229,6 +231,18 @@ void KstRVector::changeFrames(int in_f0, int in_n,
   }
 }
 
+
+void KstRVector::setFromEnd() {
+  ReqF0 = -1;
+  if (ReqNF < 2) {
+    ReqNF = numFrames();
+    if (ReqNF < 2) {
+      ReqF0 = 0;
+    }
+  }
+}
+
+
 KstRVector::~KstRVector() {  
   _file = 0;
 
@@ -238,48 +252,60 @@ KstRVector::~KstRVector() {
   }
 }
 
+
 bool KstRVector::readToEOF() const {
   return ReqNF <= 0;
 }
 
+
 bool KstRVector::countFromEOF() const {
   return ReqF0 < 0;
 }
 
+
 /** Return Starting Frame of Vector */
 int KstRVector::startFrame() const {
   return F0;
 }
 
+
 /** Return frames per skip to read */
 int KstRVector::skip() const {
   return DoSkip ? Skip : 0;
 }
 
+
 bool KstRVector::doSkip() const {
   return DoSkip;
 }
 
+
 bool KstRVector::doAve() const {
   return DoAve;
 }
 
+
 /** Return frames held in Vector */
 int KstRVector::numFrames() const {
   return NF;
 }
 
+
 int KstRVector::reqNumFrames() const {
   return ReqNF;
 }
 
+
 int KstRVector::reqStartFrame() const {
   return ReqF0;
 }
 
+
 /** Save vector information */
 void KstRVector::save(QTextStream &ts, const QString& indent, bool saveAbsolutePosition) {
   if (_file) {    
+    ts << "  <vector>" << endl;
+
     ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
     _file->readLock();
     ts << indent << "<filename>" << QStyleSheet::escape(_file->fileName()) << "</filename>" << endl;
@@ -298,9 +324,11 @@ void KstRVector::save(QTextStream &ts, const QString& indent, bool saveAbsoluteP
         ts << indent << "<doAve/>" << endl;
       }
     }
+    ts << "  </vector>" << endl;
   }
 }
 
+
 /** return the name of the file */
 QString KstRVector::filename() const {
   QString rc;
@@ -312,11 +340,13 @@ QString KstRVector::filename() const {
   return rc;
 }
 
+
 /** return the field */
 const QString& KstRVector::field() const {
   return _field;
 }
 
+
 QString KstRVector::label() const {
   bool ok;
   QString label;
@@ -337,7 +367,9 @@ QString KstRVector::label() const {
   return label;
 }
 
+
 void KstRVector::reset() { // must be called with a lock
+  _dontUseSkipAccel = false;
   if (_file) {
     SPF = _file->samplesPerFrame(_field);
   }
@@ -347,6 +379,7 @@ void KstRVector::reset() { // must be called with a lock
   _dirty = true;
 }
 
+
 void KstRVector::checkIntegrity() {
   if (DoSkip && Skip < 1) {
     Skip = 1;
@@ -356,16 +389,12 @@ void KstRVector::checkIntegrity() {
     reset();
   }
 
-  //
   // if it looks like we have a new file, reset
-  //
   if (_file && (SPF != _file->samplesPerFrame(_field) || _file->frameCount(_field) < NF)) {
     reset();
   }
 
-  //
-  // check for illeagal NF and F0 values
-  //
+  // check for illegal NF and F0 values
   if (ReqNF < 1 && ReqF0 < 0) {
     ReqF0 = 0; // for this illegal case, read the whole file
   }
@@ -377,39 +406,42 @@ void KstRVector::checkIntegrity() {
 
 
 /** Update an RVECTOR */
-/** if update_counter < 1, the update will happen */
-/** if update_counter == LastUpdateCouner, the update won't happen */
-
 KstObject::UpdateType KstRVector::update(int update_counter) {
-  if (KstObject::checkUpdateCounter(update_counter)) {
-    return NO_CHANGE;
+  bool force = dirty();
+  setDirty(false);
+  if (KstObject::checkUpdateCounter(update_counter) && !force) {
+    return lastUpdateResult();
   }
 
   if (_file) {
     _file->writeLock();
   }
-  KstObject::UpdateType rc = doUpdate();
+  KstObject::UpdateType rc = doUpdate(force);
   if (_file) {
     _file->writeUnlock();
   }
-  
-  return rc;
+
+  setDirty(false);
+  return setLastUpdateResult(rc);
 }
 
+
 KstObject::UpdateType KstRVector::doUpdate(bool force) {
-  int i, j, k, shift, n_read=0;
+  int i, k, shift, n_read=0;
   int ave_nread;
   int new_f0, new_nf, tmp_fn;
 
   checkIntegrity();
 
+  if (DoSkip && Skip < 2 && SPF == 1) {
+    DoSkip = false;
+  }
+
   if (!_file) {
     return NO_CHANGE;
   }
 
-  //
   // set new_nf and new_f0
-  //
   int fc = _file->frameCount(_field);
   if (ReqNF < 1) { // read to end of file
     new_f0 = ReqF0;
@@ -432,21 +464,18 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   }
 
   if (DoSkip) {
-    //
-    // change new_f0 and new_nf so they both lie on skip boundaries
-    //
-    tmp_fn = ((new_f0+new_nf)/Skip) * Skip;
-    new_f0 = (((new_f0-1)/Skip)+1) * Skip;
-    new_nf = tmp_fn-new_f0;
+      // change new_f0 and new_nf so they both lie on skip boundaries
+      if (new_f0 != 0) {
+        new_f0 = (((new_f0-1)/Skip)+1)*Skip;
+      }
+      new_nf = (new_nf/Skip)*Skip;
   }
 
   if (NF == new_nf && F0 == new_f0 && !force) {
     return NO_CHANGE;
   }
 
-  //
   // shift vector if necessary
-  //
   if (new_f0 < F0 || new_f0 >= F0 + NF) { // No useful data around.
     reset();
   } else { // shift stuff rather than re-read
@@ -466,43 +495,70 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   }
 
   if (DoSkip) {
-    //
     // reallocate V if necessary
-    //
+    //kdDebug() << "new_nf = " << new_nf << " and skip = " << Skip << " so new_nf/Skip+1 = " << (new_nf / Skip + 1) << endl;
     if (new_nf / Skip != _size) {
       bool rc = resize(new_nf/Skip);
       if (!rc) {
         // FIXME: handle failed resize
       }
     }
-    n_read = 0;
-    /** read each sample from the File */
-    for (i = NF, j = 0; i < new_nf; i += Skip, j++) {
+    // for debugging: _dontUseSkipAccel = true;
+    if (!_dontUseSkipAccel) {
+      int rc;
+      int lastRead = -1;
       if (DoAve) {
-        /* enlarge AveReadBuf if necessary */
-        if (N_AveReadBuf < Skip*SPF) {
-          N_AveReadBuf = Skip*SPF;
-          AveReadBuf = static_cast<double*>(realloc(AveReadBuf, N_AveReadBuf*sizeof(double)));
-          if (!AveReadBuf) {
-            // FIXME: handle failed resize
+        // We don't support boxcar inside data sources yet.
+        _dontUseSkipAccel = true;
+      } else {
+        rc = _file->readField(_v + _numSamples, _field, new_f0, (new_nf - NF)/Skip, Skip, &lastRead);
+        if (rc != -9999) {
+          //kdDebug() << "USED SKIP FOR READ - " << _field << " - rc=" << rc << " for Skip=" << Skip << " s=" << new_f0 << " n=" << (new_nf - NF)/Skip << endl;
+          if (rc >= 0) {
+            n_read = rc;
+          } else {
+            n_read = 0;
           }
+        } else {
+          _dontUseSkipAccel = true;
         }
-        ave_nread = _file->readField(AveReadBuf, _field, new_f0+i, Skip);
-        for (k = 1; k < ave_nread; k++) {
-          AveReadBuf[0] += AveReadBuf[k];
-        }
-        if (ave_nread > 0) {
-          _v[_numSamples + j] = AveReadBuf[0]/double(ave_nread);
-          n_read++;
+      }
+    }
+    if (_dontUseSkipAccel) {
+      n_read = 0;
+      /** read each sample from the File */
+      //kdDebug() << "NF = " << NF << " numsamples = " << _numSamples << " new_f0 = " << new_f0 << endl;
+      double *t = _v + _numSamples;
+      int new_nf_Skip = new_nf - Skip;
+      if (DoAve) {
+        for (i = NF; new_nf_Skip >= i; i += Skip) {
+          /* enlarge AveReadBuf if necessary */
+          if (N_AveReadBuf < Skip*SPF) {
+            N_AveReadBuf = Skip*SPF;
+            AveReadBuf = static_cast<double*>(realloc(AveReadBuf, N_AveReadBuf*sizeof(double)));
+            if (!AveReadBuf) {
+              // FIXME: handle failed resize
+            }
+          }
+          ave_nread = _file->readField(AveReadBuf, _field, new_f0+i, Skip);
+          for (k = 1; k < ave_nread; k++) {
+            AveReadBuf[0] += AveReadBuf[k];
+          }
+          if (ave_nread > 0) {
+            *t = AveReadBuf[0]/double(ave_nread);
+            n_read++;
+          }
+          ++t;
         }
       } else {
-        n_read += _file->readField(_v + _numSamples + j, _field, new_f0 + i, -1);
+        for (i = NF; new_nf_Skip >= i; i += Skip) {
+          //kdDebug() << "readField " << _field << " start=" << new_f0 + i << " n=-1" << endl;
+          n_read += _file->readField(t++, _field, new_f0 + i, -1);
+        }
       }
     }
   } else {
-    //
     // reallocate V if necessary
-    //
     if ((new_nf - 1)*SPF + 1 != _size) {
       bool rc = resize((new_nf - 1)*SPF + 1);
       if (!rc) {
@@ -514,11 +570,13 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       NF--; /* last frame read was only partially read... */
     }
 
-    //
     // read the new data from file
-    //
-    n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF - 1);
-    n_read += _file->readField(_v+(new_nf-1)*SPF, _field, new_f0 + new_nf-1, -1);
+    if (_file->samplesPerFrame(_field) > 1) {
+      n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF - 1);
+      n_read += _file->readField(_v+(new_nf-1)*SPF, _field, new_f0 + new_nf-1, -1);
+    } else {
+      n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF);
+    }
   }
 
   NumNew = _size - _numSamples;
@@ -526,7 +584,6 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   F0 = new_f0;
   _numSamples += n_read;
 
-  //
   // if for some reason (eg, realtime reading an nfs mounted
   // dirfile) not all of the data was read, the data will never
   // be read; the samples will be filled in with the last data
@@ -534,9 +591,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   // This is bad - I think it will be worthwhile
   // to add blocking w/ timeout to KstFile.
   // As a first fix, mount all nsf mounted dirfiles with "-o noac"
-  //
   _dirty = false;
   if (_numSamples != _size && !(_numSamples == 0 && _size == 1)) {
+    //kdDebug() << "SET DIRTY since _numSamples = " << _numSamples << " but _size = " << _size << endl;
     _dirty = true;
     for (i = _numSamples; i < _size; i++) {
       _v[i] = _v[0];
@@ -550,7 +607,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
     NumShifted = _size;
   }
 
-  return KstVector::update(-1);
+  return KstVector::internalUpdate(UPDATE);
 }
 
 
@@ -559,6 +616,7 @@ int KstRVector::samplesPerFrame() const {
   return SPF;
 }
 
+
 /** return true if it has a valid file and field, or false otherwise */
 bool KstRVector::isValid() const {
   if (_file) {
@@ -570,6 +628,7 @@ bool KstRVector::isValid() const {
   return false;
 }
 
+
 int KstRVector::fileLength() const {
   if (_file) {
     _file->readLock();
@@ -582,24 +641,20 @@ int KstRVector::fileLength() const {
   return 0;
 }
 
+
 void KstRVector::reload() {
   if (_file) {
     _file->writeLock();
-    if (_file->reset()) {
-      //
-      // try the efficient way first
-      //
+    if (_file->reset()) { // try the efficient way first
       reset();
-    } else {
-      //
-      // the inefficient way
-      //
+    } else { // the inefficient way
       KstDataSourcePtr newsrc = KstDataSource::loadSource(_file->fileName(), _file->fileType());
       assert(newsrc != _file);
       if (newsrc) {
         _file->writeUnlock();
         KST::dataSourceList.lock().writeLock();
         KST::dataSourceList.remove(_file);
+        _dontUseSkipAccel = false;
         _file = newsrc;
         _file->writeLock();
         KST::dataSourceList.append(_file);
@@ -611,6 +666,7 @@ void KstRVector::reload() {
   }
 }
 
+
 KstDataSourcePtr KstRVector::dataSource() const {
   return _file;
 }
