@@ -21,10 +21,12 @@
 #include <stdlib.h>
 
 #include <qdeepcopy.h>
+#include <qstylesheet.h>
 
 #include <kdebug.h>
 #include <klocale.h>
 #include "kstdatacollection.h"
+#include "kstdefaultnames.h"
 #include "kstmath.h"
 #include "kstvector.h"
 
@@ -48,9 +50,13 @@ KstVector::KstVector(const QString& name, int size, bool isScalarList)
 : KstObject(), _nsum(0), _scalars(isScalarList ? 0 : 11) {
   //kdDebug() << "+++ CREATING VECTOR: " << (void*) this << endl;
   KstObject::setTagName(name);
+  _editable = false;
   NumShifted = 0;
   NumNew = 0;
   _isScalarList = isScalarList;
+  _label = QString::null;
+
+  _saveable = false;
 
   if (size <= 0) {
     size = INITSIZE;
@@ -69,13 +75,19 @@ KstVector::KstVector(const QString& name, int size, bool isScalarList)
   }
 
   _v = static_cast<double*>(KST::malloc(size*sizeof(double)));
-  _size = size;
+  if (!_v) { // Malloc failed
+    _v = static_cast<double*>(KST::malloc(sizeof(double)));
+    _size = 1;
+  } else {
+    _size = size;
+  }
   _is_rising = false;
 
   CreateScalars();
   zero();
 }
 
+
 KstVector::~KstVector() {
   //kdDebug() << "+++ DELETING VECTOR: " << (void*) this << endl;
   KST::scalarList.lock().writeLock();
@@ -92,56 +104,62 @@ KstVector::~KstVector() {
 }
 
 
+#define GENERATE_INTERPOLATION              \
+  double fj, fdj;                           \
+  int j;                                    \
+                                            \
+  /** Limits checks - optional? **/         \
+  if (in_i < 0) {                           \
+    return _v[0];                           \
+  }                                         \
+                                            \
+  if (in_i >= ns_i - 1) {                   \
+    return _v[_size - 1];                   \
+  }                                         \
+                                            \
+  /** speedup check **/                     \
+  if (ns_i == _size) { /* no extrapolating or decimating needed */  \
+    return _v[in_i];                        \
+  }                                         \
+                                            \
+  fj = in_i * double(_size - 1) / double(ns_i-1); /* scaled index */ \
+                                            \
+  j = int(floor(fj)); /* index of sample one lower */ \
+                                            \
+  /* This is optimized to avoid unnecessary isnan calls! */  \
+  if (_v[j + 1] != _v[j + 1]) {             \
+    if (KST_ISNAN(_v[j + 1])) {             \
+      return KST::NOPOINT;                  \
+    }                                       \
+  }                                         \
+  if (_v[j] != _v[j]) {                     \
+    if (KST_ISNAN(_v[j])) {                 \
+      return KST::NOPOINT;                  \
+    }                                       \
+  }                                         \
+                                            \
+  fdj = fj - float(j); /* fdj is fraction between _v[j] and _v[j+1] */ \
+                                            \
+  return _v[j + 1] * fdj + _v[j] * (1.0 - fdj);
+
+
 // FIXME: optimize me - possible that floor() (especially) and isnan() are
 //        expensive here.
 /** Return v[i], i is sample number, interpolated to have ns_i total
     samples in vector */
-double KstVector::interpolate(int in_i, int ns_i) {
-  double fj, fdj;
-  int j;
-
-  /** Limits checks - optional? **/
-  if (in_i < 0) {
-    return _v[0];
-  }
-
-  if (in_i >= ns_i - 1) {
-    return _v[_size - 1];
-  }
-
-  /** speedup check **/
-  if (ns_i == _size) { /* no extrapolating or decimating needed */
-    return _v[in_i];
-  }
-
-  fj = in_i * double(_size - 1) / double(ns_i-1); //scaled index
-
-  j = int(floor(fj));	       // index of sample one lower
-
-  // This is optimized to avoid unnecessary isnan calls!
-  if (_v[j + 1] != _v[j + 1]) {
-    if (KST_ISNAN(_v[j + 1])) {
-      return KST::NOPOINT;
-    }
-  }
-  if (_v[j] != _v[j]) {
-    if (KST_ISNAN(_v[j])) {
-      return KST::NOPOINT;
-    }
-  }
-
-  fdj = fj - float(j);     // fdj is fraction between _v[j] and _v[j+1]
-
-  return _v[j + 1] * fdj + _v[j] * (1.0 - fdj);
+double KstVector::interpolate(int in_i, int ns_i) const {
+  GENERATE_INTERPOLATION
 }
 
 
-double KstVector::value(int i) {
-  if (i < 0) {//can't look before begining
-    return 0.0;
-  }
+double kstInterpolate(double *_v, int _size, int in_i, int ns_i) {
+  GENERATE_INTERPOLATION
+}
 
-  if (i >= _size) {// can't look past end
+#undef GENERATE_INTERPOLATION
+
+double KstVector::value(int i) {
+  if (i < 0 || i >= _size) { // can't look before beginning or past end
     return 0.0;
   }
   return _v[i];
@@ -204,14 +222,11 @@ void KstVector::updateScalars() {
   }
 }
 
-double* KstVector::realloced(double *memptr, int newSize, bool bNew) {
+
+double* KstVector::realloced(double *memptr, int newSize) {
   double *old = _v;
   _v = memptr;
-  if (bNew) {
-    NumNew = newSize;
-  } else {
-    NumNew = newSize - _size;
-  }
+  NumNew = newSize - _size;
   _size = newSize;
   updateScalars();
   return old;
@@ -219,6 +234,7 @@ double* KstVector::realloced(double *memptr, int newSize, bool bNew) {
 
 
 void KstVector::zero() {
+  setDirty();
   _ns_min = _ns_max = 0.0;
   for (int i = 0; i < _size; i++) {
     _v[i] = 0.0;
@@ -244,40 +260,50 @@ bool KstVector::resize(int sz, bool reinit) {
       }
 #endif
     }
+#else
+    abort();  // avoid unpleasant surprises
 #endif
     _size = sz;
     updateScalars();
   }
 
+  setDirty();
   return true;
 }
 
 
 KstObject::UpdateType KstVector::update(int update_counter) {
+  bool force = dirty();
+  setDirty(false);
+
+  if (KstObject::checkUpdateCounter(update_counter) && !force) {
+    return lastUpdateResult();
+  }
+
+  KstObject::UpdateType providerRC = NO_CHANGE;
+
+  if (update_counter > 0) {
+    KstObjectPtr prov = KstObjectPtr(_provider);
+    if (prov) {
+      providerRC = prov->update(update_counter);
+      if (!force && providerRC == KstObject::NO_CHANGE) {
+        return setLastUpdateResult(providerRC);
+      }
+    }
+  }
+
+  KstObject::UpdateType rc = internalUpdate(providerRC);
+  setDirty(false);
+  return rc;
+}
+
+
+KstObject::UpdateType KstVector::internalUpdate(KstObject::UpdateType providerRC) {
   int i, i0;
   double max, min, sum, sum2, minpos, last, v;
   double last_v;
   double dv2 = 0.0, dv, no_spike_max_dv;
-
   max = min = sum = sum2 = minpos = last = 0.0;
-  if (KstObject::checkUpdateCounter(update_counter)) {
-    return KstObject::NO_CHANGE;
-  }
-
-  if (update_counter != -1) {
-    KstObjectPtr prov = KstObjectPtr(_provider);
-    if (prov) {
-      prov->update(update_counter);
-      /*  Update anyway, it's never possible to get NO_CHANGE because of
-       *  the guard above
-      KstObject::UpdateType u = prov->update(update_counter);
-      if (u == KstObject::NO_CHANGE) {
-        return u;
-      }
-      */
-    }
-  }
-
   _nsum = 0;
   if (_size > 0) {
     _is_rising = true;
@@ -297,7 +323,7 @@ KstObject::UpdateType KstVector::update(int update_counter) {
       _ns_max = _ns_min = 0;
 
       updateScalars();
-      return UPDATE;
+      return setLastUpdateResult(providerRC);
     }
 
     i0 = i;
@@ -337,10 +363,8 @@ KstObject::UpdateType KstVector::update(int update_counter) {
 
         if (v > max) {
           max = v;
-        } else {
-          if (v < min) {
-            min = v;
-          }
+        } else if (v < min) {
+          min = v;
         }
         if (v < minpos && v > 0) {
             minpos = v;
@@ -387,17 +411,17 @@ KstObject::UpdateType KstVector::update(int update_counter) {
 
     updateScalars();
 
-    return UPDATE;
+    return setLastUpdateResult(providerRC);
   }
 
-  return NO_CHANGE;
+  return setLastUpdateResult(NO_CHANGE);
 }
 
 void KstVector::save(QTextStream &ts, const QString& indent, bool saveAbsolutePosition) {
-  Q_UNUSED(ts)
-  Q_UNUSED(indent)
   Q_UNUSED(saveAbsolutePosition)
-  abort();
+    Q_UNUSED(ts)
+    Q_UNUSED(indent)
+    abort();
 }
 
 
@@ -406,23 +430,28 @@ void KstVector::setTagName(const QString& newTag) {
   RenameScalars();
 }
 
+
 void KstVector::setNewAndShift(int inNew, int inShift) {
   NumNew = inNew;
   NumShifted = inShift;
 }
 
+
 QString KstVector::label() const {
-  return tagName(); // default
+  return _label; // default
 }
 
+
 QString KstVector::fileLabel() const {
   return QString::null;
 }
 
+
 double *const KstVector::value() const {
   return _v;
 }
 
+
 double KstVector::min() const {
   if (!_isScalarList) {
     return _scalars["min"]->value();
@@ -431,6 +460,7 @@ double KstVector::min() const {
   }
 }
 
+
 double KstVector::max() const {
   if (!_isScalarList) {
     return _scalars["max"]->value();
@@ -439,6 +469,7 @@ double KstVector::max() const {
   }
 }
 
+
 double KstVector::mean() const {
   if (!_isScalarList) {
     return _scalars["mean"]->value();
@@ -447,6 +478,7 @@ double KstVector::mean() const {
   }
 }
 
+
 double KstVector::minPos() const {
   if (!_isScalarList) {
     return _scalars["minpos"]->value();
@@ -455,26 +487,27 @@ double KstVector::minPos() const {
   }
 }
 
-int KstVector::sampleCount() const {
-  return _size;
-}
 
 int KstVector::numNew() const {
   return NumNew;
 }
 
+
 int KstVector::numShift() const {
   return NumShifted;
 }
 
+
 void KstVector::newSync() {
   NumNew = NumShifted = 0;
 }
 
+
 int KstVector::length() const {
   return _size;
 }
 
+
 KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QString &tag) {
   if (n < 2) {
     n = 2;
@@ -493,17 +526,12 @@ KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QStrin
 
   QString t = tag;
   if (t.isEmpty()) {
-    KST::vectorList.lock().readLock();
-    t = "V" + QString::number(KST::vectorList.count() + 1) + "-" + "X(" + QString::number(x0) + ".." + QString::number(x1) + ")";
-    KST::vectorList.lock().readUnlock();
-  }
-
-  while (KST::vectorTagNameNotUnique(t, false)) {
-    t += "'";
+    t = KST::suggestVectorName("X(" + QString::number(x0) + ".." + QString::number(x1) + ")");
   }
 
   KstVectorPtr xv = new KstVector(t, n);
   KST::addVectorToList(xv);
+  xv->_saveable = false;
 
   for (int i = 0; i < n; i++) {
     xv->value()[i] = x0 + double(i) * (x1 - x0) / (n - 1);
@@ -535,10 +563,40 @@ bool KstVector::deleteDependents() {
       KST::dataObjectList.lock().writeUnlock();
       dop->deleteDependents();
     }
-  } 
+  }
 
   return true;
 }
 
+
+void KstVector::setLabel(const QString& label_in) {
+  _label = label_in;
+}
+
+
+int KstVector::getUsage() const {
+  int adj = 0;
+  for (QDictIterator<KstScalar> it(_scalars); it.current(); ++it) {
+    adj += it.current()->getUsage() - 1;
+  }
+  return KstObject::getUsage() + adj;
+}
+
+
+bool KstVector::saveable() const {
+  return _saveable;
+}
+
+
+bool KstVector::editable() const {
+  return _editable;
+}
+
+
+void KstVector::setEditable(bool editable) {
+  _editable = editable;
+}
+
+
 #include "kstvector.moc"
 // vim: et sw=2 ts=2
