@@ -47,6 +47,7 @@
 #include "kmdiiterator.h"
 #include "kmdilistiterator.h"
 #include "kmdinulliterator.h"
+#include "../kst_export.h"
 
 class QTimer;
 class QPopupMenu;
@@ -201,7 +202,7 @@ public:
   * deletes the view object. So if your application wants to control that by itself, call removeWindowFromMdi()
   * and call delete by yourself. See also KMdiChildView::closeEvent() for tat issue.
   */
-class KMdiMainFrm : public KParts::DockMainWindow
+class KST_EXPORT KMdiMainFrm : public KParts::DockMainWindow
 {
    friend class KMdiChildView;
    friend class KMdiTaskBar;
