@@ -1,5 +1,5 @@
 /***************************************************************************
-                                  main.cpp 
+                                  main.cpp
                              -------------------
     begin                : Tue Aug 22 13:46:13 CST 2000
     copyright            : (C) 2000 by Barth Netterfield
@@ -18,6 +18,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include <qstringlist.h>
+
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
@@ -33,6 +35,8 @@
 #include "kstlegend.h"
 #include "kstpsd.h"
 #include "kstrvector.h"
+#include "kstsvector.h"
+#include "kstavector.h"
 #include "kstvcurve.h"
 #include "kstversion.h"
 #include "plugincollection.h"
@@ -43,7 +47,7 @@
 // properly and cleanly deal with all of this.
 
 static const char description[] = I18N_NOOP("Kst: a data viewing program.");
-
+static QStringList strErrors;
 
 static KCmdLineOptions options[] = {
   {"F <dataFile>", I18N_NOOP("Specify data file: used to override a kst file default"), "|"},
@@ -67,6 +71,7 @@ static KCmdLineOptions options[] = {
   { "g",      I18N_NOOP("Provide a legend box"),0},
   { "print <file>",  I18N_NOOP("Print to file and exit"),"<none>"},
   { "png <file>",  I18N_NOOP("Save as a png file and exit"),"<none>"},
+  { "nq",     I18N_NOOP("Bypass the quickstart dialog"), 0},
   { "+[Files]", I18N_NOOP("Data files (if -y given) or *.kst file"), 0},
   KCmdLineLastOption
 };
@@ -99,13 +104,12 @@ static void CheckForCMDErrors(KCmdLineArgs *args) {
 
   if (args->getOption("n").toInt(&nOK) == 0) {
     kdError() << i18n("Exiting because '-n 0' requests vectors with no data in them.") << endl;
-    exit(0); // FIXME: can cause crashes
+    exit(0); // ugly but now safe
   }
   if (args->getOption("n").toInt(&nOK)>0) {
-    if (args->getOption("n").toInt(&nOK) <
-        2*args->getOption("s").toInt(&nOK)) {
+    if (args->getOption("n").toInt(&nOK) < 2*args->getOption("s").toInt(&nOK)) {
       kdError() << i18n("Requested data skipping would leave vector with fewer than two points.") << endl;
-      exit(0); // FIXME: can cause crashes
+      exit(0); // ugly but now safe
     }
   }
 }
@@ -118,23 +122,37 @@ static void CheckForCMDErrors(KCmdLineArgs *args) {
 /****************************************************************/
 static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
   bool nOK;
+
   in.n = args->getOption("n").toInt(&nOK);
   in.f = args->getOption("f").toInt(&nOK);
   in.rate = args->getOption("r").toDouble(&nOK);
   in.len = args->getOption("l").toInt(&nOK);
-  if (in.n<-1) in.n = -1;
-  if (in.f<-1) in.f = -1;
-  if (in.rate<=0) in.rate = 1.0;
-  if (in.len<3) in.len = 3;
-  if (in.len>23) in.len = 23;
+  if (in.n < -1) {
+    in.n = -1;
+  }
+  if (in.f < -1) {
+    in.f = -1;
+  }
+  if (in.rate <= 0) {
+    in.rate = 1.0;
+  }
+  if (in.len < 3) {
+    in.len = 3;
+  }
+  if (in.len > 23) {
+    in.len = 23;
+  }
 
   in.has_points = args->isSet("d");
   in.n_cols = args->getOption("m").toInt(&nOK);
-  if (in.n_cols>0) {
-    if (args->count()>0) {
+  if (in.n_cols > 0) {
+    if (in.n_cols > 10) {
+      in.n_cols = 10;
+    }
+    if (args->count() > 0) {
       in.sep_plots = true;
       in.n_plots = args->count() * (n_y);
-    } else if (args->getOptionList("ye").count()>0) {
+    } else if (args->getOptionList("ye").count() > 0) {
       in.sep_plots = true;
       in.n_plots = args->getOptionList("ye").count();
     }
@@ -146,9 +164,9 @@ static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
 
   in.n_rows = (in.n_plots-1)/in.n_cols + 1;
 
-  /* set skip options */
+  // set skip options
   in.skip = args->getOption("s").toInt(&nOK);
-  if (in.skip<1) {
+  if (in.skip < 1) {
     in.skip = 0;
     in.doskip = false;
     in.doave = false;
@@ -157,7 +175,7 @@ static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
     in.doave = args->isSet("a");
   }
 
-  /* set Units for PSDs */
+  // set Units for PSDs
   in.VUnits = args->getOption("yu");
   in.RUnits = args->getOption("ru");
   in.dolegend = args->isSet("g");
@@ -176,7 +194,7 @@ static void CreatePlots(InType &in, KstTopLevelViewPtr tlv) {
   int size_boost;
   int count;
   int handled;
-  
+
   kst->slotUpdateProgress( 0, 0, creatingPlots );
   count = in.n_plots;
   handled = 0;
@@ -211,14 +229,15 @@ static void CreatePlots(InType &in, KstTopLevelViewPtr tlv) {
 static void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *ok) {
   *ok = true;
   QStringList fields = QStringList::split( QChar(':'), xeqS );
-  if (fields.count()!=3) {
-    *ok=false;
-    *n=2;
+  if (fields.count() != 3) {
+    *ok = false;
+    *n = 2;
     *min = -1;
     *max = 1;
     return;
   }
   QString f;
+
   f = *fields.at(0);
   *min = f.toDouble(ok);
   f = *fields.at(1);
@@ -226,8 +245,12 @@ static void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *o
   f = *fields.at(2);
   *n = f.toInt(ok);
 
-  if (*n<2) *ok=false;
-  if (*min>=*max) *ok = false;
+  if (*n < 2) {
+    *ok=false;
+  }
+  if (*min >= *max) {
+    *ok = false;
+  }
 }
 
 /******************************************************************/
@@ -249,14 +272,17 @@ static KstRVector *GetOrCreateVector(const QString& field, KstDataSourcePtr file
       return V;
     }
   }
-  V = new KstRVector(file, field, QString("V") + QString::number(1 + i_v++)
-       + "-" + field, in.f, in.n, in.skip, in.doskip, in.doave);
-  if (!V->isValid()) {
-    kdError() << i18n("Invalid field: %1 in file %2").arg(V->field()).arg(V->filename()) << endl;
-    exit(0); // FIXME: this can cause crashes
+
+  V = new KstRVector(file, field, KST::suggestVectorName(field), in.f, in.n, in.skip, in.doskip, in.doave);
+  if (V->isValid()) {
+    KST::addVectorToList(V);
+  } else {
+    strErrors.append(i18n("Failed to create vector '%1' from file '%2'.").arg(field).arg(file->fileName()));
+
+    delete V;
+    V = 0L;
   }
 
-  KST::addVectorToList(V);
   return V;
 }
 
@@ -281,7 +307,9 @@ static void ProcessEq(QString &eq, KstDataSourcePtr file, InType &in, bool *ok)
     if (i1>0) {
       field = eq.mid(i0+1, i1-i0-1);
       v = GetOrCreateVector(field, file, in);
-      eq.replace(i0,i1-i0+1, "["+v->tagName()+"]");
+      if (v) {
+        eq.replace(i0,i1-i0+1, "["+v->tagName()+"]");
+       }
     }
     i0++;
   }
@@ -303,9 +331,9 @@ int main(int argc, char *argv[]) {
 
   KAboutData aboutData("kst", I18N_NOOP("Kst"),
                        KSTVERSION, description, KAboutData::License_GPL,
-                       I18N_NOOP("(c) 2000-2004 Barth Netterfield"),
+                       I18N_NOOP("(c) 2000-2005 Barth Netterfield"),
                        0,
-                       "http://extragear.kde.org/apps/kst.php");
+                       "http://extragear.kde.org/apps/kst/");
   aboutData.addAuthor("Barth Netterfield",
                       I18N_NOOP("Original author and maintainer."),
                       "netterfield@astro.utoronto.ca",
@@ -322,6 +350,10 @@ int main(int argc, char *argv[]) {
                       I18N_NOOP("University of British Columbia"),
                       "",
                       "");
+  aboutData.addAuthor("Nicolas Brisset",
+                      "",
+                      "",
+                      "");
 
   KCmdLineArgs::init( argc, argv, &aboutData );
   KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
@@ -351,8 +383,6 @@ int main(int argc, char *argv[]) {
     bool showQuickStart = false;
     bool nOK;
     int n_y = 0;
-    int n_eq = 0;
-
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
     CheckForCMDErrors(args);
@@ -361,6 +391,22 @@ int main(int argc, char *argv[]) {
     QString pngfile = args->getOption("png");
     bool print_and_exit = false;
 
+
+#if 0
+    // test code
+    KstAVectorPtr tmpav = new KstAVector(3, "tmpV" );
+    tmpav->value()[0] = 0;
+    tmpav->value()[1] = 10;
+    tmpav->value()[2] = 4;
+
+    KST::addVectorToList(KstVectorPtr(tmpav));
+
+    KstVectorPtr tmpv;
+    tmpv = new KstSVector(-5,5,100,"tmpV");
+    KST::addVectorToList( tmpv );
+#endif
+    // end test code...
+
     if (printfile != "<none>") {
       print_and_exit = true;
     }
@@ -376,18 +422,14 @@ int main(int argc, char *argv[]) {
       kst->show();
     }
 
-    //
     // get Y axis columns
-    //
     ycolList = args->getOptionList("y");
     yEqList = args->getOptionList("ye");
     psdList = args->getOptionList("p");
     hsList = args->getOptionList("h");
     errorList = args->getOptionList("e");
 
-    //
     // y axis or PSD specified, so the files are data files, not kst files.
-    //
     n_y = ycolList.count() + psdList.count() + hsList.count() + yEqList.count();
     if (n_y > 0) {
       QString creatingEquations = i18n("Creating equations");
@@ -395,7 +437,7 @@ int main(int argc, char *argv[]) {
       QString creatingPlots = i18n("Creating plots");
       int count;
       int handled;
-      
+
       kst->slotUpdateProgress( 0, 0, QString::null );
 
       SetCMDOptions(args, in, n_y);
@@ -403,15 +445,14 @@ int main(int argc, char *argv[]) {
       KstTopLevelViewPtr tlv = kst->activeView();
 
       if (!tlv) {
-        //
         // if there was no active view then we create one...
-        //
         kst->newWindow(false);
         tlv = kst->activeView();
       }
 
       if (!tlv) {
-        exit(0); // FIXME: can cause crashes
+        kdError() << i18n("Can't create a view.") << endl;
+        return 0;
       }
 
       CreatePlots(in, tlv);
@@ -420,16 +461,14 @@ int main(int argc, char *argv[]) {
       i_plot = 0;
       Kst2DPlotPtr plot = *plist.at(i_plot);
 
-      //
       // make stand alone equations if there are no files
-      //
       if (args->count() < 1) {
         if (!yEqList.isEmpty()) {
           QString eqS;
           double max, min;
           int n;
           bool xeq;
-          
+
           SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
           if (xeq) {
             count = yEqList.size();
@@ -439,9 +478,10 @@ int main(int argc, char *argv[]) {
             for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
               eqS = *eq_i;
               if (NoVectorEq(eqS)) {
-                KstEquationPtr eq = new KstEquation(QString("E") +QString::number(n_eq+1)+ "-" + eqS, eqS, min, max, n);
-                KstVCurvePtr vc = new KstVCurve(eq->tagName() + "-curve", eq->vX(), eq->vY(), 0L, 0L, KstColorSequence::next(plot->Curves,plot->backgroundColor()));
-
+                KstEquationPtr eq = new KstEquation(KST::suggestEQName(eqS), eqS, min, max, n);
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tagName(), true),
+                                        eq->vX(), eq->vY(), 0L, 0L, 0L, 0L,
+                                        KstColorSequence::next(plot->Curves,plot->backgroundColor()));
                 KST::dataObjectList.lock().writeLock();
                 KST::dataObjectList.append(eq.data());
                 KST::dataObjectList.append(vc.data());
@@ -455,205 +495,201 @@ int main(int argc, char *argv[]) {
               }
 
               handled++;
-              kst->slotUpdateProgress( count, handled, creatingEquations );            
+              kst->slotUpdateProgress( count, handled, creatingEquations );
             }
           }
         }
       }
 
-      //
       // make the requested curves for each data file
-      //
       count = args->count();
       handled = 0;
       kst->slotUpdateProgress( count, handled, creatingCurves );
 
       for (i_curve = i_v = 0, i_file = 0; i_file < args->count(); i_file++) {
-        //
         // make the file
-        //
         file = KstDataSource::loadSource(args->arg(i_file));
+        if (file) {
+          if (!file->isValid() || file->isEmpty()) {
+            kdError() << i18n("No data in file %1.  Trying to continue...").arg(args->arg(i_file)) << endl;
+            // The file might get data later!
+          }
 
-        if (!file) {
-          kdError() << i18n("No data in file: %1").arg(args->arg(i_file)) << endl;
-          delete kst;
-          exit(0);
-        }
+          KST::dataObjectList.lock().writeLock();
+          KST::dataSourceList.append(file);
+          KST::dataObjectList.lock().writeUnlock();
 
-        if (!file->isValid() || file->isEmpty()) {
-          kdError() << i18n("No data in file %1.  Trying to continue...").arg(args->arg(i_file)) << endl;
-          // The file might get data later!
-        }
+          KstRVectorPtr yvector;
+          KstRVectorPtr evector;
+          KstVCurvePtr curve;
+          KstPSDPtr psd;
+          KstHistogramPtr hs;
+          KstRVectorPtr xvector;
 
-        KST::dataObjectList.lock().writeLock();
-        KST::dataSourceList.append(file);
-        KST::dataObjectList.lock().writeUnlock();
+          if (!ycolList.isEmpty()) { // if there are some xy plots
+            // make the x axis vector
+            xvector = GetOrCreateVector(args->getOption("x"), file, in);
+            if (xvector) {
+              // make the y axis vectors
+              for (i_ycol = 0; i_ycol < ycolList.count(); ++i_ycol ) {
+                yvector = GetOrCreateVector(*(ycolList.at(i_ycol)), file, in);
+                if (yvector) {
+                  // make the curves
+                  color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
+                  curve = new KstVCurve(KST::suggestCurveName(yvector->field(), false),
+                                      KstVectorPtr(xvector), KstVectorPtr(yvector),
+                                      0L, 0L, 0L, 0L, color);
+                  if (in.has_points) {
+                    curve->setHasPoints(true);
+                    curve->setHasLines(false);
+                  }
 
-        KstRVectorPtr yvector;
-        KstRVectorPtr evector;
-        KstVCurvePtr curve;
-        KstPSDPtr psd;
-        KstHistogramPtr hs;
-        KstRVectorPtr xvector;
+                  if (i_ycol<errorList.count()) {
+                    evector = GetOrCreateVector(*(errorList.at(i_ycol)), file, in);
+                    if (evector) {
+                      curve->setYError(KstVectorPtr(evector));
+                    }
+                  }
 
-        if (!ycolList.isEmpty()) { // if there are some xy plots
-          //
-          // make the x axis vector
-          //
-          xvector = GetOrCreateVector(args->getOption("x"), file, in);
+                  KST::dataObjectList.lock().writeLock();
+                  KST::dataObjectList.append(curve.data());
+                  KST::dataObjectList.lock().writeUnlock();
+                  plot->addCurve(curve.data());
 
-          //
-          // make the y axis vectors
-          //
-          for (i_ycol = 0; i_ycol < ycolList.count(); ++i_ycol ) {
-            yvector = GetOrCreateVector(*(ycolList.at(i_ycol)), file, in);
-
-            //
-            // make the curves
-            //
-            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
-            curve = new KstVCurve(QString("C") + QString::number(1+i_curve++)
-                                  + "-" + yvector->field(),
-                                  KstVectorPtr(xvector), KstVectorPtr(yvector),
-                                  0L, 0L, color);
-            if (in.has_points) {
-              curve->setHasPoints(true);
-              curve->setHasLines(false);
+                  if (in.sep_plots) {
+                    plot->setTagName(curve->tagName());
+                    i_plot++;
+                    if (i_plot < in.n_plots) {
+                      plot = *plist.at(i_plot);
+                    }
+                  } // end (if they are separate plots)
+                }
+              } // next y col
             }
+          } // end (if there are some xy plots)
 
-            if (i_ycol<errorList.count()) {
-              evector = GetOrCreateVector(*(errorList.at(i_ycol)), file, in);
-              curve->setYError(KstVectorPtr(evector));
-            }
+          if (!yEqList.isEmpty()) {
+            QString eqS;
+            double max, min;
+            int n;
+            bool xeq, eq_ok;
 
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(curve.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(curve.data());
+            SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
+            for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
+              KstEquationPtr eq;
 
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot < in.n_plots) {
-                plot =  *plist.at(i_plot);
+              eqS = *eq_i;
+              ProcessEq(eqS, file, in, &eq_ok);
+
+              if (xeq) {
+                eq = new KstEquation(KST::suggestEQName(eqS), eqS, min,max,n);
+              } else {
+                if (!xvector) {
+                  xvector = GetOrCreateVector(args->getOption("x"), file, in);
+                }
+                if (xvector) {
+                  eq = new KstEquation(KST::suggestEQName(eqS), eqS, KstVectorPtr(xvector), true);
+                }
               }
-            } // end (if they are separate plots)
-          } // next y col
-        } // end (if there are some xy plots)
 
-        if (!yEqList.isEmpty()) {
-          QString eqS;
-          double max, min;
-          int n;
-          bool xeq, eq_ok;
+              if (eq) {
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(eq->tagName(), true),
+                                      eq->vX(), eq->vY(), 0L, 0L, 0L, 0L,
+                                      KstColorSequence::next(plot->Curves,plot->backgroundColor()));
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(eq.data());
+                KST::dataObjectList.append(vc.data());
+                KST::dataObjectList.lock().writeUnlock();
+                plot->addCurve(vc.data());
 
-          SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
-          for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
-            eqS = *eq_i;
-            ProcessEq(eqS, file, in, &eq_ok);
-            KstEquationPtr eq;
-            if (xeq) {
-              eq = new KstEquation(QString("E")+QString::number(n_eq+1)+
-                    "-" + eqS, eqS, min,max,n);
-            } else {
-              if (!xvector)
-                xvector = GetOrCreateVector(args->getOption("x"), file, in);
-
-              eq = new KstEquation(QString("E")+QString::number(n_eq+1)+eqS,
-                    eqS,
-                    KstVectorPtr(xvector),
-                    true);
-            }
-            KstVCurvePtr vc = new KstVCurve(eq->tagName() + "-curve", eq->vX(), eq->vY(), 0L, 0L, KstColorSequence::next(plot->Curves,plot->backgroundColor()));
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(eq.data());
-            KST::dataObjectList.append(vc.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(vc.data());
-
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot <in.n_plots) {
-                plot = *plist.at(i_plot);
+                if (in.sep_plots) {
+                  plot->setTagName(eq->tagName());
+                  i_plot++;
+                  if (i_plot <in.n_plots) {
+                    plot = *plist.at(i_plot);
+                  }
+                }
               }
             }
           }
+
+          if (psdList.count() > 0) { // if there are some psd plots
+            KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+            for (QCStringList::ConstIterator it = psdList.begin(); it != psdList.end(); ++it) {
+
+              yvector = GetOrCreateVector(*it, file, in);
+              if (yvector) {
+                color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
+
+                psd = new KstPSD( KST::suggestPSDName(yvector->field()),
+                    KstVectorPtr(yvector), in.rate, true, in.len,
+                    true, true, in.VUnits, in.RUnits);
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(psd->tagName(), true),
+                    psd->vX(), psd->vY(),
+                    0L, 0L, 0L, 0L, color);
+                if (in.has_points) {
+                  vc->setHasPoints(true);
+                  vc->setHasLines(false);
+                }
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(psd.data());
+                KST::dataObjectList.append(vc.data());
+                KST::dataObjectList.lock().writeUnlock();
+                plot->addCurve(vc.data());
+
+                if (in.sep_plots) {
+                  plot->setTagName(psd->tagName());
+                  i_plot++;
+                  if (i_plot <in.n_plots) {
+                    plot = *plist.at(i_plot);
+                  }
+                }
+              }
+            } // next psd
+          } // end (if there are some psds)
+
+          if (hsList.count() > 0) { // if there are some histograms
+            double max, min;
+            int N;
+
+            KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+            for (hs_string = hsList.begin(); hs_string != hsList.end(); ++hs_string) {
+              yvector = GetOrCreateVector(*hs_string, file, in);
+              if (yvector) {
+                color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
+
+                KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
+
+                hs = new KstHistogram(KST::suggestHistogramName(yvector->field()),
+                    KstVectorPtr(yvector), min, max, N, KST_HS_NUMBER);
+                KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(hs->tagName(), true),
+                    hs->vX(), hs->vY(),
+                    0L, 0L, 0L, 0L, color);
+
+                vc->setHasPoints(false);
+                vc->setHasLines(false);
+                vc->setHasBars(true);
+                vc->setBarStyle(1);
+
+                KST::dataObjectList.lock().writeLock();
+                KST::dataObjectList.append(hs.data());
+                KST::dataObjectList.append(vc.data());
+                KST::dataObjectList.lock().writeUnlock();
+                plot->addCurve(vc.data());
+
+                if (in.sep_plots) {
+                  plot->setTagName(hs->tagName());
+                  i_plot++;
+                  if (i_plot < in.n_plots) {
+                    plot = *plist.at(i_plot);
+                  }
+                }
+              }
+            } // next histogram
+          } // end (if there are some histograms)
+        } else {
+          strErrors.append(i18n("Failed to load file '%1'.").arg(args->arg(i_file)));
         }
-
-        if (psdList.count() > 0) { // if there are some psd plots
-          KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-          for (QCStringList::ConstIterator it = psdList.begin(); it != psdList.end(); ++it) {
-
-            yvector = GetOrCreateVector(*it, file, in);
-
-            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
-
-            psd = new KstPSD( QString("P") + QString::number(1+i_curve++) + "-" + yvector->field(),
-                KstVectorPtr(yvector), in.rate, true, in.len,
-                true, true, in.VUnits, in.RUnits);
-            KstVCurvePtr vc = new KstVCurve(psd->tagName() + "-curve",
-					    psd->vX(), psd->vY(),
-					    0L, 0L, color);
-            if (in.has_points) {
-              vc->setHasPoints(true);
-              vc->setHasLines(false);
-            }
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(psd.data());
-            KST::dataObjectList.append(vc.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(vc.data());
-
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot <in.n_plots) {
-                plot = *plist.at(i_plot);
-              }
-            }
-          } // next psd
-        } // end (if there are some psds)
-
-        if (hsList.count() > 0) { // if there are some histograms
-          double max, min;
-          int N;
-
-          KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-          for (hs_string = hsList.begin();
-	       hs_string != hsList.end(); ++hs_string ) {
-            yvector = GetOrCreateVector(*hs_string, file, in);
-
-            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
-
-            KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
-
-            hs = new KstHistogram(QString("H") +
-                QString::number(1+i_curve++)
-                + "-" + yvector->field(),
-                KstVectorPtr(yvector), min, max, N,
-                KST_HS_NUMBER);
-            KstVCurvePtr vc = new KstVCurve(hs->tagName() + "-curve",
-					    hs->vX(), hs->vY(),
-					    0L, 0L, color);
-
-            vc->setHasPoints(false);
-            vc->setHasLines(false);
-            vc->setHasBars(true);
-            vc->setBarStyle(1);
-
-            KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(hs.data());
-            KST::dataObjectList.append(vc.data());
-            KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(vc.data());
-
-            if (in.sep_plots) {
-              i_plot++;
-              if (i_plot < in.n_plots) {
-                plot = *plist.at(i_plot);
-              }
-            }
-          } // next histogram
-        } // end (if there are some histograms)
-
         handled++;
         kst->slotUpdateProgress( count, handled, creatingCurves );
       } // next data file
@@ -664,7 +700,7 @@ int main(int argc, char *argv[]) {
       for (i_plot = 0; i_plot < in.n_plots; i_plot++) {
         plot = *plist.at(i_plot);
         plot->GenerateDefaultLabels();
-        if (in.dolegend) {
+        if (plot->Curves.count() > 3 || in.dolegend) {
           plot->Legend->setShow(true);
           plot->Legend->setFront(true);
         }
@@ -680,12 +716,17 @@ int main(int argc, char *argv[]) {
           args->getOption("n").toInt(&nOK), // override number of frames
           args->getOption("f").toInt(&nOK), // override starting frame
           args->getOption("s").toInt(&nOK), // override skip
-          args->isSet("a"));                // add averaging
+          args->isSet("a"),                 // add averaging
+          !print_and_exit);                 // delayed
     } else {
       //kst->openDocumentFile();
       showQuickStart = true;
     }
 
+    if (args->isSet("nq")) {
+      showQuickStart = false;
+    }
+
     if (printfile != "<none>") {
       kst->forceUpdate();
       kst->immediatePrintToFile(printfile);
@@ -699,7 +740,7 @@ int main(int argc, char *argv[]) {
     if (print_and_exit) {
       args->clear();
       delete kst;
-      exit(0);
+      return 0;
     } else {
       args->clear();
       kst->updateDialogs();
@@ -707,15 +748,18 @@ int main(int argc, char *argv[]) {
       if (showQuickStart) {
         kst->showQuickStartDialog();
       }
+      for (size_t i=0; i<strErrors.size(); i++) {
+        KstDebug::self()->log(strErrors[i], KstDebug::Warning);
+      }
+      strErrors.clear();
     }
 
-    //
     // LEAVE THIS HERE - causes crashes otherwise!
-    //
     int rc = app.exec();
     delete kst;
     return rc;
   }
+
   return app.exec();
 }
 // vim: ts=2 sw=2 et
