@@ -22,6 +22,7 @@
 #include "pluginxmlparser.h"
 
 #include <kdebug.h>
+#include <kdeversion.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 
@@ -33,7 +34,7 @@ static KStaticDeleter<PluginCollection> _pcSelf;
 
 PluginCollection *PluginCollection::self() {
   if (!_self) {
-    _self = _pcSelf.setObject(new PluginCollection);
+    _pcSelf.setObject(_self, new PluginCollection);
   }
 
 return _self;
@@ -42,11 +43,9 @@ return _self;
 
 PluginCollection::PluginCollection() 
 : QObject(0L, "KST Plugin Collection") {
-  KGlobal::dirs()->addResourceType("kst", KStandardDirs::kde_default("data") + "kst");
   KGlobal::dirs()->addResourceType("kstplugins", KStandardDirs::kde_default("data") + "kst" + QDir::separator() + "plugins");
-  // KDE3 provides no way to get the plugin directory so this will break
-  // for KDE 4  ### FIXME
-  KGlobal::dirs()->addResourceType("kstpluginlib", KStandardDirs::kde_default("lib") + "kde3" + QDir::separator() + "kstplugins");
+  // KDE3 provides no way to get the plugin directory
+  KGlobal::dirs()->addResourceType("kstpluginlib", KStandardDirs::kde_default("lib") + QString("kde%1").arg(KDE_VERSION_MAJOR) + QDir::separator() + "kstplugins");
   _parser = new PluginXMLParser;
   scanPlugins();
 }
@@ -115,7 +114,7 @@ int PluginCollection::loadPlugin(const QString& xmlfile) {
     return 0;
   }
 
-return -2;
+  return -2;
 }
 
 
@@ -156,7 +155,8 @@ KstSharedPtr<Plugin> PluginCollection::plugin(const QString& name) {
     }
 
     if (_installedPluginNames.contains(name)) {
-      loadPlugin(_installedPluginNames[name]);
+      int rc = loadPlugin(_installedPluginNames[name]);
+      Q_UNUSED(rc)
 
       if (!_plugins.contains(name)) {
         return 0L;
@@ -222,8 +222,9 @@ void PluginCollection::scanPlugins() {
     QFileInfoListIterator fit(*list);
     QFileInfo *fi;
     int status;
-    
+
     while ((fi = fit.current()) != 0L) {
+      //kdDebug() << "Parsing [" << (*it + fi->fileName()) << "]" << endl;
       status = _parser->parseFile(*it + fi->fileName());
       if (status == 0) {
         // dupe? - prefer earlier installations
