@@ -33,7 +33,7 @@ int main() {
   int i, count = 0;
   float x;
 
-  sprintf(dirfilename, "%d.dm", time());
+  sprintf(dirfilename, "%d.dm", time(NULL));
 
   printf("Writing dirfile %s\n", dirfilename);
   printf("The fields are:\n");
